/***********************************************************************************************************
***   GROOVESIZER TB2 - Attack! - 6 Voice 12 Bit Drum Machine
***   for the Arduino DUE with TB2 shield http://groovesizer.com
***   May 2015
***   by MoShang (Jean Marais) - moshang@groovesizer.com
***   Licensed under a Creative Commons Attribution-ShareAlike 3.0
************************************************************************************************************/
int versionNumber = 56;

#include <DueTimer.h>
#include <LiquidCrystal.h>
#include <SdFat.h>
#include <MIDI.h>

// *** LED ***
int led = 13;

// *** LCD ***
// initialize the LCD library with the numbers of the interface pins
LiquidCrystal lcd(12, 11, 5, 4, 3, 2);

// *** BUTTONS ***
#define DEBOUNCE 10  // button debouncer, how many ms to debounce, 5+ ms is usually plenty
// here is where we define the buttons that we'll use. button "1" is the first, button "6" is the 6th, etc
byte buttons[] = {22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36}; // the analog 0-5 pins are also known as 14-19
// This handy macro lets us determine how big the array up above is, by checking the size
#define NUMBUTTONS sizeof(buttons)
// we will track if a button is just pressed, just released, or 'currently pressed'
byte pressed[NUMBUTTONS], justpressed[NUMBUTTONS], justreleased[NUMBUTTONS];

int whiteKey[8] = {0, 2, 4, 5, 7, 9, 11, 12};

boolean shiftL = false;
boolean shiftR = false;

int adjustVoice = 0;

int *adjustValue;
int increment = 0;
int upperLimit = 0;
int lowerLimit = 0;

// *** POTS ***
int pot[5]; // to store the values of out 5 pots
int potLock[5]; // to lock the pots when they're not being adjusted
unsigned long lockTimer = 0; // keep track of millis() to see if it's time to lock the pots
int lastPotValue[5]; // keep track of the last value so the pot doesn't lock while you're adjusting it

// *** SD CARD ***
// SD chip select pin
const uint8_t chipSelect = SS;
// file system object
SdFat sd;
SdFile file;

int16_t sdBuffer[5500] = {
};

char fileName[13];
char folderName[13];

boolean folder = false;
boolean inFolder = false;
int tempCount = 0;
int dirCount = 0;
int dirChoice = 0;
boolean dirChecked = true;
boolean justEnteredFolder = false;
int lastDirChoice = 0;
boolean voiceLoaded = false;
boolean kitLoaded = false;
boolean seqLoaded = false;
boolean saveConfirm = false;
int numberName = 0;
char saveName[13] = {" "};

boolean wrongFormat = false;

// *** POTS ***
int adjustStart = 0;
int adjustEnd = 0;
int adjustPan = 0;

// *** AUDIO ***
typedef struct Voices
{
  int increment = 512; // adjusts the playback speed of the sample - is multiplied by 2 before bitshiting >> 10 for effective range from 0 - 2
  int defaultIncrement = 512; // so we have a default value for unassigned values in the sequencer
  uint32_t index = 0; // a 23bit value - 10 bits for extra precision (increment), and 13 bits to represent a maximum index value of 5799
  uint16_t sampleIndex = 0;
  int sampleStart = 0;
  uint16_t tempSampleStart = 0; // needed to prevent noise when we adjust sample start
  int defaultSampleStart = 0; // so we have a default value for unassigned values in the sequencer
  int sampleEnd = 5499;
  int16_t wave[5500];
  boolean playing = false;
  boolean looping = false;
  int wavLength; // the maximum length of the current sample as read from SD
  int pan = 512; // a 10 bit value (0 - 1023) - 512 is centre
  int defaultLoseBits = 0; // how many bits to throw away in the bitmuncher (up to 11)
  int defaultVolume = 1023;
  int currentVolume = 1023;
  char shortName[9];
  byte nextFlam = 255; // range from 4 - 48 ticks,  255 = off
  int flamVelocity = 127;
  int flamTime = 12; // range from 2 - 95 ticks
  int flamDecay = 30;
  int standardVel = 80;
  int accentVel = 127;
  int velVolFactor = 1023; // 0 - 1023 user set velocity to volume
  int velSStFactor = 0; // 0 - 1023 user set velocity to sample start
  int velPitchFactor = 512; // 0 - 1023 user set velocity to pitch  - 512 is centre
  int velBitFactor = 0; // 0 - 1023 user set velocity to bit muncher
  int velLoseBits = 0; // number of bits after velocity modulation
} Voice;

Voice voice[6];

// *** SEQUENCER ***
typedef struct Sequences
{
  byte seq[6][32]; // 0 = off, 1 = standard velocity, 2 = accent velocity, 3 = flam standard velocity, 4 = flam accent velocity, 5 = placeholder standard, 6 = placeholder accent, 7 placeholder standard flam, placeholder accent flam
  byte startAuto[6][32];
  byte pitchAuto[6][32];
  boolean ssAuto[6]; // is sample start automation active?
  boolean pAuto[6]; // is pitch automation active?
  boolean editingAuto[6] = {false, false, false, false, false, false}; // are we editing automation?
  int seqLength = 32;
  int swing = 0;
  int interpolate = 1;
} Sequence;

Sequence sequence[8];

boolean seqRunning = false;

int bpm = 125;
byte playhead = 0;
byte realStep = 0;
byte currentSeq = 0;
byte cuedSeq = 0;
byte seqRow = 0;

boolean record = false;
byte targetStep = 0;
boolean beforeTargetStep = false;

int seqBpm = 125; // saved with a sequence bank for cases where TB2 is master

// *** 96PPQ CLOCK ***
int ticks = 0; // resets every 8th (48 ticks)
int flamTicks = 0; // resets every 1/4 notes (96 ticks)

boolean copied = false;
int sourceSeq = 0;
int destinationSeq = 0;

int bankMode = 0; // 0 = loop 1, 1 = loop 2, 2 = loop 4, 3 = loop 8, 4 = random,
const char loopMode[5] = {'1', '2', '4', '8', 'R'};

// *** GAIN ***
// uint16_t gainTable[4096]; // a lookup table to store post gain values

// *** WAVESHAPER ***
uint16_t waveShaper[4096]; // a lookup table to store waveshaped values
int waveShapeAmount = 2; // max 10 or 15?

// *** UI ***
boolean splash = true; // show the splash on bootup
boolean uiRefresh = false;
int mainMenu = 0; // which main mode are we in?
boolean arrowAnimation = false;
/*
0 = KIT
1 = SEQUENCER
2 = SETTINGS
*/
int lastMainMenu = 0;

/*
100 - KIT VOICE SETUP
101 - KIT LOAD VOICE
110 - KIT DEFAULTS
120 - KIT VELOCITY MODULATION
150 - KIT LOAD
160 - KIT SAVE
200 - SEQUENCE TRIGGER
210 - SEQUENCE EDIT
220 - SEQUENCE SETTINGS
250 - SEQUENCE BANK LOAD
260 - SEQUENCE BANK SAVE
300 - SETTINGS MIDI GENERAL
310 - SETTINGS MIDI VOICE
320 - SETTINGS TRIGGER MIDI
330 - SETTINGS SEQUENCER
*/
int menu = 0;
boolean valueChange = false;
const int kitMenu[4] = {0, 100, 110, 120};
const int seqMenu[4] = {0, 200, 210, 220};
const int settingsMenu[5] = {0, 300, 310, 320, 330};
int menuChoice = 0;
int menuPages = 0;
int lastMenu = 0;
unsigned long longPress = 0; // to keep track of millis() for long presses

int kitLoadSave = 0; // 0 or false = load, 1 or true = save
int seqLoadSave = 0; // 0 or false = load, 1 or true = save
boolean settingsConfirm = false;

boolean justSavedBank = false;
boolean justSavedKit = false;

// *** MIDI ***
MIDI_CREATE_INSTANCE(HardwareSerial, Serial1, midiA);

boolean receivingClock = false; // are we receiving MIDI clock?
unsigned long lastClock = 0;
boolean seqJustStarted = false;
int midiThruType = 2; // 0 = off, 1 = clock only, 2 = all
int midiSync = 0; // 0 = none, 1 = master, 2 = slave
boolean syncIn = true;
unsigned long inPulseCounter = 0;
unsigned long clockInMicros = 0; // the time in microseconds when the pulse arrived
int tmpBpm[24];
int pulseMicros = 0; // the duration of a pulse in microseconds
int clockInBpm = 0; // the average of the last 4 BPM measurements
int clockOutCounter = 0;
int midiClockOut = 1; // are we sending 24ppq clock pulses (clock master)
int midiChannel = 10; // which channel are we sending data on? 0 is off
byte midiVelocity = 0; // the last received MIDI velocity
int voiceNoteNumber[6] = {36, 38, 42, 46, 54, 49};

int midiTriggerOut = 1;
int midiTriggerChannel = 1;
int midiTrigger[8] = {60, 61, 62, 63, 64, 65, 66, 67};
int editTrigger = 0;

// *** METRONOME ***
boolean metronome = false;
byte metroIndex = 255;
byte metroIncrement = 1;
int metroCounter = 0;


const int metroClick[239] = {
  0, 22, 65, 129, 161, 194, 226, 258, 242, 242, 242, 194, 129, 48, -65, -210, -387, -548, -726, -855, -935, -984, -1000,
  -935, -806, -613, -403, -129, 177, 484, 790, 1048, 1242, 1387, 1435, 1419, 1306, 1113, 855, 548, 226, -161, -548,
  -887, -1226, -1451, -1629, -1661, -1613, -1419, -1161, -855, -484, -48, 387, 806, 1177, 1435, 1629, 1709, 1629,
  1467, 1177, 790, 355, -113, -581, -1000, -1371, -1645, -1806, -1806, -1709, -1484, -1129, -677, -194, 355, 855,
  1274, 1629, 1871, 2000, 2000, 1887, 1629, 1258, 790, 290, -242, -758, -1209, -1596, -1871, -2016, -2032, -1919,
  -1693, -1338, -887, -387, 145, 677, 1145, 1532, 1838, 2000, 2048, 1951, 1725, 1371, 935, 435, -97, -613, -1080,
  -1500, -1790, -1967, -2000, -1919, -1693, -1355, -935, -435, 65, 564, 1032, 1419, 1693, 1871, 1887, 1774, 1548,
  1209, 822, 371, -113, -564, -984, -1322, -1548, -1661, -1661, -1516, -1290, -968, -581, -145, 290, 710, 1048, 1306,
  1484, 1548, 1484, 1322, 1080, 726, 355, -32, -435, -774, -1080, -1290, -1387, -1387, -1290, -1113, -839, -516,
  -161, 210, 532, 822, 1048, 1193, 1242, 1193, 1064, 871, 597, 290, -32, -355, -645, -887, -1032, -1129, -1113, -1048,
  -903, -710, -468, -194, 81, 339, 564, 742, 855, 903, 871, 790, 661, 484, 290, 81, -129, -306, -468, -581, -645, -645,
  -629, -548, -435, -306, -145, 0, 145, 258, 355, 419, 435, 435, 419, 339, 242, 161, 65, -16, -113, -177, -226, -258,
  -242, -226, -89, 0,
};


// *** GRAIN FX ***
uint8_t grainIndexL = 0; // keep a byte so we can just increment and not worry about resetting to 0
uint16_t grainIndexR = 256;
int grainPitch = 255; // adjusts the playback speed of the grain - is multiplied by 4 before bitshifting >> 10 for effective range from 0 - 4
uint32_t grainAcc = 0;

boolean grainPlay = false;
boolean grainRhythm = false;
boolean grainRhythm2 = false;
boolean grabBuffer = false;


// *** BEAT REPEAT ***
boolean beatRepeat = false;


// *** FLANGE FX ***
boolean flanger = false;
uint8_t flangeIndex = 0;  // keep it a byte so we can just increment and not worry about resetting to 0
uint16_t flangeSpeed = 300;
uint8_t flangeDelay = 20; // range from 22 (1ms) to 100
const uint16_t flangeOffsetL = 512;
const uint16_t flangeOffsetR = 768;

// *** SETTINGS ***
int autoChannel[6] = {0, 0, 0, 0, 0, 0}; // the channel number to send automation on - 0 for off
int autoCC[6] = {0, 0, 0, 0, 0, 0}; // the controller number to send to - 44 is filter cutoff on Korg Volca Keys
int autoVal[6] = {0, 0, 0, 0, 0, 0}; // the value to send

int recallKit = true; // load kit when loading a sequence bank
int recallTempo = true; // change to the bpm saved with the sequence bank
int settingsVoice = 0; // which voice are we editing on settings page 310?

int midiOut = true; // does the TB2 send out MIDI notes from its front panel keyboard and sequencer?

// *** DEFAULT KIT ***
const int16_t KICK[5500] = {2048, 2056, 2068, 2088, 2112, 2143, 2179, 2223, 2270, 2326, 2384, 2448, 2511, 2589, 2665, 2755, 2806, 2833, 2844, 2818, 2811, 2787, 2765, 2746, 2730, 2708, 2695, 2665, 2647, 2625, 2611, 2587, 2583, 2556, 2534, 2517, 2489, 2469, 2437, 2416, 2388, 2375, 2348, 2334, 2319, 2332, 2383, 2404, 2391, 2401, 2381, 2357, 2296, 2256, 2238, 2227, 2209, 2168, 2115, 2071, 2062, 2090, 2107, 2077, 2066, 2032, 1977, 1916, 1847, 1777, 1725, 1679, 1604, 1548, 1490, 1443, 1395, 1343, 1316, 1285, 1243, 1199, 1177, 1156, 1122, 1108, 1093, 1087, 1068, 1068, 1055, 1028, 980, 988, 1119, 1076, 991, 940, 1075, 1203, 1256, 1271, 1193, 1054, 895, 769, 700, 686, 698, 789, 784, 752, 745, 803, 911, 1017, 1110, 1189, 1252, 1347, 1424, 1520, 1581, 1706, 1893, 2016, 2057, 2039, 2030, 2103, 2231, 2367, 2522, 2628, 2662, 2683, 2687, 2705, 2767, 2787, 2870, 2974, 3055, 3088, 3135, 3167, 3170, 3194, 3234, 3270, 3275, 3296, 3314, 3308, 3263, 3099, 2973, 2915, 2916, 2961, 3058, 3220, 3432, 3424, 3159, 2935, 2647, 2349, 2222, 2399, 2594, 2630, 2517, 2341, 2176, 1946, 1923, 1930, 1944, 2017, 2047, 2109, 2014, 1861, 1767, 1667, 1567, 1462, 1332, 1256, 1190, 1105, 1109, 1064, 1001, 941, 858, 812, 797, 803, 718, 759, 832, 887, 782, 714, 730, 738, 800, 869, 968, 919, 852, 869, 830, 844, 905, 941, 931, 901, 889, 838, 823, 901, 951, 950, 964, 934, 914, 882, 974, 1028, 991, 1027, 1029, 1065, 1180, 1245, 1293, 1384, 1455, 1495, 1513, 1525, 1615, 1689, 1781, 1899, 2018, 2069, 2170, 2229, 2326, 2396, 2407, 2466, 2500, 2608, 2731, 2826, 2946, 3022, 3025, 3119, 3303, 3299, 3257, 3433, 3545, 3661, 3669, 3706, 3772, 3745, 3766, 3860, 3895, 3898, 3905, 3854, 3830, 3803, 3828, 3846, 3808, 3794, 3856, 3878, 3819, 3840, 3882, 3830, 3830, 3840, 3749, 3748, 3821, 3831, 3802, 3761, 3755, 3806, 3792, 3795, 3742, 3750, 3725, 3700, 3687, 3680, 3586, 3570, 3552, 3555, 3526, 3500, 3456, 3362, 3335, 3444, 3382, 3225, 3392, 3342, 3291, 3394, 3198, 3217, 3205, 3196, 3222, 3004, 3145, 3046, 3087, 3021, 2816, 2902, 3036, 2826, 2690, 2727, 2605, 2557, 2484, 2398, 2444, 2336, 2310, 2243, 2180, 2156, 2139, 1950, 1822, 1956, 1959, 1711, 1737, 1681, 1656, 1558, 1580, 1524, 1426, 1438, 1416, 1446, 1256, 1236, 1218, 947, 974, 1198, 758, 881, 894, 727, 626, 516, 529, 456, 389, 428, 429, 248, 360, 350, 211, 365, 307, 246, 188, 278, 101, 169, 203, 129, 191, 262, 105, 85, 335, 131, 187, 296, 176, 167, 198, 321, 263, 156, 329, 272, 291, 284, 237, 265, 397, 325, 225, 323, 335, 434, 311, 360, 457, 367, 466, 522, 393, 452, 526, 457, 540, 573, 545, 599, 615, 633, 666, 554, 625, 628, 642, 667, 726, 718, 753, 752, 732, 717, 844, 846, 930, 1008, 962, 1023, 1113, 1093, 1060, 1162, 1202, 1234, 1325, 1240, 1391, 1379, 1357, 1535, 1347, 1553, 1445, 1644, 1595, 1625, 1667, 1680, 1842, 1710, 1882, 1761, 1979, 1991, 1907, 2128, 1954, 2152, 2082, 2080, 2144, 2045, 2257, 2208, 2279, 2332, 2329, 2510, 2401, 2431, 2557, 2473, 2643, 2590, 2679, 2603, 2718, 2697, 2731, 2741, 2661, 2833, 2681, 2844, 2819, 2805, 2849, 2833, 2996, 2786, 2919, 3067, 2907, 2926, 3048, 3145, 3032, 3152, 3069, 3111, 3088, 3149, 3163, 3165, 3157, 3107, 3135, 3277, 3168, 3169, 3286, 3215, 3250, 3337, 3294, 3395, 3368, 3336, 3429, 3336, 3449, 3486, 3413, 3474, 3489, 3468, 3572, 3480, 3527, 3444, 3576, 3455, 3472, 3446, 3478, 3534, 3473, 3476, 3445, 3526, 3481, 3505, 3472, 3603, 3426, 3474, 3530, 3332, 3426, 3417, 3287, 3269, 3279, 3250, 3246, 3161, 3223, 3063, 3149, 3102, 3043, 3087, 3064, 3061, 3016, 2998, 3004, 2968, 2944, 2979, 2844, 2877, 2793, 2761, 2677, 2715, 2663, 2597, 2600, 2558, 2620, 2560, 2546, 2577, 2494, 2423, 2463, 2388, 2474, 2435, 2358, 2257, 2334, 2254, 2227, 2198, 2052, 2146, 2058, 1993, 1985, 2071, 1934, 1928, 1876, 1939, 1852, 1889, 1885, 1780, 1867, 1750, 1731, 1734, 1739, 1607, 1652, 1666, 1645, 1553, 1472, 1614, 1548, 1513, 1426, 1566, 1415, 1430, 1465, 1289, 1427, 1302, 1299, 1354, 1338, 1284, 1338, 1277, 1219, 1188, 1207, 1307, 1161, 1165, 1245, 1072, 1182, 1181, 1090, 1067, 1063, 1093, 1065, 1065, 1037, 1057, 997, 1049, 1015, 970, 949, 990, 989, 956, 882, 972, 905, 887, 901, 879, 940, 919, 914, 869, 904, 918, 890, 926, 955, 798, 931, 993, 876, 901, 882, 967, 1027, 907, 988, 963, 951, 1043, 1063, 1029, 998, 999, 1054, 1021, 1042, 1118, 1040, 1083, 1085, 1078, 1140, 1090, 1070, 1102, 1099, 1150, 1094, 1114, 1143, 1171, 1196, 1200, 1229, 1226, 1248, 1287, 1289, 1329, 1302, 1265, 1388, 1389, 1348, 1361, 1447, 1406, 1376, 1459, 1498, 1551, 1572, 1549, 1652, 1591, 1623, 1788, 1662, 1745, 1727, 1745, 1808, 1770, 1839, 1832, 1816, 1939, 1876, 1957, 1990, 1965, 2068, 2051, 2110, 2037, 2043, 2148, 2172, 2093, 2166, 2225, 2245, 2267, 2289, 2347, 2395, 2360, 2379, 2476, 2504, 2477, 2505, 2559, 2536, 2532, 2650, 2610, 2629, 2710, 2630, 2720, 2701, 2737, 2808, 2729, 2811, 2827, 2797, 2854, 2851, 2807, 2861, 2816, 2747, 2867, 2820, 2788, 2826, 2817, 2838, 2769, 2812, 2849, 2805, 2824, 2806, 2842, 2815, 2811, 2761, 2823, 2749, 2700, 2801, 2706, 2745, 2741, 2682, 2703, 2776, 2699, 2684, 2717, 2742, 2766, 2705, 2712, 2752, 2752, 2711, 2731, 2710, 2723, 2762, 2750, 2705, 2739, 2728, 2757, 2742, 2752, 2758, 2767, 2746, 2788, 2788, 2761, 2825, 2755, 2893, 2762, 2761, 2819, 2770, 2778, 2830, 2771, 2672, 2846, 2789, 2759, 2795, 2799, 2773, 2807, 2844, 2773, 2848, 2913, 2843, 2913, 2891, 2866, 2895, 2981, 2964, 2889, 3032, 2941, 2985, 2995, 2951, 3025, 2958, 2958, 2898, 2992, 2924, 2906, 2952, 2903, 2902, 2873, 2939, 2873, 2882, 2848, 2850, 2798, 2856, 2860, 2838, 2824, 2787, 2812, 2766, 2801, 2711, 2671, 2721, 2694, 2652, 2621, 2612, 2604, 2570, 2569, 2559, 2485, 2472, 2490, 2393, 2427, 2384, 2307, 2287, 2227, 2229, 2147, 2178, 2084, 2061, 2095, 2055, 2008, 1988, 1974, 1944, 1997, 1922, 1891, 1869, 1828, 1807, 1806, 1754, 1741, 1746, 1679, 1645, 1720, 1684, 1609, 1607, 1583, 1591, 1570, 1561, 1537, 1500, 1471, 1464, 1465, 1417, 1416, 1388, 1355, 1386, 1365, 1308, 1262, 1259, 1257, 1243, 1241, 1170, 1139, 1161, 1156, 1105, 1081, 1096, 1050, 1108, 1071, 1032, 1041, 1037, 1049, 991, 980, 1004, 946, 961, 971, 931, 934, 884, 898, 884, 840, 894, 869, 804, 835, 835, 851, 836, 872, 857, 842, 885, 866, 892, 878, 903, 872, 911, 909, 908, 959, 908, 918, 955, 943, 921, 1003, 946, 954, 1004, 1000, 1019, 1031, 1019, 1015, 1062, 1062, 1029, 1041, 1066, 1051, 1077, 1096, 1114, 1077, 1100, 1106, 1142, 1140, 1116, 1132, 1115, 1135, 1164, 1155, 1145, 1187, 1155, 1158, 1229, 1187, 1178, 1230, 1208, 1249, 1261, 1269, 1249, 1297, 1308, 1326, 1331, 1294, 1335, 1335, 1377, 1380, 1352, 1375, 1364, 1395, 1421, 1408, 1426, 1435, 1402, 1444, 1447, 1457, 1481, 1471, 1472, 1509, 1521, 1509, 1542, 1598, 1634, 1608, 1678, 1699, 1743, 1789, 1818, 1853, 1899, 1896, 1973, 1999, 2005, 2075, 2097, 2141, 2148, 2178, 2190, 2281, 2298, 2317, 2361, 2368, 2444, 2429, 2515, 2521, 2540, 2581, 2602, 2644, 2681, 2720, 2720, 2768, 2782, 2834, 2839, 2849, 2913, 2924, 3000, 2962, 2992, 3023, 3019, 3061, 3053, 3092, 3115, 3175, 3120, 3185, 3215, 3232, 3249, 3258, 3286, 3326, 3351, 3350, 3364, 3350, 3352, 3366, 3366, 3346, 3332, 3340, 3350, 3332, 3327, 3317, 3329, 3331, 3280, 3295, 3321, 3284, 3291, 3293, 3284, 3279, 3268, 3255, 3258, 3270, 3225, 3212, 3239, 3232, 3236, 3195, 3204, 3206, 3197, 3196, 3174, 3148, 3154, 3188, 3143, 3160, 3130, 3110, 3144, 3107, 3102, 3086, 3105, 3110, 3069, 3064, 3078, 3047, 3023, 3056, 3030, 3029, 3013, 2990, 3020, 2976, 2978, 2996, 2964, 2959, 2966, 2953, 2932, 2918, 2945, 2919, 2898, 2914, 2880, 2871, 2868, 2857, 2856, 2869, 2820, 2814, 2825, 2809, 2797, 2759, 2794, 2801, 2768, 2763, 2750, 2741, 2739, 2739, 2718, 2696, 2683, 2681, 2699, 2682, 2642, 2643, 2624, 2578, 2539, 2517, 2492, 2438, 2419, 2401, 2372, 2317, 2302, 2266, 2225, 2204, 2156, 2129, 2070, 2032, 2011, 1983, 1930, 1897, 1881, 1830, 1792, 1745, 1724, 1691, 1660, 1646, 1592, 1568, 1547, 1519, 1499, 1478, 1450, 1427, 1402, 1370, 1338, 1322, 1298, 1242, 1229, 1204, 1178, 1159, 1146, 1103, 1077, 1088, 1040, 1035, 995, 970, 974, 970, 947, 909, 897, 883, 878, 827, 815, 800, 796, 780, 751, 758, 749, 746, 735, 707, 701, 700, 683, 665, 641, 650, 656, 649, 627, 635, 635, 651, 650, 643, 658, 645, 659, 659, 670, 661, 674, 698, 691, 684, 691, 704, 716, 714, 722, 728, 744, 757, 775, 763, 755, 792, 801, 805, 813, 824, 842, 837, 830, 857, 855, 874, 872, 886, 880, 883, 921, 923, 930, 948, 946, 956, 985, 976, 991, 1020, 1016, 1030, 1035, 1065, 1080, 1078, 1115, 1117, 1110, 1140, 1163, 1156, 1173, 1191, 1209, 1234, 1242, 1250, 1267, 1270, 1284, 1297, 1322, 1333, 1322, 1341, 1377, 1398, 1427, 1462, 1493, 1521, 1544, 1567, 1607, 1635, 1673, 1688, 1725, 1760, 1777, 1821, 1842, 1879, 1904, 1935, 1965, 1990, 2007, 2051, 2092, 2112, 2156, 2188, 2230, 2257, 2303, 2346, 2361, 2391, 2429, 2454, 2479, 2523, 2553, 2568, 2588, 2628, 2648, 2662, 2689, 2712, 2728, 2757, 2781, 2814, 2832, 2866, 2898, 2910, 2938, 2958, 2975, 3017, 3036, 3038, 3084, 3110, 3138, 3150, 3158, 3182, 3211, 3220, 3231, 3237, 3268, 3296, 3290, 3316, 3313, 3346, 3363, 3347, 3375, 3366, 3372, 3383, 3390, 3393, 3405, 3410, 3418, 3416, 3409, 3426, 3421, 3435, 3440, 3419, 3430, 3448, 3442, 3428, 3420, 3442, 3440, 3430, 3430, 3430, 3412, 3431, 3428, 3418, 3415, 3415, 3419, 3427, 3417, 3396, 3408, 3394, 3397, 3392, 3384, 3371, 3353, 3353, 3341, 3318, 3319, 3304, 3292, 3288, 3268, 3257, 3256, 3236, 3230, 3225, 3206, 3191, 3181, 3168, 3147, 3141, 3128, 3108, 3085, 3070, 3041, 3031, 3016, 2990, 2970, 2950, 2943, 2927, 2905, 2886, 2861, 2849, 2833, 2812, 2793, 2774, 2761, 2737, 2713, 2701, 2665, 2643, 2621, 2602, 2591, 2571, 2533, 2503, 2491, 2467, 2458, 2423, 2398, 2363, 2337, 2328, 2294, 2260, 2246, 2212, 2176, 2142, 2109, 2094, 2063, 2029, 2008, 1977, 1957, 1937, 1908, 1883, 1865, 1858, 1850, 1823, 1806, 1794, 1766, 1759, 1734, 1707, 1694, 1678, 1653, 1639, 1620, 1593, 1575, 1548, 1527, 1518, 1506, 1487, 1462, 1449, 1429, 1408, 1396, 1381, 1362, 1354, 1345, 1330, 1307, 1278, 1274, 1258, 1247, 1232, 1218, 1201, 1186, 1191, 1180, 1161, 1151, 1138, 1132, 1119, 1108, 1082, 1070, 1067, 1062, 1050, 1034, 1028, 1010, 1003, 996, 995, 991, 974, 977, 972, 958, 955, 949, 937, 929, 933, 936, 950, 945, 945, 938, 943, 945, 940, 942, 953, 955, 953, 962, 966, 977, 976, 976, 981, 988, 1003, 1013, 1012, 1028, 1032, 1042, 1051, 1062, 1065, 1067, 1088, 1103, 1115, 1122, 1135, 1148, 1168, 1191, 1196, 1208, 1230, 1247, 1264, 1269, 1282, 1301, 1328, 1348, 1359, 1372, 1385, 1414, 1437, 1454, 1470, 1490, 1504, 1520, 1542, 1559, 1584, 1602, 1620, 1630, 1649, 1675, 1688, 1703, 1715, 1734, 1754, 1771, 1774, 1797, 1821, 1836, 1859, 1874, 1895, 1914, 1938, 1956, 1969, 1985, 2008, 2028, 2047, 2062, 2072, 2098, 2115, 2131, 2147, 2156, 2176, 2189, 2202, 2219, 2229, 2250, 2268, 2280, 2295, 2313, 2325, 2344, 2353, 2366, 2386, 2402, 2421, 2426, 2432, 2447, 2461, 2465, 2479, 2479, 2489, 2505, 2512, 2517, 2523, 2532, 2542, 2552, 2555, 2562, 2575, 2587, 2591, 2598, 2609, 2611, 2618, 2627, 2633, 2636, 2641, 2647, 2661, 2671, 2671, 2671, 2676, 2679, 2683, 2687, 2694, 2699, 2697, 2706, 2705, 2709, 2703, 2707, 2715, 2715, 2712, 2709, 2711, 2707, 2704, 2693, 2687, 2683, 2677, 2674, 2672, 2664, 2663, 2665, 2660, 2658, 2655, 2656, 2655, 2649, 2643, 2632, 2632, 2630, 2624, 2619, 2609, 2609, 2600, 2594, 2593, 2590, 2587, 2588, 2589, 2590, 2583, 2578, 2574, 2567, 2569, 2561, 2556, 2547, 2535, 2532, 2527, 2517, 2506, 2501, 2492, 2489, 2476, 2466, 2462, 2452, 2446, 2437, 2426, 2422, 2417, 2409, 2394, 2385, 2375, 2369, 2364, 2358, 2349, 2339, 2347, 2335, 2328, 2321, 2313, 2306, 2293, 2283, 2270, 2261, 2250, 2237, 2227, 2217, 2203, 2191, 2185, 2173, 2163, 2160, 2145, 2134, 2132, 2124, 2113, 2103, 2096, 2085, 2076, 2068, 2053, 2035, 2021, 2012, 2004, 1993, 1980, 1969, 1960, 1951, 1935, 1926, 1911, 1904, 1896, 1884, 1872, 1864, 1861, 1855, 1849, 1842, 1834, 1827, 1820, 1807, 1789, 1781, 1776, 1762, 1747, 1739, 1727, 1717, 1708, 1700, 1692, 1685, 1682, 1675, 1670, 1665, 1658, 1654, 1649, 1641, 1639, 1626, 1618, 1615, 1609, 1599, 1588, 1577, 1569, 1562, 1561, 1556, 1556, 1556, 1552, 1554, 1548, 1549, 1550, 1551, 1551, 1549, 1553, 1550, 1544, 1545, 1549, 1553, 1553, 1550, 1552, 1555, 1558, 1564, 1562, 1563, 1570, 1574, 1577, 1579, 1582, 1586, 1595, 1599, 1604, 1613, 1617, 1625, 1623, 1629, 1634, 1641, 1644, 1644, 1647, 1653, 1663, 1669, 1677, 1679, 1682, 1685, 1696, 1699, 1706, 1717, 1724, 1736, 1745, 1756, 1764, 1770, 1779, 1783, 1790, 1794, 1801, 1807, 1812, 1820, 1825, 1836, 1846, 1855, 1866, 1876, 1881, 1888, 1895, 1906, 1916, 1926, 1929, 1934, 1948, 1956, 1965, 1977, 1982, 1984, 1988, 1995, 2003, 2013, 2020, 2023, 2028, 2032, 2034, 2038, 2040, 2045, 2052, 2055, 2060, 2070, 2073, 2078, 2089, 2096, 2105, 2107, 2113, 2122, 2133, 2141, 2152, 2161, 2163, 2169, 2173, 2177, 2179, 2186, 2189, 2194, 2200, 2205, 2204, 2205, 2207, 2209, 2213, 2215, 2216, 2219, 2221, 2221, 2228, 2229, 2232, 2233, 2232, 2235, 2236, 2240, 2241, 2242, 2242, 2244, 2248, 2252, 2252, 2256, 2263, 2269, 2275, 2276, 2278, 2282, 2287, 2287, 2287, 2289, 2291, 2290, 2294, 2297, 2300, 2297, 2297, 2295, 2293, 2293, 2294, 2293, 2292, 2291, 2295, 2295, 2292, 2297, 2297, 2299, 2302, 2299, 2293, 2292, 2293, 2295, 2295, 2288, 2284, 2277, 2273, 2271, 2270, 2268, 2263, 2258, 2255, 2256, 2253, 2252, 2250, 2246, 2242, 2235, 2235, 2236, 2232, 2226, 2224, 2223, 2221, 2222, 2222, 2223, 2219, 2218, 2219, 2220, 2221, 2223, 2224, 2223, 2223, 2215, 2212, 2210, 2206, 2206, 2205, 2204, 2201, 2198, 2192, 2188, 2184, 2179, 2174, 2171, 2167, 2158, 2153, 2150, 2152, 2151, 2146, 2142, 2136, 2133, 2130, 2128, 2123, 2122, 2121, 2115, 2113, 2107, 2105, 2104, 2101, 2096, 2090, 2088, 2082, 2078, 2074, 2074, 2064, 2058, 2056, 2054, 2051, 2045, 2045, 2040, 2038, 2033, 2028, 2025, 2024, 2019, 2013, 2009, 2004, 1999, 2000, 1999, 1991, 1986, 1979, 1972, 1965, 1960, 1958, 1954, 1948, 1942, 1938, 1935, 1933, 1931, 1927, 1925, 1920, 1913, 1907, 1900, 1896, 1893, 1889, 1883, 1878, 1876, 1873, 1868, 1860, 1858, 1858, 1853, 1852, 1852, 1851, 1847, 1846, 1844, 1842, 1840, 1836, 1834, 1830, 1833, 1835, 1833, 1829, 1830, 1830, 1830, 1829, 1827, 1828, 1826, 1828, 1828, 1828, 1829, 1826, 1826, 1829, 1834, 1834, 1833, 1837, 1842, 1845, 1848, 1853, 1855, 1859, 1863, 1866, 1868, 1868, 1868, 1871, 1875, 1881, 1888, 1894, 1899, 1903, 1909, 1912, 1911, 1912, 1912, 1916, 1921, 1924, 1929, 1933, 1937, 1941, 1942, 1944, 1949, 1954, 1959, 1964, 1969, 1974, 1975, 1978, 1983, 1987, 1991, 1991, 1991, 1994, 1999, 2001, 2005, 2012, 2018, 2020, 2025, 2029, 2030, 2033, 2035, 2038, 2039, 2044, 2045, 2044, 2045, 2045, 2048, 2049, 2051, 2055, 2057, 2062, 2066, 2069, 2075, 2075, 2080, 2085, 2090, 2093, 2097, 2098, 2095, 2098, 2102, 2103, 2104, 2107, 2107, 2111, 2115, 2115, 2115, 2113, 2115, 2117, 2119, 2122, 2124, 2126, 2126, 2129, 2132, 2135, 2139, 2141, 2144, 2148, 2151, 2154, 2156, 2158, 2161, 2168, 2174, 2178, 2180, 2182, 2186, 2188, 2189, 2190, 2193, 2195, 2197, 2194, 2193, 2192, 2193, 2194, 2192, 2192, 2189, 2189, 2191, 2191, 2191, 2191, 2192, 2197, 2198, 2199, 2199, 2199, 2201, 2203, 2207, 2208, 2209, 2210, 2212, 2211, 2211, 2214, 2217, 2216, 2215, 2211, 2208, 2210, 2208, 2203, 2200, 2200, 2200, 2198, 2196, 2195, 2192, 2193, 2194, 2192, 2191, 2188, 2187, 2184, 2182, 2183, 2183, 2179, 2178, 2175, 2174, 2175, 2175, 2176, 2173, 2170, 2167, 2166, 2164, 2164, 2163, 2159, 2159, 2157, 2153, 2148, 2144, 2141, 2137, 2135, 2132, 2129, 2125, 2123, 2118, 2115, 2112, 2111, 2106, 2103, 2098, 2093, 2092, 2087, 2084, 2081, 2079, 2076, 2072, 2071, 2069, 2067, 2064, 2058, 2054, 2051, 2049, 2044, 2039, 2033, 2030, 2026, 2022, 2018, 2013, 2010, 2008, 2002, 1998, 1994, 1989, 1988, 1986, 1985, 1984, 1983, 1980, 1974, 1969, 1967, 1966, 1964, 1962, 1958, 1954, 1949, 1944, 1941, 1940, 1938, 1935, 1934, 1929, 1924, 1921, 1919, 1917, 1916, 1911, 1906, 1904, 1900, 1899, 1898, 1898, 1897, 1894, 1896, 1896, 1895, 1895, 1893, 1894, 1894, 1892, 1890, 1890, 1888, 1886, 1886, 1886, 1886, 1885, 1884, 1884, 1883, 1879, 1878, 1876, 1874, 1875, 1878, 1878, 1876, 1877, 1877, 1879, 1881, 1883, 1885, 1888, 1890, 1892, 1893, 1895, 1897, 1898, 1901, 1901, 1904, 1909, 1913, 1915, 1920, 1923, 1926, 1928, 1931, 1934, 1938, 1939, 1939, 1938, 1939, 1942, 1943, 1945, 1945, 1948, 1950, 1952, 1952, 1954, 1958, 1961, 1966, 1971, 1977, 1980, 1981, 1985, 1987, 1991, 1993, 1997, 2000, 2001, 2004, 2006, 2010, 2011, 2013, 2017, 2019, 2019, 2022, 2023, 2023, 2021, 2022, 2025, 2027, 2027, 2026, 2028, 2030, 2028, 2029, 2031, 2032, 2037, 2041, 2043, 2044, 2049, 2052, 2054, 2056, 2057, 2060, 2063, 2064, 2064, 2066, 2066, 2068, 2068, 2068, 2070, 2073, 2076, 2077, 2078, 2077, 2076, 2075, 2074, 2073, 2075, 2075, 2074, 2072, 2070, 2071, 2073, 2074, 2076, 2078, 2082, 2085, 2089, 2092, 2094, 2098, 2099, 2101, 2100, 2101, 2104, 2105, 2107, 2109, 2110, 2111, 2113, 2112, 2114, 2112, 2113, 2114, 2115, 2116, 2117, 2121, 2124, 2125, 2127, 2129, 2130, 2133, 2136, 2138, 2141, 2144, 2146, 2149, 2149, 2150, 2153, 2156, 2157, 2158, 2161, 2162, 2165, 2168, 2171, 2174, 2178, 2180, 2183, 2186, 2189, 2189, 2189, 2188, 2188, 2190, 2188, 2188, 2187, 2188, 2186, 2186, 2185, 2183, 2184, 2185, 2187, 2188, 2186, 2184, 2182, 2181, 2180, 2179, 2176, 2174, 2171, 2170, 2170, 2169, 2170, 2169, 2169, 2165, 2163, 2161, 2158, 2157, 2154, 2151, 2149, 2150, 2149, 2146, 2142, 2140, 2137, 2135, 2134, 2131, 2128, 2123, 2119, 2117, 2115, 2113, 2110, 2110, 2108, 2106, 2102, 2099, 2097, 2093, 2090, 2087, 2086, 2083, 2083, 2081, 2078, 2074, 2073, 2072, 2073, 2072, 2070, 2068, 2066, 2064, 2066, 2066, 2065, 2063, 2058, 2055, 2052, 2050, 2048, 2046, 2045, 2043, 2041, 2040, 2038, 2035, 2033, 2034, 2034, 2032, 2030, 2028, 2027, 2027, 2025, 2024, 2022, 2021, 2018, 2016, 2015, 2014, 2014, 2017, 2018, 2019, 2018, 2013, 2012, 2011, 2009, 2008, 2006, 2004, 2003, 2000, 1999, 1998, 1995, 1994, 1992, 1990, 1988, 1985, 1982, 1981, 1981, 1980, 1982, 1981, 1979, 1978, 1978, 1981, 1980, 1980, 1981, 1982, 1983, 1985, 1986, 1985, 1984, 1982, 1982, 1982, 1981, 1980, 1978, 1976, 1976, 1975, 1975, 1975, 1975, 1976, 1978, 1977, 1977, 1977, 1978, 1981, 1982, 1981, 1980, 1980, 1979, 1980, 1980, 1979, 1979, 1978, 1977, 1976, 1975, 1975, 1975, 1975, 1974, 1972, 1972, 1970, 1966, 1966, 1967, 1967, 1967, 1965, 1965, 1966, 1967, 1968, 1967, 1970, 1972, 1974, 1975, 1976, 1976, 1975, 1976, 1975, 1976, 1975, 1976, 1975, 1974, 1974, 1973, 1974, 1974, 1973, 1973, 1974, 1975, 1975, 1976, 1977, 1978, 1979, 1978, 1978, 1978, 1979, 1979, 1978, 1977, 1976, 1978, 1981, 1982, 1982, 1980, 1980, 1981, 1982, 1982, 1981, 1982, 1984, 1985, 1985, 1988, 1991, 1991, 1991, 1988, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 2000, 2001, 2003, 2002, 2006, 2009, 2011, 2014, 2017, 2019, 2023, 2025, 2025, 2027, 2029, 2033, 2036, 2039, 2040, 2042, 2042, 2043, 2045, 2051, 2054, 2056, 2059, 2060, 2060, 2060, 2063, 2065, 2065, 2065, 2065, 2066, 2068, 2068, 2071, 2074, 2077, 2079, 2082, 2085, 2087, 2092, 2095, 2098, 2102, 2103, 2107, 2110, 2109, 2111, 2113, 2116, 2119, 2121, 2123, 2124, 2123, 2126, 2126, 2127, 2129, 2129, 2129, 2129, 2131, 2132, 2131, 2132, 2132, 2134, 2138, 2141, 2140, 2140, 2142, 2145, 2145, 2148, 2149, 2150, 2151, 2150, 2149, 2150, 2148, 2147, 2147, 2145, 2145, 2144, 2143, 2142, 2141, 2142, 2141, 2140, 2138, 2136, 2136, 2134, 2133, 2131, 2130, 2128, 2124, 2122, 2120, 2119, 2117, 2115, 2113, 2113, 2112, 2111, 2110, 2109, 2108, 2107, 2106, 2106, 2105, 2104, 2104, 2102, 2103, 2103, 2102, 2101, 2099, 2100, 2096, 2096, 2095, 2092, 2092, 2090, 2089, 2088, 2086, 2082, 2081, 2080, 2078, 2077, 2075, 2073, 2072, 2070, 2070, 2070, 2069, 2069, 2068, 2068, 2068, 2068, 2068, 2070, 2071, 2071, 2069, 2065, 2063, 2061, 2060, 2060, 2059, 2060, 2061, 2059, 2058, 2056, 2056, 2056, 2057, 2057, 2056, 2057, 2058, 2057, 2055, 2054, 2053, 2053, 2053, 2052, 2049, 2048, 2047, 2048, 2049, 2050, 2052, 2053, 2052, 2052, 2051, 2050, 2051, 2051, 2052, 2051, 2050, 2048, 2048, 2046, 2044, 2043, 2041, 2041, 2039, 2040, 2042, 2043, 2045, 2043, 2043, 2042, 2042, 2042, 2043, 2042, 2041, 2043, 2042, 2042, 2042, 2043, 2043, 2041, 2040, 2040, 2039, 2038, 2038, 2039, 2039, 2037, 2037, 2036, 2036, 2035, 2035, 2037, 2037, 2036, 2036, 2036, 2033, 2033, 2034, 2035, 2036, 2035, 2035, 2036, 2037, 2038, 2037, 2037, 2036, 2038, 2037, 2035, 2035, 2032, 2033, 2033, 2032, 2032, 2028, 2028, 2028, 2027, 2027, 2028, 2031, 2030, 2029, 2029, 2029, 2030, 2030, 2030, 2029, 2029, 2028, 2027, 2028, 2029, 2029, 2028, 2028, 2029, 2028, 2026, 2027, 2024, 2025, 2026, 2027, 2027, 2028, 2028, 2028, 2029, 2030, 2031, 2033, 2034, 2035, 2039, 2041, 2045, 2048, 2048, 2051, 2052, 2053, 2055, 2058, 2059, 2060, 2059, 2058, 2061, 2063, 2063, 2063, 2064, 2066, 2067, 2067, 2070, 2072, 2072, 2072, 2072, 2073, 2073, 2073, 2076, 2075, 2075, 2076, 2076, 2076, 2076, 2076, 2075, 2077, 2077, 2078, 2078, 2079, 2078, 2079, 2080, 2081, 2081, 2080, 2079, 2078, 2076, 2077, 2078, 2078, 2079, 2077, 2074, 2074, 2073, 2071, 2071, 2071, 2070, 2069, 2067, 2065, 2064, 2063, 2063, 2063, 2063, 2062, 2061, 2061, 2061, 2060, 2058, 2057, 2059, 2057, 2056, 2054, 2053, 2053, 2052, 2051, 2050, 2049, 2048, 2048, 2046, 2045, 2043, 2040, 2040, 2040, 2039, 2039, 2037, 2037, 2038, 2037, 2037, 2036, 2035, 2033, 2034, 2034, 2032, 2031, 2030, 2028, 2028, 2028, 2025, 2022, 2021, 2019, 2019, 2018, 2015, 2015, 2015, 2015, 2015, 2012, 2011, 2012, 2013, 2013, 2012, 2013, 2013, 2013, 2012, 2012, 2013, 2013, 2013, 2012, 2010, 2010, 2009, 2007, 2007, 2007, 2008, 2007, 2005, 2004, 2003, 2004, 2003, 2003, 2002, 2001, 2003, 2003, 2004, 2006, 2007, 2007, 2006, 2007, 2006, 2007, 2008, 2009, 2008, 2007, 2006, 2005, 2006, 2005, 2004, 2005, 2006, 2006, 2005, 2007, 2008, 2010, 2010, 2011, 2011, 2012, 2013, 2013, 2015, 2015, 2015, 2017, 2017, 2018, 2019, 2019, 2021, 2023, 2023, 2025, 2025, 2025, 2026, 2026, 2028, 2029, 2031, 2033, 2034, 2037, 2038, 2038, 2041, 2043, 2044, 2046, 2048, 2048, 2050, 2051, 2053, 2054, 2054, 2055, 2055, 2055, 2054, 2056, 2056, 2058, 2059, 2061, 2062, 2063, 2063, 2065, 2065, 2066, 2069, 2071, 2071, 2072, 2072, 2072, 2073, 2072, 2072, 2073, 2074, 2073, 2074, 2075, 2076, 2077, 2078, 2079, 2079, 2079, 2079, 2080, 2081, 2082, 2083, 2083, 2084, 2083, 2083, 2083, 2082, 2082, 2082, 2082, 2083, 2083, 2083, 2082, 2081, 2081, 2081, 2081, 2082, 2082, 2081, 2082, 2082, 2082, 2082, 2083, 2083, 2083, 2084, 2084, 2083, 2084, 2085, 2085, 2084, 2083, 2083, 2083, 2083, 2082, 2082, 2082, 2081, 2080, 2081, 2082, 2081, 2081, 2081, 2079, 2079, 2081, 2084, 2083, 2083, 2083, 2082, 2082, 2083, 2084, 2084, 2085, 2086, 2086, 2086, 2085, 2085, 2085, 2085, 2085, 2086, 2085, 2086, 2088, 2086, 2087, 2088, 2088, 2089, 2089, 2089, 2090, 2089, 2088, 2088, 2088, 2087, 2086, 2085, 2084, 2084, 2084, 2084, 2083, 2083, 2082, 2081, 2081, 2081, 2080, 2080, 2079, 2079, 2078, 2078, 2079, 2079, 2078, 2077, 2077, 2076, 2075, 2074, 2074, 2074, 2073, 2073, 2072, 2070, 2068, 2067, 2067, 2067, 2065, 2064, 2063, 2062, 2063, 2061, 2061, 2062, 2061, 2059, 2060, 2060, 2059, 2057, 2056, 2055, 2054, 2055, 2054, 2053, 2051, 2049, 2047, 2047, 2046, 2045, 2045, 2044, 2043, 2044, 2044, 2043, 2042, 2042, 2042, 2040, 2039, 2039, 2037, 2036, 2034, 2032, 2031, 2031, 2031, 2032, 2032, 2032, 2031, 2030, 2031, 2032, 2031, 2031, 2031, 2030, 2030, 2029, 2029, 2029, 2030, 2032, 2033, 2032, 2031, 2031, 2029, 2029, 2029, 2029, 2028, 2029, 2028, 2027, 2026, 2024, 2023, 2023, 2024, 2024, 2024, 2022, 2022, 2021, 2021, 2021, 2021, 2022, 2023, 2024, 2023, 2023, 2023, 2022, 2024, 2023, 2022, 2021, 2020, 2020, 2020, 2019, 2019, 2020, 2019, 2019, 2019, 2019, 2018, 2019, 2021, 2022, 2022, 2022, 2024, 2024, 2024, 2023, 2023, 2025, 2025, 2025, 2026, 2027, 2027, 2027, 2027, 2027, 2027, 2027, 2029, 2029, 2030, 2030, 2030, 2031, 2030, 2029, 2029, 2030, 2029, 2029, 2029, 2029, 2029, 2028, 2028, 2027, 2026, 2026, 2025, 2026, 2025, 2025, 2024, 2024, 2024, 2024, 2023, 2023, 2023, 2022, 2023, 2023, 2022, 2022, 2022, 2022, 2021, 2020, 2020, 2020, 2019, 2019, 2019, 2018, 2018, 2017, 2017, 2016, 2015, 2013, 2012, 2012, 2011, 2011, 2011, 2010, 2010, 2009, 2009, 2009, 2008, 2007, 2007, 2007, 2007, 2007, 2006, 2006, 2006, 2006, 2006, 2006, 2005, 2005, 2005, 2005, 2006, 2006, 2005, 2005, 2006, 2006, 2006, 2006, 2006, 2005, 2006, 2006, 2006, 2007, 2007, 2007, 2008, 2009, 2009, 2009, 2010, 2011, 2011, 2012, 2012, 2013, 2013, 2014, 2015, 2016, 2017, 2017, 2018, 2021, 2022, 2023, 2024, 2024, 2025, 2025, 2026, 2029, 2030, 2031, 2033, 2033, 2034, 2036, 2037, 2038, 2039, 2040, 2040, 2040, 2041, 2041, 2042, 2042, 2044, 2045, 2047, 2048, 2049, 2050, 2051, 2052, 2054, 2054, 2054, 2055, 2056, 2057, 2059, 2060, 2061, 2061, 2062, 2064, 2066, 2066, 2067, 2067, 2068, 2069, 2070, 2070, 2071, 2071, 2070, 2071, 2073, 2073, 2072, 2072, 2073, 2075, 2076, 2078, 2078, 2079, 2079, 2079, 2079, 2079, 2079, 2078, 2077, 2076, 2077, 2076, 2076, 2075, 2075, 2074, 2075, 2074, 2074, 2074, 2074, 2074, 2073, 2073, 2074, 2073, 2072, 2071, 2070, 2070, 2070, 2071, 2070, 2069, 2069, 2068, 2067, 2067, 2068, 2067, 2067, 2068, 2069, 2069, 2069, 2070, 2070, 2070, 2071, 2071, 2071, 2071, 2072, 2072, 2072, 2073, 2074, 2075, 2075, 2076, 2075, 2075, 2075, 2075, 2075, 2075, 2076, 2076, 2076, 2077, 2077, 2077, 2077, 2077, 2077, 2078, 2078, 2077, 2076, 2076, 2076, 2074, 2075, 2075, 2074, 2074, 2074, 2074, 2073, 2073, 2074, 2074, 2074, 2074, 2074, 2074, 2073, 2072, 2072, 2073, 2073, 2072, 2071, 2072, 2071, 2071, 2071, 2071, 2070, 2069, 2069, 2068, 2068, 2067, 2066, 2066, 2065, 2064, 2064, 2063, 2063, 2063, 2063, 2063, 2063, 2063, 2064, 2063, 2063, 2064, 2064, 2064, 2064, 2064, 2064, 2065, 2065, 2066, 2067, 2066, 2065, 2065, 2065, 2065, 2065, 2065, 2065, 2066, 2066, 2065, 2065, 2065, 2064, 2064, 2063, 2062, 2062, 2062, 2061, 2062, 2062, 2061, 2061, 2062, 2060, 2059, 2058, 2057, 2058, 2056, 2055, 2055, 2054, 2053, 2052, 2052, 2052, 2051, 2050, 2049, 2049, 2048, 2048, 2046, 2045, 2045, 2044, 2043, 2042, 2042, 2042, 2041, 2041, 2040, 2039, 2039, 2038, 2037, 2037, 2037, 2036, 2035, 2035, 2034, 2035, 2034, 2033, 2033, 2032, 2032, 2032, 2033, 2033, 2033, 2033, 2032, 2031, 2031, 2032, 2032, 2032, 2032, 2032, 2031, 2031, 2031, 2032, 2033, 2033, 2034, 2034, 2034, 2034, 2035, 2034, 2035, 2035, 2036, 2035, 2036, 2036, 2036, 2036, 2036, 2036, 2036, 2037, 2037, 2036, 2037, 2038, 2038, 2039, 2039, 2038, 2039, 2040, 2039, 2039, 2040, 2041, 2041, 2041, 2042, 2044, 2045, 2046, 2046, 2046, 2044, 2045, 2046, 2047, 2047, 2047, 2047, 2047, 2048, 2049, 2050, 2050, 2050, 2050, 2051, 2051, 2051, 2052, 2052, 2053, 2054, 2054, 2055, 2054, 2054, 2055, 2056, 2056, 2055, 2059, 2060, 2056, 2054, 2054, 2054, 2055, 2056, 2056, 2056, 2056, 2056, 2056, 2056, 2056, 2057, 2057, 2057, 2055, 2056, 2055, 2055, 2055, 2053, 2053, 2052, 2052, 2051, 2051, 2051, 2051, 2051, 2051, 2050, 2049, 2048, 2048, 2048, 2047, 2047, 2047, 2046, 2046, 2045, 2044, 2045, 2044, 2043, 2043, 2042, 2043, 2043, 2042, 2041, 2041, 2040, 2040, 2041, 2041, 2041, 2041, 2041, 2040, 2040, 2040, 2040, 2039, 2039, 2038, 2038, 2038, 2037, 2037, 2037, 2037, 2037, 2037, 2036, 2036, 2036, 2035, 2035, 2035, 2035, 2036, 2035, 2034, 2034, 2034, 2035, 2033, 2033, 2033, 2032, 2032, 2032, 2032, 2032, 2032, 2031, 2030, 2030, 2028, 2028, 2028, 2029, 2030, 2029, 2029, 2030, 2029, 2029, 2028, 2028, 2028, 2028, 2029, 2029, 2029, 2029, 2029, 2029, 2028, 2028, 2029, 2029, 2029, 2030, 2030, 2030, 2031, 2031, 2031, 2031, 2031, 2032, 2032, 2033, 2033, 2034, 2035, 2035, 2035, 2035, 2034, 2034, 2035, 2035, 2034, 2035, 2035, 2036, 2035, 2036, 2036, 2036, 2036, 2037, 2037, 2038, 2038, 2038, 2038, 2038, 2038, 2038, 2038, 2038, 2039, 2039, 2039, 2040, 2040, 2039, 2039, 2038, 2039, 2039, 2039, 2039, 2039, 2038, 2038, 2038, 2038, 2038, 2038, 2038, 2038, 2038, 2037, 2037, 2037, 2037, 2036, 2036, 2036, 2035, 2035, 2036, 2037, 2036, 2036, 2035, 2035, 2035, 2035, 2034, 2034, 2034, 2034, 2034, 2034, 2034, 2034, 2035, 2035, 2035, 2035, 2036, 2035, 2035, 2035, 2035, 2036, 2036, 2036, 2036, 2036, 2036, 2036, 2036, 2036, 2035, 2036, 2036, 2036, 2036, 2037, 2037, 2037, 2037, 2038, 2037, 2038, 2038, 2039, 2039, 2040, 2040, 2040, 2041, 2041, 2042, 2043, 2045, 2046, 2046, 2048, 2048, 2048, 2049, 2050, 2051, 2051, 2053, 2054, 2055, 2056, 2057, 2057, 2059, 2060, 2060, 2061, 2061, 2062, 2063, 2064, 2065, 2066, 2066, 2067, 2069, 2071, 2071, 2072, 2071, 2071, 2072, 2072, 2073, 2073, 2073, 2074, 2074, 2075, 2075, 2076, 2076, 2077, 2077, 2076, 2077, 2077, 2078, 2078, 2079, 2079, 2079, 2079, 2079, 2080, 2079, 2079, 2078, 2078, 2079, 2080, 2079, 2079, 2079, 2079, 2078, 2079, 2079, 2078, 2078, 2078, 2077, 2078, 2078, 2078, 2077, 2078, 2077, 2077, 2076, 2076, 2076, 2076, 2077, 2076, 2076, 2076, 2076, 2076, 2076, 2076, 2076, 2076, 2075, 2075, 2075, 2074, 2074, 2074, 2073, 2074, 2073, 2073, 2073, 2073, 2073, 2074, 2074, 2073, 2073, 2073, 2073, 2073, 2073, 2073, 2072, 2072, 2072, 2072, 2072, 2072, 2072, 2071, 2071, 2071, 2070, 2071, 2070, 2069, 2069, 2069, 2068, 2068, 2068, 2068, 2068, 2067, 2067, 2067, 2066, 2065, 2065, 2065, 2064, 2064, 2065, 2065, 2065, 2064, 2064, 2063, 2063, 2062, 2061, 2061, 2060, 2059, 2060, 2059, 2059, 2058, 2057, 2057, 2057, 2057, 2057, 2056, 2056, 2056, 2055, 2055, 2055, 2054, 2054, 2053, 2053, 2053, 2053, 2053, 2051, 2051, 2051, 2051, 2052, 2051, 2051, 2051, 2051, 2049, 2049, 2049, 2049, 2048, 2047, 2048, 2047, 2047, 2047, 2046, 2046, 2046, 2045, 2045, 2044, 2044, 2043, 2043, 2043, 2043, 2043, 2043, 2042, 2041, 2041, 2040, 2039, 2039, 2039, 2039, 2037, 2037, 2037, 2037, 2036, 2035, 2035, 2035, 2035, 2034, 2033, 2033, 2033, 2032, 2032, 2032, 2031, 2031, 2031, 2031, 2031, 2031, 2030, 2030, 2029, 2029, 2029, 2029, 2029, 2028, 2028, 2027, 2027, 2027, 2026, 2026, 2026, 2026, 2026, 2025, 2025, 2026, 2026, 2025, 2024, 2025, 2025, 2025, 2025, 2025, 2024, 2025, 2025, 2025, 2025, 2025, 2025, 2025, 2025, 2025, 2025, 2025, 2025, 2026, 2026, 2026, 2026, 2025, 2025, 2025, 2025, 2024, 2024, 2025, 2025, 2025, 2026, 2026, 2026, 2025, 2026, 2027, 2027, 2027, 2026, 2026, 2026, 2027, 2026, 2026, 2026, 2026, 2026, 2027, 2028, 2028, 2029, 2030, 2030, 2030, 2030, 2029, 2030, 2030, 2031, 2031, 2031, 2032, 2033, 2033, 2034, 2035, 2035, 2035, 2036, 2036, 2037, 2039, 2039, 2040, 2040, 2039, 2039, 2040, 2040, 2041, 2042, 2042, 2043, 2044, 2045, 2045, 2045, 2045, 2045, 2046, 2046, 2046, 2047, 2048, 2048, 2049, 2049, 2049, 2050, 2050, 2050, 2051, 2051, 2052, 2052, 2052, 2052, 2052, 2053, 2053, 2053, 2053, 2052, 2053, 2053, 2054, 2054, 2054, 2055, 2055, 2055, 2055, 2056, 2056, 2056, 2056, 2056, 2055, 2056, 2055, 2055, 2055, 2054, 2055, 2054, 2054, 2053, 2053, 2052, 2053, 2053, 2052, 2052, 2052, 2053, 2053, 2052, 2052, 2052, 2052, 2052, 2052, 2052, 2053, 2053, 2052, 2053, 2053, 2053, 2053, 2054, 2053, 2053, 2053, 2053, 2054, 2053, 2054, 2054, 2054, 2054, 2054, 2055, 2054, 2054, 2054, 2054, 2054, 2054, 2054, 2054, 2054, 2055, 2055, 2055, 2055, 2056, 2056, 2056, 2057, 2057, 2057, 2057, 2057, 2057, 2058, 2058, 2058, 2058, 2058, 2059, 2059, 2060, 2060, 2060, 2060, 2060, 2060, 2060, 2061, 2061, 2062, 2063, 2063, 2064, 2065, 2065, 2065, 2065, 2065, 2065, 2065, 2065, 2066, 2067, 2066, 2067, 2067, 2068, 2067, 2067, 2068, 2068, 2068, 2068, 2068, 2068, 2068, 2069, 2069, 2070, 2071, 2071, 2071, 2070, 2071, 2071, 2071, 2071, 2072, 2072, 2072, 2071, 2071, 2071, 2071, 2072, 2072, 2071, 2071, 2070, 2070, 2071, 2071, 2071, 2071, 2070, 2069, 2069, 2069, 2069, 2069, 2069, 2069, 2069, 2068, 2068, 2068, 2067, 2067, 2066, 2066, 2065, 2065, 2064, 2063, 2063, 2063, 2063, 2062, 2061, 2060, 2060, 2060, 2060, 2059, 2059, 2058, 2057, 2056, 2056, 2055, 2054, 2054, 2054, 2054, 2053, 2052, 2051, 2051, 2050, 2050, 2049, 2050, 2049, 2048, 2048, 2048, 2047, 2047, 2047, 2047, 2046, 2045, 2045, 2045, 2044, 2044, 2043, 2043, 2043, 2043, 2043, 2043, 2042, 2041, 2042, 2041, 2041, 2040, 2040, 2039, 2039, 2038, 2038, 2037, 2037, 2037, 2036, 2036, 2036, 2036, 2036, 2036, 2036, 2036, 2035, 2035, 2035, 2035, 2035, 2035, 2035, 2035, 2035, 2035, 2035, 2035, 2035, 2035, 2035, 2034, 2034, 2034, 2034, 2034, 2034, 2034, 2033, 2034, 2035, 2035, 2035, 2034, 2035, 2034, 2033, 2034, 2034, 2034, 2034, 2034, 2035, 2035, 2035, 2035, 2035, 2035, 2034, 2034, 2033, 2033, 2033, 2034, 2033, 2032, 2032, 2032, 2032, 2032, 2032, 2032, 2032, 2032, 2032, 2032, 2031, 2031, 2032, 2032, 2032, 2031, 2031, 2031, 2031, 2031, 2030, 2030, 2030, 2030, 2030, 2030, 2029, 2029, 2029, 2029, 2029, 2028, 2028, 2028, 2028, 2028, 2027, 2028, 2028, 2027, 2027, 2027, 2027, 2027, 2028, 2028, 2028, 2028, 2029, 2029, 2029, 2029, 2029, 2029, 2029, 2029, 2030, 2030, 2030, 2030, 2031, 2031, 2032, 2032, 2033, 2033, 2033, 2034, 2034, 2034, 2035, 2035, 2036, 2035, 2036, 2036, 2036, 2036, 2036, 2038, 2038, 2038, 2038, 2039, 2039, 2039, 2040, 2041, 2041, 2041, 2042, 2042, 2042, 2043, 2043, 2044, 2045, 2045, 2045, 2046, 2047, 2047, 2047, 2047, 2048, 2048, 2049, 2049, 2050, 2050, 2050, 2051, 2051, 2051, 2052, 2052, 2052, 2052, 2053, 2054, 2054, 2054, 2054, 2054, 2054, 2054, 2054, 2053, 2053, 2054, 2053, 2053, 2054, 2054, 2054, 2054, 2054, 2054, 2053};

const int16_t SNARE[5500] = {2032, 2030, 2041, 2039, 2058, 2078, 2008, 2084, 2074, 2004, 2043, 2180, 2203, 2131, 2162, 2237, 2174, 2214, 2442, 2328, 2196, 2191, 2336, 2412, 2645, 2584, 2332, 2589, 2470, 2207, 2577, 2621, 2532, 1922, 2130, 2533, 2616, 2587, 2602, 1883, 786, 926, 1106, 793, 348, 520, 673, 807, 844, 974, 903, 776, 863, 1028, 1067, 977, 929, 1077, 1134, 1218, 1230, 1151, 1276, 1358, 1505, 1765, 1853, 2067, 2533, 2595, 2491, 2266, 2250, 2508, 2535, 2561, 2558, 2515, 2463, 2344, 2287, 2370, 2476, 2493, 2363, 2210, 2292, 2574, 2694, 2633, 2633, 2607, 2462, 2423, 2581, 2799, 2875, 2930, 2802, 2551, 2677, 3259, 3530, 3653, 3631, 3389, 3191, 2963, 2950, 3335, 3710, 3727, 3727, 3728, 3725, 3731, 3700, 3646, 3546, 3490, 3428, 3297, 3172, 3118, 3073, 3013, 2986, 2933, 2705, 2584, 2609, 2556, 2518, 2366, 2212, 2027, 1875, 1414, 1098, 993, 754, 440, 356, 377, 363, 374, 364, 373, 365, 372, 366, 371, 368, 368, 387, 395, 391, 447, 397, 604, 841, 920, 935, 856, 868, 967, 1087, 1181, 1189, 1059, 788, 530, 445, 591, 942, 1291, 1360, 1252, 1014, 907, 967, 1025, 1179, 1250, 1256, 1307, 1423, 1407, 1451, 1505, 1637, 1762, 1831, 1923, 1993, 2089, 2251, 2354, 2298, 2301, 2368, 2462, 2535, 2646, 2966, 3248, 3372, 3534, 3672, 3689, 3620, 3618, 3711, 3729, 3726, 3728, 3726, 3728, 3726, 3728, 3727, 3721, 3625, 3611, 3514, 3442, 3420, 3412, 3452, 3440, 3289, 3199, 3133, 3077, 3077, 2985, 2956, 2938, 2860, 2779, 2712, 2611, 2520, 2443, 2361, 2306, 2217, 2156, 2041, 1999, 2091, 2264, 2258, 2139, 2078, 2028, 1844, 1768, 1860, 1649, 1520, 1511, 1428, 1423, 1275, 1280, 1378, 1277, 1143, 1127, 1141, 1005, 916, 759, 556, 518, 459, 366, 372, 366, 431, 364, 381, 377, 372, 440, 564, 597, 706, 711, 755, 771, 811, 868, 831, 946, 994, 1070, 1058, 1093, 1268, 1485, 1421, 1422, 1812, 1890, 1549, 1690, 2100, 2091, 2288, 2556, 2422, 2414, 2667, 2570, 2420, 2602, 2599, 2397, 2448, 2936, 2944, 3030, 2890, 2579, 2880, 2954, 2701, 2778, 2827, 2677, 3009, 3178, 2855, 2712, 2950, 3138, 3255, 3204, 2992, 3268, 3319, 3027, 2968, 2784, 2778, 3008, 3230, 3052, 2909, 3030, 3111, 2940, 2946, 2832, 2736, 2823, 2662, 2617, 2652, 2590, 2511, 2486, 2504, 1878, 1923, 2205, 1827, 1908, 2184, 2028, 1573, 1476, 1334, 1435, 1539, 1577, 1416, 1336, 1505, 1468, 1074, 850, 830, 885, 1225, 1284, 899, 1282, 1266, 1230, 1224, 924, 1140, 978, 1032, 981, 990, 1209, 1258, 1436, 1735, 1671, 1506, 1726, 1780, 1829, 2014, 2189, 2109, 1933, 1779, 1900, 2224, 2122, 2176, 2193, 1897, 1831, 2099, 1906, 1707, 2017, 2270, 2370, 2489, 2498, 2410, 2403, 2205, 2408, 2591, 2360, 2310, 2300, 2485, 2516, 2631, 2302, 2563, 2589, 2441, 2678, 2720, 2844, 2683, 2800, 2827, 2704, 2695, 2765, 2864, 2758, 2794, 2827, 2790, 2804, 2938, 2895, 2614, 2667, 2430, 2764, 2803, 2593, 2346, 2451, 2493, 2486, 2592, 2710, 2468, 2278, 2386, 2300, 2318, 2086, 2367, 2190, 2037, 1825, 1689, 1820, 1932, 2136, 1906, 1634, 1758, 1862, 1552, 1311, 1538, 1862, 1862, 1531, 1274, 1267, 1329, 1360, 1310, 1750, 1830, 1532, 1483, 1774, 1748, 1838, 1779, 1484, 1616, 1564, 1624, 1382, 1476, 1436, 1187, 1076, 1335, 1561, 1531, 1611, 1524, 1509, 1478, 1466, 1612, 1519, 1506, 1538, 1529, 1530, 1743, 1842, 1590, 1726, 2093, 2063, 2053, 2344, 2239, 2038, 2172, 2607, 2397, 1952, 2329, 2172, 2051, 2132, 2200, 2579, 2693, 2542, 2570, 2473, 2294, 2712, 2789, 2768, 2644, 2467, 2660, 2615, 2789, 2946, 2559, 2539, 2465, 2518, 2429, 2637, 2602, 2553, 2487, 2630, 2782, 2444, 2477, 2400, 2533, 2427, 2088, 2500, 2533, 2256, 2255, 2234, 2152, 1967, 2118, 2376, 2301, 2102, 2345, 2148, 2044, 2013, 1806, 1829, 2158, 2423, 2222, 2002, 1771, 1688, 2046, 2296, 1888, 1624, 1802, 1778, 1790, 1721, 1723, 1591, 1339, 1375, 1612, 1318, 1301, 1400, 1544, 1475, 1512, 1837, 1630, 1488, 1535, 1613, 1723, 1665, 1355, 1298, 1696, 1837, 1462, 1391, 1347, 1598, 1619, 1790, 1894, 1969, 2069, 1910, 2026, 1969, 2025, 1862, 2010, 2239, 1954, 1937, 1967, 2209, 2297, 2398, 2064, 1943, 2329, 2285, 2021, 2126, 2359, 2191, 1989, 2373, 2291, 1975, 2253, 2189, 2033, 2291, 2480, 2305, 2209, 2307, 2506, 2489, 2360, 2327, 2462, 2491, 2375, 2382, 2325, 2367, 2391, 2406, 2359, 2270, 2299, 2695, 2684, 2356, 2454, 2626, 2808, 2263, 2065, 2294, 2325, 2189, 1968, 2037, 2211, 2174, 2004, 1940, 1939, 1702, 2155, 2480, 2091, 1777, 1658, 1864, 1958, 1803, 1723, 1783, 1867, 1715, 1758, 1664, 1646, 1682, 1725, 1779, 1796, 1871, 1805, 1857, 1905, 1941, 1732, 1769, 1645, 1420, 1590, 1679, 1895, 1519, 1580, 2069, 1678, 1688, 1984, 1841, 1641, 1710, 1845, 1886, 2000, 2081, 1701, 1457, 1618, 1849, 1738, 1660, 1668, 1879, 1918, 1708, 1599, 1858, 2029, 1821, 1695, 1898, 1994, 2102, 1993, 1878, 2023, 2045, 2120, 2029, 2255, 2193, 2117, 2143, 1836, 2112, 2389, 2225, 2184, 2235, 2253, 2338, 2800, 2422, 2263, 2631, 2570, 2280, 2327, 2575, 2429, 2603, 2606, 2359, 2247, 2455, 2501, 2385, 2705, 2343, 2202, 2494, 2425, 2361, 2448, 2395, 2092, 2219, 2558, 2283, 1863, 2300, 2396, 2270, 2172, 2072, 2015, 2038, 2156, 2152, 2087, 1905, 1619, 2076, 2316, 2011, 1912, 1829, 2026, 1694, 1975, 2243, 2097, 2094, 1900, 1752, 1886, 1889, 1708, 1970, 1826, 1696, 2004, 1948, 1385, 1546, 1894, 1783, 1725, 1707, 1577, 1722, 1903, 1490, 1525, 1740, 1589, 1616, 1834, 1785, 1655, 1763, 1866, 1813, 1734, 1421, 1612, 1739, 1485, 1558, 1972, 2289, 1665, 1533, 1877, 1979, 1689, 1873, 2078, 2124, 2160, 1870, 2004, 1909, 2154, 2315, 2279, 1935, 1990, 2055, 1941, 2187, 2395, 2191, 2082, 2401, 2457, 2191, 2027, 2154, 2162, 2379, 2322, 2174, 2339, 2329, 2083, 2439, 2523, 1980, 2253, 2533, 2269, 2426, 2066, 2227, 2512, 2435, 2442, 2640, 2228, 2180, 2590, 2459, 2726, 2450, 2377, 2277, 2387, 2566, 2420, 2458, 2172, 1954, 2310, 2191, 2245, 2767, 2121, 1888, 2264, 2462, 2157, 2164, 2150, 1721, 1914, 2012, 1883, 1729, 2122, 1966, 1615, 1663, 1786, 1876, 1712, 1658, 1847, 2084, 1980, 1416, 2081, 2284, 1841, 1481, 1669, 1961, 1736, 1771, 1757, 1563, 1729, 1915, 1861, 1732, 1594, 2022, 1819, 1676, 1863, 2049, 2093, 1689, 1562, 1820, 1764, 1819, 2039, 1990, 1672, 1660, 1940, 2364, 1911, 1700, 2050, 1978, 1925, 2020, 2026, 1958, 2219, 2263, 2222, 2100, 2056, 2050, 2097, 2275, 2115, 2306, 2162, 1703, 2064, 2256, 2427, 2424, 2404, 1948, 1868, 2249, 2496, 2384, 2288, 2412, 2251, 2626, 2408, 2252, 2452, 2362, 2187, 2526, 2375, 2322, 2423, 2268, 2274, 2256, 2320, 2211, 2116, 2316, 2179, 2000, 2128, 2083, 2031, 2215, 2400, 2235, 2175, 2192, 2129, 2045, 2445, 1919, 1628, 1772, 2229, 2050, 1777, 1630, 1848, 2297, 1827, 1989, 2107, 2060, 1876, 1921, 2063, 1998, 2105, 2123, 1770, 1943, 2090, 1886, 1989, 2131, 1963, 1996, 2121, 1879, 1546, 1753, 1994, 2064, 1840, 1660, 1855, 1736, 1986, 2147, 1820, 1793, 2032, 2077, 1609, 1630, 1751, 1921, 1947, 1804, 1970, 1815, 1802, 1881, 1783, 1871, 1920, 2007, 1871, 2004, 1985, 1938, 2407, 1932, 2133, 2179, 1984, 1938, 2130, 2252, 2234, 2199, 2023, 2007, 2096, 2217, 2157, 2303, 2083, 2113, 2112, 1948, 2113, 2313, 2256, 2084, 2138, 2382, 2305, 2260, 2134, 2109, 2507, 2405, 2192, 2308, 2143, 2255, 2194, 2287, 2302, 2255, 2143, 2249, 2361, 2168, 1975, 2337, 2405, 1995, 2150, 2106, 2208, 2119, 2184, 1890, 2247, 2393, 2004, 2205, 2196, 2234, 2096, 1949, 2039, 2163, 2030, 1868, 2063, 1949, 1999, 2072, 1900, 1979, 1839, 2014, 2017, 1799, 1897, 1982, 1895, 1942, 2066, 1827, 1895, 2003, 1994, 1811, 1731, 2007, 2214, 1741, 1557, 2197, 2149, 1902, 1861, 1754, 1779, 1955, 1908, 1798, 1912, 2036, 2089, 1958, 1868, 1998, 2076, 1916, 1794, 2013, 2141, 2110, 1937, 1899, 2095, 1840, 2025, 1966, 1860, 1956, 2129, 2138, 2212, 2237, 2068, 2194, 2022, 2071, 2198, 2028, 1970, 2330, 2370, 2144, 2179, 2088, 2055, 2049, 1916, 2223, 2033, 2190, 2072, 2041, 2289, 2152, 2121, 1939, 2117, 2197, 2133, 1838, 2002, 2173, 2346, 2215, 2238, 2254, 2039, 2291, 1906, 2115, 2101, 1893, 2246, 2337, 2082, 1924, 1925, 2256, 2184, 1751, 1873, 1896, 2070, 2334, 2168, 2021, 2065, 1951, 2026, 1908, 2111, 2445, 2155, 1766, 1749, 2382, 1995, 1773, 2446, 2435, 2160, 2032, 1930, 2000, 2216, 1874, 2321, 2290, 2221, 2122, 1782, 1893, 2252, 2339, 1637, 1761, 2063, 1946, 1784, 2035, 1883, 1900, 2159, 2128, 1906, 1694, 2029, 2030, 2023, 1803, 1827, 1945, 1965, 2015, 1982, 1928, 1894, 2023, 1908, 1789, 1773, 2044, 2071, 1705, 1827, 2089, 2097, 2023, 2073, 2115, 1871, 1981, 2033, 2143, 2029, 1962, 2061, 2154, 2092, 2077, 1897, 2142, 2281, 1970, 2110, 1880, 1990, 2245, 2182, 2130, 2048, 2020, 2236, 2010, 2039, 2168, 2085, 2339, 2276, 2079, 2076, 2044, 2053, 2359, 2178, 1979, 2047, 1941, 2107, 2299, 2257, 2092, 2075, 1924, 2367, 2314, 1811, 2018, 2047, 2066, 2038, 2030, 2231, 2082, 1902, 1991, 2061, 2089, 1955, 2043, 2199, 2039, 1973, 1987, 2100, 2031, 1854, 2260, 1975, 1796, 2016, 2095, 2093, 1934, 1961, 1928, 2068, 2117, 2032, 1867, 1851, 2121, 2128, 1788, 1913, 2124, 2088, 1987, 1944, 1942, 1824, 2053, 1968, 1942, 1903, 1918, 2007, 1947, 2075, 2069, 1929, 1988, 2156, 2109, 2074, 2122, 2107, 2005, 2119, 2094, 2244, 2178, 1905, 1821, 2193, 2212, 1856, 2074, 2158, 1863, 2100, 2235, 2014, 2101, 2104, 2068, 2045, 2116, 2031, 2115, 2181, 2073, 2193, 2047, 2126, 1926, 2067, 2195, 1906, 2047, 2207, 2128, 1820, 1887, 2094, 2185, 2045, 2061, 2030, 2061, 2068, 2048, 2200, 2224, 2031, 1975, 1969, 2135, 2008, 2292, 2160, 1986, 2089, 1856, 2109, 2122, 2073, 1958, 2143, 2193, 1968, 1950, 2018, 2065, 2086, 2156, 2067, 1998, 2015, 1937, 2015, 2066, 1976, 1938, 2034, 2038, 2100, 2079, 2000, 1991, 1874, 2062, 2053, 2022, 1975, 1983, 1948, 2041, 2225, 2001, 2134, 2018, 1970, 1909, 2222, 1988, 1880, 2038, 1983, 2007, 1956, 1965, 2035, 2022, 1812, 2049, 2240, 1914, 1746, 2142, 2088, 1991, 2130, 2101, 2048, 2167, 1991, 1889, 1889, 2012, 2146, 2196, 1940, 2146, 2224, 1942, 1955, 2153, 2203, 1962, 2012, 2178, 2178, 1983, 1960, 2127, 2144, 2165, 2125, 2080, 1974, 2073, 2232, 2103, 2147, 2279, 2118, 1964, 2175, 2256, 2135, 2060, 2128, 2275, 2217, 2124, 2068, 1888, 1984, 2175, 2162, 2026, 1923, 2168, 2136, 1918, 2042, 1969, 2033, 2046, 1981, 2016, 1970, 1966, 2073, 1941, 2066, 2243, 2011, 1899, 2180, 2071, 1815, 1963, 1907, 2079, 2198, 2070, 2006, 2017, 2050, 2053, 1996, 1984, 1967, 1907, 1998, 2041, 2094, 2020, 2002, 2142, 2113, 1955, 1978, 1999, 1936, 1983, 2093, 2044, 1909, 2066, 2055, 2067, 1968, 2077, 2050, 1962, 2074, 2002, 2006, 2043, 2024, 2036, 2136, 1890, 1860, 1914, 1995, 2089, 2054, 2033, 2072, 2109, 1884, 1981, 2213, 2188, 2110, 2083, 1952, 1997, 2024, 1979, 2058, 2158, 2328, 2202, 1985, 2080, 2109, 2045, 2129, 2106, 2080, 2043, 2198, 2160, 1996, 1998, 2015, 2144, 2173, 2135, 2046, 2077, 2143, 2125, 2031, 1980, 2091, 2030, 1994, 2048, 2207, 2019, 2043, 2121, 2033, 2071, 1962, 2088, 2082, 2038, 2052, 2153, 2073, 1986, 2123, 2117, 2084, 2095, 2118, 2126, 2060, 1988, 2144, 2022, 1906, 2132, 2282, 2140, 2046, 2018, 2107, 2064, 1961, 2121, 2144, 2050, 1879, 1997, 2001, 2151, 1968, 1935, 2148, 2015, 1993, 2074, 1989, 1940, 1881, 2032, 2165, 1887, 1877, 2005, 2122, 2006, 1855, 1958, 2057, 2059, 2004, 1990, 2092, 2030, 1979, 2136, 2012, 1906, 2004, 2049, 2061, 1948, 2081, 2002, 2127, 1962, 1936, 2092, 2121, 2049, 1968, 2073, 1939, 2051, 1936, 1955, 1999, 2055, 2143, 2007, 2017, 2224, 1919, 1937, 2175, 2142, 2054, 1922, 2132, 1967, 1870, 2119, 2162, 1926, 2092, 2103, 2125, 2180, 2151, 2057, 1967, 1976, 2053, 2192, 2081, 1977, 2033, 2192, 2099, 2046, 1976, 2068, 2214, 2027, 1970, 2140, 2081, 2076, 2188, 2120, 1996, 2040, 2083, 2252, 2014, 1900, 1980, 2143, 2215, 1965, 2055, 2051, 2023, 2170, 1994, 1952, 2132, 2024, 2002, 2129, 2097, 2065, 2070, 2096, 1997, 1995, 2166, 2049, 1958, 1957, 2104, 2100, 2002, 2090, 2147, 2037, 2006, 2084, 2078, 1964, 2037, 2138, 2064, 2012, 1950, 2199, 2080, 1929, 2060, 1993, 2112, 2025, 1954, 2072, 2095, 1878, 2066, 2212, 1943, 1955, 2062, 2114, 2020, 1909, 2021, 2036, 2056, 1981, 1962, 1987, 1958, 2014, 2015, 2021, 2061, 1920, 1960, 2095, 2019, 2062, 2196, 2109, 1911, 2112, 2109, 2075, 2018, 1969, 2011, 2159, 2129, 2073, 2079, 2036, 2027, 1993, 2058, 2122, 2079, 2061, 2065, 2107, 2089, 2063, 2034, 1950, 2074, 2062, 2102, 2105, 2033, 2015, 2066, 2166, 2023, 2092, 2135, 2019, 2057, 2051, 2021, 2108, 2056, 2005, 2084, 2052, 2100, 2186, 2121, 2019, 1994, 2126, 2104, 2030, 2009, 2088, 2121, 2035, 1941, 2056, 2098, 2074, 2052, 2023, 2125, 1956, 2019, 1987, 1985, 2054, 2167, 2062, 2009, 2023, 1999, 2005, 2029, 2023, 2000, 2024, 1975, 2105, 2068, 1947, 1970, 2083, 1950, 2007, 2036, 1958, 2044, 2005, 2015, 2048, 2105, 1935, 1937, 1970, 1958, 2086, 2081, 2026, 2099, 2065, 1952, 2028, 2013, 2060, 2037, 1967, 2074, 2124, 2082, 2072, 2012, 1978, 2069, 2089, 2084, 2010, 2004, 2100, 2130, 2010, 2023, 2157, 2039, 2021, 2091, 2110, 1965, 2061, 2104, 2017, 2148, 2117, 2103, 2038, 1982, 2180, 2078, 1969, 1982, 2061, 2081, 2124, 2168, 2006, 2053, 2142, 2118, 2014, 2063, 2093, 2064, 2100, 2165, 2132, 2023, 2034, 2058, 2147, 2138, 1986, 2088, 2095, 2220, 2037, 2048, 2070, 2104, 2106, 2024, 2054, 2021, 2084, 1977, 2111, 2076, 2085, 1942, 1978, 2030, 2033, 2060, 2005, 1994, 2038, 2045, 1982, 1945, 1944, 2005, 1985, 2006, 2084, 2069, 2072, 2021, 1991, 2046, 1997, 2043, 2063, 2044, 1945, 2104, 2054, 1952, 2037, 2087, 1985, 1979, 1983, 1942, 2101, 2131, 2006, 1905, 2084, 1988, 2048, 2001, 1966, 2074, 2027, 1963, 2008, 2181, 1962, 2055, 2067, 2006, 2097, 2071, 1990, 1990, 2070, 2041, 2044, 2117, 2063, 1953, 2057, 2066, 2058, 1961, 2042, 2148, 2104, 2063, 2013, 2066, 2092, 2099, 2086, 2057, 2059, 2064, 2054, 2086, 2071, 2041, 2032, 2007, 2178, 2025, 1994, 2125, 2104, 2071, 1984, 2136, 2123, 2160, 2030, 2072, 2120, 2059, 2013, 2020, 2021, 2101, 2131, 2050, 2090, 2013, 2068, 2097, 2001, 2019, 2128, 2043, 2022, 2089, 2119, 2098, 2034, 1987, 1977, 2035, 2105, 2069, 2076, 1991, 2078, 2084, 2014, 2022, 2089, 2032, 1992, 2055, 2064, 2031, 1943, 2007, 2034, 2058, 2051, 2054, 2085, 2014, 1993, 2056, 2043, 2032, 2029, 2040, 2016, 2070, 1982, 1968, 2070, 2051, 1952, 2025, 2105, 2045, 1998, 2034, 2020, 1993, 2069, 2015, 1931, 2020, 2075, 2084, 2084, 2060, 1999, 2038, 2146, 2062, 1995, 2005, 2014, 2104, 2041, 1986, 2083, 2097, 2014, 1874, 2013, 2110, 2100, 2022, 1991, 2034, 2077, 2020, 2044, 1973, 1993, 2054, 2039, 2057, 2060, 2091, 2058, 2033, 2058, 2104, 2021, 1970, 2041, 2177, 2150, 2023, 2097, 2142, 2062, 2070, 2044, 2132, 2087, 1967, 2118, 2173, 2071, 2123, 2080, 2021, 2076, 2163, 2109, 1994, 2039, 2124, 2056, 1986, 2146, 2153, 1994, 1972, 2066, 2039, 2063, 2102, 2085, 2001, 1994, 2059, 2035, 2059, 2017, 2023, 1995, 2040, 2027, 1991, 1991, 2099, 2097, 2018, 2020, 2039, 2024, 1968, 2057, 2046, 2056, 2058, 2039, 2079, 2097, 1962, 2035, 2037, 1986, 2021, 2032, 2099, 2023, 1930, 2022, 2059, 1992, 2028, 2026, 2058, 2027, 1998, 1967, 1983, 2041, 2035, 2008, 2007, 2060, 2012, 1934, 2023, 2072, 2005, 1998, 2127, 2083, 1953, 2020, 2082, 2094, 2052, 1988, 2016, 2162, 2056, 2045, 2056, 2003, 2128, 2098, 2019, 2048, 2038, 2017, 2099, 2145, 2078, 2097, 2092, 2060, 2028, 2059, 2120, 2096, 2096, 2053, 2048, 2085, 2108, 2044, 2107, 2112, 2105, 2123, 2037, 2061, 2100, 2033, 2051, 2112, 2133, 2043, 2055, 2010, 2042, 2113, 2067, 2097, 2087, 2000, 2028, 2072, 2045, 2097, 2040, 2090, 2081, 2074, 2052, 2059, 2068, 2015, 2005, 2024, 2096, 2113, 2010, 2046, 2070, 2052, 2022, 2013, 2035, 2017, 2018, 2010, 2029, 2025, 1987, 1989, 1986, 1925, 2037, 2063, 1969, 1958, 1985, 2024, 2024, 1999, 2020, 2005, 2026, 2038, 2021, 1957, 1983, 2086, 2058, 1966, 1973, 2060, 2100, 2084, 2013, 2002, 2050, 2074, 2000, 2093, 2131, 2029, 1974, 2058, 2055, 2044, 2068, 2037, 2069, 2037, 2016, 2081, 2097, 2005, 2018, 2087, 2036, 2011, 2035, 2054, 2097, 2068, 2014, 2031, 2131, 2115, 2037, 2021, 2041, 2106, 2136, 2078, 2077, 2060, 2097, 2068, 2096, 2076, 2054, 2083, 2094, 2100, 2044, 2072, 2108, 2123, 2054, 2101, 2108, 2065, 2051, 2083, 2150, 2097, 2023, 2077, 2079, 2116, 2033, 2019, 2086, 2071, 2082, 2053, 2041, 2073, 2056, 2026, 2062, 2088, 2023, 1996, 2036, 2009, 2072, 2061, 2028, 2007, 2003, 2028, 2106, 2022, 1993, 2070, 2035, 2005, 2032, 2029, 2042, 2023, 2034, 2077, 2011, 2022, 2008, 1992, 2052, 2050, 2067, 2072, 1968, 1964, 2034, 2029, 2018, 2043, 1998, 1967, 2039, 2020, 1979, 2009, 1987, 2020, 2037, 2005, 1981, 1976, 2021, 1983, 1979, 2007, 1981, 2024, 2031, 2038, 2026, 1998, 2002, 2018, 2032, 2021, 2040, 2054, 2008, 2049, 2049, 2026, 2073, 2052, 2032, 2093, 2124, 2001, 2044, 2092, 2060, 2094, 2081, 2047, 2055, 2139, 2117, 2101, 2076, 2052, 2150, 2134, 2044, 2121, 2132, 2088, 2108, 2070, 2069, 2078, 2087, 2084, 2069, 2109, 2076, 2104, 2049, 2097, 2159, 2077, 2086, 2066, 2083, 2058, 2109, 2098, 2108, 2069, 2060, 2087, 2066, 2095, 2102, 2052, 2011, 2088, 2101, 2074, 2036, 2049, 2066, 2090, 2024, 2025, 2064, 2039, 2001, 2000, 2045, 2022, 2045, 1996, 2043, 2071, 1976, 1924, 2003, 2070, 2026, 1999, 1994, 2007, 2005, 2018, 2010, 1990, 1999, 1980, 2021, 2026, 2009, 2014, 1971, 2022, 2023, 2038, 2021, 2018, 2022, 2016, 2027, 2008, 2003, 2033, 2068, 2007, 1992, 2017, 2023, 1999, 2048, 2054, 1991, 2043, 1998, 2008, 2054, 2014, 2066, 2057, 2017, 2028, 2061, 2057, 2025, 2048, 2072, 2074, 2033, 2050, 2059, 2029, 2051, 2033, 2025, 2067, 2125, 2078, 2007, 2029, 2100, 2127, 2052, 2041, 2078, 2118, 2099, 2058, 2072, 2074, 2101, 2090, 2072, 2085, 2086, 2105, 2087, 2066, 2100, 2104, 2111, 2084, 2065, 2100, 2067, 2036, 2107, 2101, 2117, 2069, 2066, 2065, 2069, 2081, 2064, 2087, 2106, 2106, 2043, 2047, 2060, 2119, 2075, 2046, 2041, 2071, 2074, 2062, 2032, 2017, 2017, 2029, 2033, 2061, 2063, 2032, 2040, 2007, 2057, 2040, 2036, 2042, 2008, 2004, 2004, 2029, 2000, 2008, 2007, 2029, 2034, 2019, 1991, 1980, 2029, 1990, 1995, 2022, 2034, 2001, 1955, 1990, 2039, 1971, 1996, 2017, 1987, 2024, 2006, 1999, 2038, 2029, 2014, 2035, 2009, 1986, 2019, 2077, 2049, 2003, 2025, 2063, 2051, 2052, 2052, 2066, 2047, 2037, 2062, 2071, 2100, 2074, 2087, 2056, 2043, 2062, 2099, 2069, 2058, 2095, 2039, 2090, 2109, 2083, 2045, 2082, 2050, 2005, 2088, 2106, 2094, 2076, 2042, 2062, 2080, 2080, 2089, 2068, 2096, 2052, 2048, 2092, 2104, 2085, 2076, 2068, 2079, 2046, 2093, 2077, 2019, 2072, 2108, 2094, 2050, 2064, 2054, 2047, 2065, 2054, 2076, 2073, 2030, 2035, 2067, 2059, 2031, 2054, 2043, 2019, 2035, 2057, 2027, 2017, 2022, 2031, 2015, 2003, 2027, 2052, 2008, 1986, 2048, 2074, 2017, 1987, 2038, 2004, 1989, 2044, 2033, 2020, 2010, 2026, 2015, 2029, 2013, 2029, 2023, 1995, 2033, 2045, 2037, 1971, 2024, 2027, 2010, 2050, 2051, 2032, 2033, 2065, 2016, 2017, 2039, 2027, 2042, 2062, 2048, 2038, 2040, 2030, 2039, 2040, 2042, 2073, 2046, 2038, 2039, 2050, 2041, 2041, 2059, 2035, 2071, 2062, 2064, 2056, 2047, 2054, 2054, 2072, 2059, 2048, 2085, 2093, 2063, 2074, 2083, 2074, 2092, 2098, 2078, 2075, 2117, 2084, 2066, 2077, 2078, 2090, 2073, 2093, 2060, 2022, 2045, 2080, 2098, 2096, 2083, 2063, 2065, 2022, 2062, 2080, 2061, 2061, 2016, 2051, 2056, 2032, 2037, 2056, 2034, 2024, 2045, 2061, 2048, 2043, 2051, 2002, 2043, 2053, 2014, 2022, 2045, 2035, 2017, 2052, 2033, 2007, 2042, 2041, 2014, 2059, 2057, 1995, 1970, 1999, 2037, 2048, 2035, 2040, 2046, 2018, 2034, 2049, 2022, 2021, 2040, 2038, 2012, 2018, 2027, 2011, 2036, 2033, 2009, 2018, 2027, 2045, 2035, 2023, 2043, 2033, 2052, 2058, 2034, 2033, 2039, 2057, 2062, 2044, 2047, 2039, 2051, 2099, 2060, 2046, 2057, 2072, 2065, 2069, 2059, 2061, 2069, 2064, 2050, 2070, 2061, 2042, 2089, 2040, 2043, 2049, 2041, 2060, 2060, 2079, 2031, 2042, 2073, 2061, 2080, 2062, 2047, 2076, 2078, 2061, 2062, 2081, 2075, 2068, 2068, 2083, 2065, 2054, 2059, 2058, 2081, 2077, 2047, 2052, 2086, 2058, 2083, 2069, 2034, 2030, 2080, 2054, 2044, 2039, 2029, 2054, 2062, 2060, 2053, 2038, 2022, 2051, 2047, 2034, 2037, 2047, 2031, 2028, 2042, 2041, 2015, 1999, 1996, 2048, 2046, 2048, 2059, 2017, 2019, 2049, 2059, 2027, 2039, 2036, 2021, 2023, 2038, 2068, 2040, 2016, 2027, 2016, 2040, 2050, 2054, 2061, 2061, 2050, 2033, 2034, 2065, 2057, 2036, 2048, 2058, 2052, 2012, 2048, 2038, 2020, 2032, 2059, 2048, 2017, 2011, 2034, 2066, 2064, 2044, 2043, 2043, 2028, 2042, 2054, 2040, 2035, 2039, 2028, 2013, 2054, 2065, 2030, 2031, 2091, 2070, 2030, 2057, 2053, 2042, 2035, 2071, 2088, 2082, 2032, 2047, 2079, 2062, 2047, 2106, 2104, 2071, 2061, 2046, 2061, 2068, 2064, 2047, 2061, 2076, 2061, 2053, 2070, 2041, 2052, 2061, 2053, 2030, 2043, 2077, 2053, 2028, 2039, 2049, 2039, 2037, 2040, 2074, 2068, 2049, 2023, 2044, 2048, 2050, 2074, 2032, 2043, 2058, 2041, 2022, 2035, 2075, 2062, 2052, 2045, 2048, 2066, 2060, 2034, 2038, 2060, 2072, 2066, 2020, 2040, 2048, 2075, 2058, 2024, 2036, 2040, 2042, 2035, 2030, 2049, 2049, 2003, 2018, 2027, 2020, 2005, 2016, 2034, 2025, 2012, 2013, 2045, 2039, 2006, 2024, 2048, 2040, 2029, 2028, 2040, 2054, 2044, 2025, 2046, 2070, 2053, 2028, 2059, 2070, 2065, 2050, 2039, 2056, 2071, 2085, 2057, 2064, 2055, 2053, 2065, 2063, 2048, 2065, 2098, 2052, 2025, 2032, 2066, 2076, 2065, 2044, 2067, 2079, 2072, 2063, 2050, 2057, 2056, 2091, 2066, 2048, 2086, 2081, 2074, 2086, 2059, 2075, 2082, 2051, 2047, 2068, 2083, 2051, 2039, 2054, 2067, 2053, 2058, 2077, 2049, 2026, 2045, 2060, 2046, 2022, 2048, 2053, 2038, 2032, 2013, 2020, 2038, 2010, 2009, 2034, 2043, 2022, 2010, 2001, 2019, 2026, 2019, 2003, 2022, 2032, 2003, 2008, 2023, 2059, 2028, 2035, 2028, 2023, 2030, 2023, 2045, 2057, 2060, 2042, 2048, 2046, 2050, 2052, 2052, 2079, 2054, 2040, 2065, 2064, 2063, 2051, 2037, 2037, 2055, 2055, 2034, 2050, 2069, 2059, 2024, 2041, 2067, 2049, 2039, 2049, 2070, 2069, 2060, 2051, 2039, 2061, 2061, 2049, 2060, 2096, 2067, 2069, 2071, 2087, 2079, 2044, 2061, 2062, 2064, 2039, 2076, 2082, 2077, 2056, 2058, 2064, 2070, 2076, 2071, 2051, 2069, 2090, 2051, 2045, 2070, 2043, 2026, 2068, 2064, 2050, 2029, 2041, 2056, 2077, 2054, 2029, 2046, 2044, 2031, 2023, 2023, 2036, 2048, 2016, 2009, 2006, 2018, 2025, 2013, 2028, 2023, 2023, 2032, 2041, 2039, 2015, 2039, 2046, 2025, 2019, 2026, 2021, 2031, 2048, 2039, 2030, 2032, 2053, 2054, 2058, 2053, 2032, 2043, 2042, 2017, 2034, 2042, 2037, 2036, 2023, 2020, 2039, 2029, 2039, 2056, 2038, 2008, 2033, 2072, 2048, 2037, 2044, 2054, 2069, 2062, 2039, 2047, 2059, 2052, 2073, 2081, 2057, 2044, 2046, 2081, 2083, 2062, 2063, 2080, 2087, 2053, 2068, 2073, 2066, 2072, 2073, 2087, 2085, 2074, 2073, 2079, 2079, 2087, 2089, 2091, 2068, 2068, 2072, 2056, 2065, 2078, 2065, 2068, 2079, 2041, 2033, 2058, 2053, 2034, 2029, 2024, 2028, 2058, 2033, 2013, 2041, 2030, 2026, 2045, 2037, 2023, 2036, 2043, 2010, 2017, 2022, 2010, 2025, 2027, 2032, 2023, 2004, 2001, 2016, 2016, 2028, 2033, 2014, 2009, 2037, 2040, 2028, 2015, 2003, 2006, 2022, 2034, 2012, 2011, 2029, 2032, 2032, 2023, 2033, 2062, 2030, 2035, 2055, 2035, 2026, 2042, 2063, 2053, 2060, 2034, 2049, 2069, 2059, 2059, 2052, 2056, 2071, 2071, 2068, 2057, 2043, 2054, 2072, 2100, 2077, 2067, 2068, 2065, 2067, 2071, 2087, 2087, 2096, 2086, 2058, 2083, 2097, 2094, 2075, 2062, 2080, 2095, 2052, 2045, 2080, 2085, 2062, 2044, 2068, 2079, 2046, 2030, 2075, 2066, 2019, 2031, 2075, 2055, 2043, 2055, 2054, 2033, 2029, 2040, 2043, 2053, 2040, 2040, 2017, 2020, 2036, 2042, 2035, 2022, 2029, 2022, 2015, 2023, 2021, 2020, 2025, 2023, 2025, 2012, 2032, 2046, 2022, 1995, 2024, 2040, 2014, 2028, 2048, 2024, 2018, 2023, 2024, 2036, 2019, 2029, 2033, 2029, 2036, 2020, 2029, 2044, 2035, 2030, 2034, 2039, 2034, 2033, 2038, 2025, 2037, 2043, 2035, 2048, 2042, 2040, 2049, 2043, 2048, 2067, 2049, 2047, 2069, 2054, 2038, 2060, 2062, 2054, 2064, 2067, 2086, 2085, 2062, 2064, 2080, 2087, 2072, 2046, 2070, 2101, 2079, 2072, 2073, 2084, 2076, 2075, 2101, 2090, 2068, 2074, 2098, 2086, 2069, 2088, 2079, 2078, 2084, 2076, 2072, 2079, 2074, 2064, 2060, 2067, 2068, 2066, 2054, 2046, 2040, 2041, 2051, 2042, 2046, 2043, 2047, 2038, 2033, 2041, 2037, 2029, 2048, 2038, 2026, 2031, 2024, 2042, 2047, 2016, 2023, 2038, 2039, 2028, 2025, 2047, 2028, 2004, 2032, 2034, 2008, 2016, 2041, 2024, 1999, 2025, 2022, 2030, 2039, 2025, 2002, 2009, 2017, 2040, 2035, 2017, 2018, 2008, 2008, 2031, 2042, 2031, 2026, 2029, 2039, 2029, 2028, 2028, 2038, 2029, 2038, 2056, 2044, 2021, 2033, 2066, 2052, 2045, 2058, 2041, 2050, 2069, 2053, 2067, 2079, 2066, 2066, 2077, 2070, 2073, 2065, 2067, 2071, 2061, 2055, 2077, 2071, 2062, 2081, 2074, 2061, 2080, 2078, 2066, 2083, 2060, 2071, 2060, 2041, 2042, 2065, 2071, 2072, 2067, 2056, 2061, 2053, 2053, 2055, 2052, 2053, 2061, 2062, 2061, 2062, 2060, 2046, 2046, 2042, 2047, 2053, 2050, 2049, 2046, 2031, 2028, 2064, 2051, 2034, 2036, 2040, 2046, 2035, 2042, 2027, 2037, 2054, 2046, 2037, 2035, 2043, 2046, 2047, 2025, 2048, 2047, 2025, 2037, 2055, 2058, 2041, 2037, 2052, 2052, 2039, 2046, 2038, 2044, 2051, 2033, 2027, 2039, 2038, 2044, 2032, 2034, 2045, 2046, 2051, 2046, 2041, 2029, 2037, 2046, 2042, 2043, 2047, 2056, 2041, 2025, 2045, 2057, 2049, 2050, 2053, 2047, 2049, 2045, 2053, 2061, 2061, 2054, 2060, 2061, 2056, 2049, 2046, 2056, 2066, 2058, 2056, 2062, 2066, 2071, 2055, 2051, 2052, 2049, 2051, 2059, 2056, 2048, 2051, 2050, 2043, 2051, 2044, 2055, 2069, 2042, 2037, 2053, 2052, 2041, 2044, 2042, 2037, 2046, 2057, 2046, 2049, 2065, 2044, 2031, 2045, 2049, 2025, 2030, 2060, 2056, 2044, 2027, 2039, 2041, 2021, 2034, 2046, 2052, 2037, 2044, 2038, 2029, 2033, 2040, 2042, 2032, 2039, 2036, 2041, 2021, 2019, 2044, 2054, 2038, 2039, 2053, 2037, 2027, 2036, 2052, 2051, 2024, 2037, 2057, 2068, 2062, 2046, 2051, 2051, 2049, 2052, 2067, 2062, 2062, 2061, 2045, 2062, 2078, 2066, 2053, 2057, 2068, 2075, 2072, 2072, 2079, 2066, 2063, 2055, 2059, 2062, 2049, 2063, 2065, 2054, 2057, 2062, 2059, 2050, 2063, 2069, 2048, 2046, 2041, 2045, 2066, 2072, 2049, 2038, 2058, 2066, 2054, 2047, 2059, 2059, 2030, 2050, 2068, 2053, 2064, 2050, 2043, 2053, 2057, 2050, 2055, 2053, 2052, 2054, 2036, 2020, 2035, 2040, 2038, 2035, 2030, 2032, 2022, 2033, 2041, 2035, 2037, 2032, 2024, 2022, 2017, 2023, 2026, 2011, 2010, 2022, 2023, 2025, 2024, 2025, 2024, 2022, 2021, 2034, 2043, 2028, 2015, 2036, 2048, 2040, 2029, 2039, 2054, 2049, 2043, 2032, 2049, 2064, 2051, 2033, 2046, 2065, 2068, 2046, 2056, 2077, 2051, 2046, 2062, 2054, 2050, 2057, 2052, 2074, 2070, 2046, 2058, 2057, 2055, 2069, 2068, 2054, 2056, 2065, 2062, 2048, 2064, 2073, 2065, 2068, 2058, 2040, 2048, 2077, 2082, 2085, 2063, 2063, 2079, 2071, 2060, 2066, 2085, 2070, 2060, 2065, 2070, 2077, 2077, 2072, 2061, 2058, 2062, 2056, 2051, 2049, 2059, 2068, 2051, 2041, 2031, 2038, 2057, 2057, 2041, 2043, 2052, 2029, 2029, 2045, 2049, 2051, 2030, 2020, 2033, 2038, 2035, 2032, 2027, 2021, 2025, 2031, 2033, 2035, 2024, 2029, 2034, 2023, 2032, 2033, 2026, 2030, 2034, 2015, 2011, 2020, 2020, 2032, 2037, 2023, 2023, 2037, 2028, 2026, 2029, 2030, 2019, 2021, 2022, 2032, 2038, 2038, 2040, 2039, 2032, 2040, 2058, 2054, 2040, 2041, 2053, 2052, 2048, 2057, 2063, 2055, 2054, 2051, 2066, 2063, 2056, 2070, 2067, 2069, 2074, 2066, 2068, 2078, 2085, 2072, 2073, 2071, 2071, 2069, 2072, 2085, 2077, 2069, 2071, 2069, 2080, 2073, 2066, 2081, 2077, 2062, 2055, 2061, 2063, 2067, 2065, 2058, 2057, 2065, 2068, 2062, 2042, 2035, 2058, 2051, 2038, 2056, 2064, 2046, 2041, 2054, 2050, 2043, 2050, 2040, 2033, 2040, 2047, 2053, 2038, 2036, 2034, 2027, 2024, 2031, 2037, 2035, 2036, 2030, 2012, 2013, 2032, 2019, 2006, 2018, 2027, 2022, 2019, 2009, 2012, 2014, 2008, 2005, 2020, 2024, 2015, 2003, 2016, 2031, 2014, 2012, 2020, 2034, 2018, 2018, 2029, 2030, 2024, 2036, 2019, 2023, 2054, 2043, 2026, 2037, 2047, 2052, 2059, 2051, 2059, 2056, 2052, 2049, 2053, 2063, 2068, 2066, 2061, 2066, 2076, 2069, 2059, 2052, 2058, 2071, 2063, 2064, 2079, 2074, 2067, 2071, 2072, 2070, 2064, 2067, 2064, 2077, 2066, 2045, 2056, 2067, 2065, 2081, 2088, 2068, 2067, 2071, 2069, 2066, 2078, 2077, 2067, 2070, 2078, 2080, 2076, 2066, 2069, 2062, 2060, 2074, 2063, 2057, 2079, 2091, 2072, 2050, 2061, 2070, 2057, 2047, 2057, 2065, 2048, 2042, 2045, 2043, 2041, 2034, 2028, 2034, 2037, 2032, 2025, 2030, 2028, 2021, 2028, 2049, 2035, 2009, 2015, 2032, 2024, 2024, 2036, 2029, 2017, 2001, 2013, 2030, 2016, 2014, 2030, 2030, 2022, 2027, 2029, 2028, 2039, 2032, 2026, 2031, 2028, 2017, 2018, 2027, 2045, 2043, 2032, 2032, 2032, 2034, 2039, 2043, 2030, 2036, 2048, 2037, 2033, 2036, 2041, 2051, 2049, 2046, 2054, 2052, 2046, 2052, 2057, 2053, 2058, 2070, 2071, 2062, 2067, 2077, 2071, 2064, 2071, 2078, 2068, 2067, 2083, 2070, 2062, 2083, 2088, 2079, 2065, 2064, 2078, 2078, 2069, 2059, 2058, 2072, 2071, 2071, 2061, 2063, 2067, 2060, 2058, 2056, 2062, 2064, 2063, 2051, 2053, 2057, 2057, 2066, 2063, 2057, 2056, 2038, 2047, 2057, 2050, 2044, 2034, 2042, 2048, 2044, 2041, 2045, 2047, 2038, 2026, 2034, 2052, 2039, 2028, 2034, 2033, 2033, 2035, 2031, 2026, 2024, 2026, 2027, 2031, 2022, 2011, 2023, 2022, 2029, 2028, 2017, 2032, 2043, 2024, 2011, 2021, 2036, 2031, 2026, 2028, 2031, 2031, 2040, 2049, 2038, 2032, 2041, 2049, 2039, 2038, 2056, 2055, 2043, 2041, 2055, 2056, 2048, 2050, 2048, 2051, 2057, 2065, 2049, 2051, 2057, 2055, 2053, 2049, 2055, 2068, 2065, 2053, 2046, 2052, 2060, 2065, 2069, 2057, 2064, 2064, 2051, 2058, 2065, 2049, 2054, 2064, 2051, 2052, 2049, 2054, 2058, 2053, 2055, 2060, 2067, 2067, 2048, 2054, 2060, 2052, 2052, 2053, 2055, 2055, 2058, 2052, 2055, 2046, 2056, 2066, 2056, 2050, 2053, 2068, 2064, 2044, 2046, 2060, 2054, 2046, 2042, 2047, 2052, 2052, 2048, 2043, 2042, 2045, 2034, 2036, 2038, 2034, 2044, 2038, 2023, 2025, 2029, 2037, 2040, 2036, 2033, 2035, 2044, 2036, 2040, 2043, 2038, 2033, 2042, 2043, 2038, 2042, 2043, 2041, 2043, 2049, 2049, 2040, 2039, 2052, 2051, 2043, 2046, 2049, 2041, 2042, 2048, 2056, 2051, 2050, 2052, 2054, 2051, 2056, 2058, 2047, 2047, 2055, 2052, 2040, 2038, 2051, 2057, 2055, 2048, 2045, 2052, 2051, 2052, 2049, 2044, 2050, 2059, 2055, 2046, 2043, 2052, 2049, 2040, 2041, 2058, 2064, 2056, 2057, 2055, 2056, 2057, 2052, 2055, 2054, 2052, 2054, 2059, 2058, 2045, 2047, 2050, 2050, 2059, 2053, 2042, 2048, 2057, 2061, 2051, 2049, 2057, 2058, 2048, 2043, 2053, 2050, 2036, 2036, 2048, 2047, 2040, 2039, 2041, 2038, 2032, 2036, 2050, 2046, 2036, 2045, 2047, 2047, 2048, 2047, 2046, 2048, 2046, 2042, 2045, 2042, 2041, 2046, 2053, 2051, 2051, 2051, 2050, 2050, 2052, 2048, 2053, 2058, 2045, 2053, 2062, 2059, 2058, 2056, 2047, 2048, 2058, 2053, 2052, 2052, 2049, 2044, 2041, 2053, 2053, 2047, 2042, 2045, 2047, 2051, 2050, 2043, 2046, 2050, 2054, 2046, 2037, 2042, 2050, 2052, 2045, 2039, 2051, 2053, 2050, 2047, 2048, 2053, 2042, 2039, 2048, 2058, 2050, 2048, 2057, 2050, 2046, 2053, 2049, 2041, 2044, 2049, 2046, 2043, 2055, 2051, 2042, 2048, 2047, 2041, 2046, 2051, 2043, 2037, 2046, 2049, 2046, 2046, 2045, 2053, 2054, 2050, 2052, 2046, 2041, 2046, 2047, 2040, 2047, 2050, 2047, 2050, 2055, 2050, 2043, 2051, 2054, 2055, 2052, 2047, 2050, 2049, 2047, 2052, 2051, 2047, 2045, 2050, 2057, 2052, 2046, 2051, 2051, 2046, 2055, 2050, 2051, 2051, 2050, 2053, 2056, 2051, 2039, 2041, 2048, 2053, 2059, 2052, 2041, 2045, 2044, 2041, 2042, 2053, 2060, 2053, 2042, 2048, 2049, 2050, 2054, 2053, 2050, 2052, 2056, 2049, 2042, 2043, 2049, 2053, 2054, 2048, 2051, 2051, 2051, 2059, 2053, 2050, 2053, 2055, 2046, 2049, 2054, 2050, 2046, 2045, 2046, 2051, 2049, 2046, 2049, 2041, 2034, 2037, 2042, 2042, 2044, 2042, 2042, 2043, 2045, 2043, 2038, 2047, 2048, 2042, 2040, 2038, 2045, 2047, 2050, 2045, 2043, 2045, 2048, 2043, 2041, 2042, 2039, 2049, 2049, 2048, 2042, 2042, 2049, 2050, 2048, 2046, 2053, 2053, 2055, 2052, 2050, 2047, 2042, 2047, 2053, 2045, 2043, 2051, 2051, 2043, 2039, 2046, 2056, 2053, 2041, 2046, 2051, 2052, 2049, 2050, 2054, 2056, 2052, 2053, 2057, 2055, 2056, 2056, 2055, 2053, 2055, 2058, 2059, 2057, 2053, 2052, 2053, 2054, 2054, 2056, 2057, 2057, 2059, 2057, 2054, 2058, 2060, 2051, 2047, 2050, 2045, 2047, 2051, 2050, 2050, 2049, 2047, 2046, 2046, 2048, 2050, 2048, 2048, 2045, 2044, 2050, 2046, 2043, 2044, 2043, 2045, 2045, 2046, 2047, 2048, 2044, 2045, 2044, 2045, 2051, 2049, 2044, 2048, 2053, 2047, 2044, 2046, 2050, 2047, 2046, 2045, 2047, 2043, 2041, 2047, 2048, 2048, 2052, 2045, 2038, 2045, 2044, 2044, 2043, 2039, 2043, 2041, 2040, 2043, 2041, 2039, 2041, 2043, 2042, 2039, 2038, 2041, 2043, 2051, 2049, 2041, 2037, 2042, 2045, 2044, 2044, 2048, 2048, 2049, 2048, 2046, 2045, 2049, 2054, 2052, 2047, 2045, 2051, 2052, 2048, 2048, 2053, 2050, 2046, 2048, 2054, 2052, 2052, 2046, 2047, 2053, 2053, 2047, 2047, 2049, 2052, 2054, 2049, 2044, 2043, 2047, 2050, 2052, 2054, 2056, 2055, 2048, 2045, 2049, 2048, 2050, 2057, 2056, 2048, 2049, 2053, 2050, 2050, 2054, 2054, 2052, 2052, 2055, 2050, 2047, 2054, 2055, 2051, 2045, 2047, 2051, 2050, 2051, 2047, 2045, 2049, 2050, 2047, 2047, 2049, 2048, 2043, 2039, 2041, 2047, 2045, 2040, 2038, 2042, 2045, 2044, 2041, 2043, 2048, 2043, 2041, 2043, 2046, 2045, 2044, 2043, 2042, 2042, 2043, 2042, 2040, 2047, 2048, 2043, 2045, 2046, 2043, 2042, 2044, 2044, 2042, 2040, 2044, 2042, 2040, 2046, 2043, 2040, 2043, 2044, 2048, 2045, 2042, 2048, 2047, 2044, 2043, 2043, 2044, 2046, 2047, 2047, 2049, 2050, 2048, 2048, 2046, 2047, 2050, 2055, 2054, 2050, 2049, 2050, 2054, 2051, 2050, 2053, 2051, 2055, 2055, 2052, 2052, 2056, 2056, 2050, 2046, 2054, 2058, 2052, 2052, 2056, 2055, 2051, 2052, 2052, 2055, 2051, 2051, 2058, 2052, 2049, 2055, 2053, 2048, 2051, 2052, 2051, 2048, 2048, 2048, 2046, 2050, 2050, 2047, 2048, 2051, 2048, 2048, 2050, 2050, 2049, 2050, 2049, 2047};

const int16_t HAT_CL[5312] = {2145, 1943, 2250, 1325, 2612, 2892, 889, 2530, 1921, 2540, 880, 2961, 2109, 1305, 2310, 2125, 2699, 1154, 2234, 2590, 1109, 2982, 1282, 2136, 2458, 1974, 2638, 730, 2991, 1665, 1490, 2959, 1396, 2554, 2025, 2221, 2051, 1471, 3413, 950, 2304, 1382, 2566, 2223, 1864, 2265, 1797, 2103, 1437, 2647, 2260, 1267, 2311, 2608, 1840, 2395, 1497, 1679, 1609, 3024, 1919, 1877, 2566, 1864, 1777, 2265, 2594, 1129, 2395, 2388, 1309, 3320, 1525, 2307, 1883, 1228, 2855, 1859, 2047, 1917, 2348, 1890, 2162, 1993, 2300, 1359, 2996, 1043, 2555, 2073, 1816, 2381, 2153, 2347, 1177, 2156, 1938, 3115, 1276, 2294, 1645, 2468, 2103, 1370, 2336, 2259, 1133, 2263, 2478, 1489, 2342, 2069, 1777, 1713, 2424, 1948, 2187, 2165, 1434, 2757, 1120, 2766, 2106, 1917, 1950, 1782, 2400, 1837, 2658, 1130, 2543, 1938, 2101, 2047, 2157, 1799, 1600, 2462, 2226, 1834, 1928, 2188, 2486, 1863, 1813, 2579, 1604, 2501, 1939, 2313, 1817, 2047, 2928, 1474, 2310, 2127, 1885, 2428, 2098, 1638, 2292, 2278, 1825, 2231, 2146, 2045, 1725, 2112, 2323, 1942, 1700, 2265, 2321, 1620, 2550, 1907, 2027, 1573, 2183, 2122, 2401, 1639, 2008, 2234, 2340, 1373, 2049, 2328, 2105, 1590, 1949, 2689, 1456, 2355, 1934, 2214, 1341, 2516, 2277, 1730, 1935, 2338, 1732, 1918, 2549, 1941, 1794, 2125, 1911, 2522, 1633, 1996, 2318, 1729, 2501, 1776, 1967, 1978, 2393, 1620, 1890, 2407, 1740, 2073, 2316, 1957, 1993, 1891, 2146, 2058, 1790, 2045, 2089, 2196, 2030, 2233, 1845, 2071, 1955, 2190, 1912, 2192, 2053, 1933, 1932, 2415, 1892, 2004, 2231, 2050, 1745, 2355, 2108, 1930, 2212, 1928, 2183, 1926, 2387, 2145, 1439, 2349, 2135, 2187, 2019, 2003, 2464, 1276, 2643, 1872, 1976, 2139, 1695, 2654, 1767, 2316, 2040, 1541, 2355, 1953, 1778, 1996, 2613, 1608, 2480, 1706, 2270, 1823, 1949, 2324, 1689, 2326, 1662, 2260, 2154, 1872, 2327, 1687, 2222, 2269, 1606, 2342, 2090, 1983, 2070, 2143, 1865, 1788, 2768, 1691, 1806, 2416, 1785, 2341, 1727, 2324, 1680, 2276, 2026, 2272, 1906, 2141, 1955, 1919, 2320, 1695, 2024, 2373, 2167, 1578, 2508, 1888, 1899, 2097, 1955, 2037, 2189, 1934, 2126, 1870, 2033, 2123, 2210, 1560, 2490, 1801, 1888, 2283, 1844, 2413, 1446, 2483, 1755, 2196, 2118, 1747, 2263, 1700, 2110, 2286, 1747, 2346, 1886, 1769, 2169, 2221, 2214, 1914, 1912, 2229, 2259, 1919, 1725, 2626, 1744, 2010, 2376, 1979, 2031, 2126, 1985, 2072, 2071, 1672, 2790, 1497, 2261, 2234, 2198, 1897, 1712, 2372, 2332, 1731, 1881, 2066, 2207, 2213, 2017, 2184, 1896, 2236, 1725, 2099, 2170, 1756, 2733, 1828, 1764, 2651, 1309, 2504, 1700, 2067, 1851, 2461, 2013, 1853, 2257, 1571, 2410, 2028, 1678, 2132, 2335, 1884, 1661, 2323, 2125, 2011, 2038, 1973, 2032, 2006, 2386, 1864, 2046, 1899, 2566, 1684, 2335, 1737, 2350, 1934, 2187, 1952, 1983, 2181, 2152, 2161, 1680, 2190, 2011, 2226, 1784, 2382, 1577, 2323, 2190, 1891, 2154, 1779, 1976, 2208, 1835, 2300, 1903, 1913, 2216, 2110, 1761, 2008, 2143, 1801, 2297, 2131, 1934, 2093, 1946, 1974, 2081, 2148, 1876, 1867, 2411, 2195, 1944, 1890, 1922, 1492, 2892, 1550, 1838, 2558, 2392, 1716, 2536, 1578, 953, 2646, 2617, 2128, 1754, 2686, 1324, 1968, 2831, 1993, 876, 3323, 1946, 1632, 2654, 1732, 2431, 1952, 1677, 1899, 1645, 3112, 2050, 1693, 1909, 1983, 2428, 1574, 2864, 1381, 1798, 2309, 2260, 1961, 2372, 2025, 2325, 1502, 1569, 2734, 2420, 1404, 2270, 2450, 1193, 2448, 1616, 2298, 2233, 1461, 2363, 1529, 1886, 2988, 1478, 1368, 2418, 2069, 1679, 2688, 2126, 1435, 2239, 1767, 2156, 2236, 1967, 2116, 1565, 2315, 2057, 2033, 1756, 2133, 1886, 1995, 2413, 1922, 1998, 1720, 2364, 1792, 2365, 1564, 2578, 2255, 1741, 1987, 2373, 2406, 1808, 2440, 2107, 2093, 2264, 2428, 1486, 2115, 2149, 2738, 1826, 1950, 2283, 2048, 2107, 2012, 2208, 1812, 2603, 1857, 1876, 2229, 2009, 2038, 1939, 2279, 1847, 2047, 1941, 2207, 1858, 1839, 2115, 2011, 2018, 2219, 1991, 1928, 1802, 1952, 2156, 1981, 2003, 1757, 2121, 2017, 2262, 2184, 1466, 2351, 2275, 1925, 2001, 1985, 2396, 1590, 2316, 1846, 2304, 1948, 2069, 2057, 2101, 1811, 2322, 1904, 1742, 2378, 1621, 2129, 2385, 2060, 1495, 2267, 2049, 1802, 2142, 2355, 1576, 2388, 1930, 2164, 1781, 2013, 2242, 1922, 2194, 2231, 2085, 1673, 2464, 1785, 1967, 1827, 2401, 2018, 2138, 1993, 2027, 1914, 2411, 1865, 2197, 1971, 1792, 2459, 1714, 2431, 1702, 2337, 1946, 2213, 2233, 1729, 2180, 1507, 2544, 2512, 1814, 1717, 2131, 2145, 1989, 2149, 1803, 2327, 2047, 1873, 2182, 2301, 1996, 1780, 1871, 2059, 1925, 2395, 1837, 2307, 1926, 1844, 2116, 1992, 2218, 2089, 1815, 2058, 2098, 2147, 2023, 1955, 2131, 1999, 2160, 1959, 2048, 2205, 1800, 2004, 2448, 1745, 2206, 2056, 2029, 2182, 1786, 2167, 2098, 1924, 2376, 1761, 2118, 1943, 2138, 2204, 1664, 2487, 1800, 1976, 2458, 1710, 2065, 2073, 2044, 2231, 1777, 2021, 2157, 1874, 2410, 1864, 1916, 2114, 2063, 1937, 2123, 2162, 1588, 2415, 1825, 1955, 2104, 1926, 2088, 1858, 2029, 2143, 2131, 1898, 2167, 1756, 1916, 2124, 1848, 2195, 2418, 1952, 1519, 2060, 2362, 2426, 1573, 2043, 2315, 2093, 2085, 1934, 2150, 1965, 2032, 1988, 2144, 2266, 1946, 2088, 2075, 1930, 2065, 2338, 2088, 1562, 2502, 2331, 1711, 1870, 2673, 1973, 1760, 2036, 2054, 2303, 1994, 2194, 2171, 2062, 1951, 2266, 1785, 2193, 1994, 2238, 1849, 1889, 2361, 1945, 1989, 1826, 2114, 2214, 1854, 2327, 1931, 1679, 2265, 2174, 2280, 1447, 2013, 2430, 1929, 1726, 2184, 2268, 2020, 1887, 2088, 1845, 2185, 2211, 2000, 1923, 1990, 2250, 2206, 1952, 1751, 2095, 2366, 2108, 1711, 2288, 2126, 2011, 2056, 2072, 2078, 2037, 2145, 2092, 1842, 2017, 2093, 2429, 1812, 1881, 2046, 2152, 2027, 1756, 2225, 2123, 1938, 1939, 2295, 1855, 2052, 1845, 1999, 2144, 2112, 2157, 1846, 2066, 1928, 2128, 2057, 1991, 1962, 2251, 2096, 1805, 2090, 2164, 2009, 1954, 1952, 2200, 2204, 1981, 2098, 1830, 2008, 1998, 2247, 2219, 2010, 1932, 1921, 2176, 2145, 1898, 2143, 1888, 2073, 2155, 2090, 1962, 1830, 2398, 1880, 1990, 2146, 2140, 1948, 2149, 1940, 1745, 2130, 2294, 1963, 1897, 2195, 1947, 2254, 1927, 2026, 2129, 1929, 2204, 2125, 2104, 2125, 1920, 2336, 1968, 1953, 2038, 2248, 1995, 2054, 2312, 1964, 2086, 1926, 2075, 2245, 1951, 2119, 2076, 2061, 1914, 2170, 2257, 1747, 2124, 2145, 2048, 1855, 1996, 2089, 2201, 1768, 2038, 2066, 2103, 1842, 2052, 2115, 1840, 2058, 2186, 1973, 1808, 2125, 1912, 2145, 1934, 2081, 1990, 2025, 2068, 2072, 2027, 2095, 2031, 1959, 2221, 1901, 2143, 2055, 2049, 2107, 2023, 2003, 2081, 2351, 1849, 2091, 2176, 1931, 2095, 2124, 2069, 2050, 2037, 2051, 2175, 1705, 2249, 2034, 1854, 2067, 2018, 2140, 1945, 2130, 1889, 2106, 1974, 2098, 2034, 1948, 1982, 2118, 2069, 1982, 1967, 1967, 2384, 2018, 1968, 2144, 1992, 2004, 2220, 2090, 2026, 1993, 2171, 2091, 2001, 2063, 2094, 2112, 2125, 2097, 1971, 2104, 2060, 2065, 2102, 1738, 2253, 2334, 1871, 2065, 2082, 1981, 2071, 2211, 1867, 2057, 2167, 2052, 2010, 2090, 2085, 1964, 2224, 2160, 1890, 1880, 2057, 2100, 2025, 1992, 1990, 2135, 2094, 1963, 2109, 1899, 1924, 2098, 2177, 1952, 1860, 2310, 1852, 2006, 2029, 1926, 2169, 1901, 2047, 2100, 2034, 1806, 2132, 2115, 1908, 1956, 2095, 2029, 1906, 2004, 1992, 2108, 2014, 1903, 2095, 2243, 1981, 2095, 1886, 2064, 2171, 1842, 2056, 2212, 2132, 1984, 2024, 2034, 2007, 2088, 2127, 2069, 1930, 2182, 2048, 2050, 2160, 2007, 2026, 1972, 2232, 2091, 1902, 2115, 2234, 1837, 2117, 2242, 1932, 2132, 2012, 1989, 2078, 1901, 2249, 2118, 1959, 2004, 1988, 2101, 1968, 2153, 2030, 2010, 2066, 2099, 1985, 2149, 2099, 2088, 1929, 2044, 2266, 1928, 2183, 2103, 1897, 2170, 2043, 2124, 2090, 1972, 2003, 1970, 2134, 2026, 2052, 1961, 2083, 2112, 1960, 1956, 2140, 2042, 1972, 1943, 2002, 2272, 1805, 2048, 2090, 2028, 2050, 1880, 2142, 1986, 2048, 2012, 1942, 2086, 1962, 2087, 1911, 2025, 2031, 1961, 1963, 2183, 2099, 1824, 2099, 2021, 2158, 2075, 2037, 2061, 1981, 1990, 2238, 1974, 2146, 2062, 1964, 2151, 2095, 2038, 1996, 2199, 2205, 1851, 2157, 2167, 2019, 2088, 2051, 2135, 2069, 2066, 2063, 2241, 1968, 1979, 2131, 2213, 1990, 2055, 2009, 2039, 2102, 2007, 2072, 2024, 1988, 2047, 2035, 1877, 2080, 2162, 2061, 1884, 1994, 2156, 1938, 2051, 2081, 1970, 2069, 2034, 1919, 2023, 2153, 1931, 2102, 2149, 1958, 2088, 2048, 1943, 2000, 2180, 2085, 1995, 2054, 1983, 2081, 2197, 2033, 1956, 1997, 2098, 2073, 2092, 2120, 2002, 1977, 2083, 2157, 1993, 2064, 2118, 1998, 1998, 2113, 2134, 2021, 2020, 2124, 2054, 2010, 2038, 1940, 2217, 2014, 2062, 2085, 2007, 2047, 2088, 2020, 1960, 2076, 2106, 2108, 1850, 2115, 2066, 2029, 2067, 1988, 2100, 2024, 2011, 2167, 2015, 1924, 2109, 2089, 2026, 2077, 1999, 2048, 2003, 2037, 2066, 2100, 2009, 2045, 2111, 1995, 2028, 1969, 2017, 2039, 2094, 1988, 1975, 2098, 2160, 1988, 1940, 1985, 2135, 2058, 2007, 2107, 2090, 2122, 1988, 2035, 2070, 2008, 2067, 2142, 2008, 2199, 1990, 2009, 2127, 2114, 2021, 1979, 2143, 2064, 2046, 2067, 2104, 2035, 2132, 1966, 1994, 2191, 2090, 1999, 2062, 2063, 2090, 2054, 1982, 2148, 2026, 1989, 2027, 2050, 2129, 1984, 2123, 2041, 1939, 2035, 2168, 2069, 1932, 1981, 2174, 2091, 1935, 2075, 2053, 2078, 2017, 1948, 2093, 2094, 2072, 2021, 1984, 2040, 2052, 2072, 2103, 2025, 1953, 2123, 2039, 2015, 2001, 2109, 2053, 1969, 2083, 2094, 1993, 2059, 2042, 2053, 2095, 1932, 2062, 2076, 1980, 2110, 1965, 1993, 2031, 2055, 2086, 1967, 1958, 2045, 2150, 2023, 1986, 1986, 2082, 2113, 1984, 2132, 2024, 1968, 2214, 1971, 2087, 2049, 2029, 2122, 1987, 2037, 2162, 2057, 2077, 2020, 2031, 2169, 2011, 1932, 2042, 2110, 2104, 2035, 1994, 2135, 2043, 2022, 2064, 2114, 1941, 2155, 2119, 1957, 2169, 1992, 2004, 2105, 2107, 2042, 1991, 2048, 2050, 2008, 2035, 2099, 2010, 2002, 1995, 2060, 2081, 2001, 2067, 2017, 1976, 2055, 2173, 1982, 2024, 2058, 2040, 2036, 2051, 2037, 2054, 2104, 1978, 2099, 1987, 2053, 2048, 2033, 2076, 2002, 2041, 1978, 2022, 2081, 2034, 1975, 2086, 2080, 1986, 2044, 2048, 2018, 2123, 2027, 2045, 2172, 1967, 2056, 2083, 2029, 2044, 2062, 1989, 2024, 2102, 2071, 2090, 1967, 2126, 2078, 1929, 2028, 2121, 2055, 1962, 2144, 2026, 2023, 2028, 2070, 2083, 1979, 2076, 2105, 2051, 1941, 2012, 2043, 2063, 2100, 2019, 1969, 2075, 2046, 2094, 2069, 1921, 2049, 2148, 2016, 2044, 2070, 2027, 2084, 2135, 2002, 2040, 2115, 2029, 2086, 2090, 2045, 2013, 2075, 2023, 2058, 2046, 2086, 2050, 2040, 2056, 2036, 2081, 1981, 2107, 2038, 2102, 2040, 2019, 2061, 2010, 2059, 2049, 2012, 2072, 2075, 2003, 2047, 2023, 1988, 2050, 2019, 2003, 2083, 2031, 1999, 2058, 2053, 1983, 2072, 2049, 1966, 2070, 2109, 2017, 2099, 2022, 2060, 2051, 2029, 2048, 2021, 2129, 2040, 1979, 2151, 2054, 1965, 2038, 2160, 2026, 1990, 2135, 2066, 2101, 2016, 2031, 2077, 2128, 2020, 2060, 2100, 2090, 2054, 2038, 2063, 2062, 2074, 2015, 2079, 2015, 2036, 2043, 2036, 2012, 2015, 2058, 1993, 2007, 2065, 2067, 2039, 2019, 2033, 2033, 2028, 2054, 2090, 2048, 2006, 2038, 2043, 2094, 2064, 1992, 2053, 2100, 2051, 2065, 2039, 1968, 2070, 2082, 2030, 1979, 2064, 2106, 2058, 2057, 2009, 2045, 2043, 2101, 2061, 2072, 2052, 2070, 2081, 1981, 2070, 2149, 2059, 2000, 2097, 2029, 2054, 2125, 2051, 2049, 2039, 2036, 2032, 2088, 2120, 2023, 2042, 2064, 2073, 2099, 2048, 2029, 2029, 2081, 2044, 2076, 2021, 2018, 2072, 2011, 2091, 2006, 2008, 2081, 2054, 2078, 2056, 1952, 1994, 2116, 2086, 2048, 2005, 2065, 2068, 2030, 2089, 2017, 2034, 2065, 2112, 2034, 2017, 2101, 2003, 1986, 2041, 2055, 1997, 2090, 2037, 2065, 2027, 1971, 2104, 2097, 1992, 1994, 2058, 2048, 2114, 2023, 2037, 2036, 2030, 2117, 2016, 2036, 2050, 2042, 2091, 2066, 2022, 2024, 2088, 2035, 2052, 2068, 2025, 2020, 2136, 2039, 2019, 2061, 2052, 2112, 2076, 2012, 2042, 2090, 2043, 2102, 2089, 2008, 1973, 2120, 2114, 1990, 2077, 2030, 2037, 2045, 2070, 2090, 1986, 2027, 2064, 2060, 2037, 2060, 2024, 2046, 2086, 2042, 2028, 2071, 2032, 2029, 2004, 2111, 2135, 2020, 2020, 1987, 2058, 2065, 2057, 2033, 2002, 2034, 2083, 2057, 2082, 2085, 1979, 2033, 2073, 2082, 2084, 2021, 2008, 2042, 2073, 2053, 2050, 2013, 2011, 2089, 2043, 2046, 2077, 2082, 2032, 2017, 2047, 2050, 2064, 2067, 2046, 2044, 2063, 2050, 2095, 2010, 2074, 2085, 2022, 2071, 2078, 2091, 2009, 2067, 2051, 2037, 2061, 2081, 2013, 1967, 2071, 2057, 2029, 2034, 2092, 2031, 2032, 2069, 2032, 2043, 2053, 2031, 2036, 2063, 2050, 2028, 2048, 2046, 2036, 2014, 2058, 2072, 1999, 2017, 2064, 2045, 2029, 2020, 1986, 2026, 2099, 2056, 2028, 2058, 1989, 2084, 2087, 2010, 2087, 2056, 2015, 2070, 2085, 2023, 2102, 2071, 2028, 2077, 2023, 2060, 2065, 2057, 2037, 2057, 2083, 2040, 1999, 2093, 2091, 1982, 2056, 2044, 2080, 2078, 2046, 1990, 2096, 2097, 2019, 2079, 2036, 2045, 2050, 2057, 2021, 2067, 2066, 2015, 2033, 2056, 2079, 2036, 2045, 2022, 2024, 2077, 2026, 2021, 2055, 2056, 2030, 2071, 2026, 2061, 2069, 2006, 2038, 2055, 2030, 2001, 2042, 2058, 2004, 2061, 2055, 1988, 2043, 2077, 2024, 2045, 2054, 2013, 2036, 2050, 2026, 2063, 2051, 2036, 2059, 2016, 2064, 2060, 2022, 2069, 2053, 2009, 2064, 2062, 2037, 2073, 2036, 2017, 2084, 2087, 2015, 2049, 2039, 2042, 2154, 2080, 2005, 2039, 2067, 2077, 2079, 2009, 2075, 2059, 2028, 2084, 2060, 2061, 2040, 2019, 2037, 2031, 2033, 2096, 2068, 2010, 2070, 2046, 2056, 2063, 2043, 2060, 2052, 2056, 2080, 2078, 2024, 2035, 2093, 2066, 2032, 2031, 2058, 2063, 2017, 2096, 2060, 2000, 2028, 2070, 2021, 2046, 2064, 2000, 2043, 2091, 2043, 2038, 2035, 2041, 2056, 2032, 2073, 2050, 2057, 2068, 2044, 2031, 2036, 2052, 2053, 2030, 2035, 2028, 2033, 2077, 2059, 2034, 2021, 2014, 2050, 2064, 2035, 2037, 2038, 2074, 2039, 2062, 2049, 2050, 2100, 2023, 2054, 2081, 2066, 2052, 2065, 2093, 2031, 2058, 2105, 2046, 2036, 2047, 2057, 2053, 2112, 2062, 1999, 2025, 2049, 2083, 2044, 2034, 2065, 2031, 2048, 2068, 2010, 2051, 2085, 2016, 2038, 2073, 2014, 2048, 2057, 2046, 2046, 2037, 2053, 2051, 2032, 2025, 2074, 2040, 2071, 2072, 2023, 2051, 2054, 2064, 2093, 2032, 2004, 2081, 2070, 2054, 2036, 2061, 2049, 2052, 2047, 2044, 2048, 2038, 2090, 2027, 2011, 2093, 2045, 2004, 2079, 2049, 2013, 2096, 2052, 2022, 2048, 2039, 2078, 2075, 2052, 2026, 2037, 2049, 2086, 2038, 2013, 2076, 2073, 2022, 2020, 2044, 2096, 2040, 1995, 2060, 2017, 2084, 2081, 2017, 2032, 2041, 2077, 2037, 2037, 2057, 2037, 2055, 2038, 2040, 2076, 2068, 2041, 2030, 2021, 2078, 2060, 2058, 2046, 2027, 2027, 2063, 2073, 2022, 2042, 2056, 2055, 2056, 2026, 2050, 2056, 2019, 2066, 2046, 2011, 2051, 2057, 2024, 2053, 2029, 2042, 2063, 2036, 2013, 2036, 2057, 2046, 2042, 2031, 2041, 2049, 2032, 2042, 2028, 2047, 2049, 2030, 2074, 2068, 2030, 2018, 2051, 2061, 2058, 2040, 2017, 2042, 2091, 2078, 2014, 2039, 2043, 2054, 2068, 2046, 2047, 2031, 2060, 2054, 2036, 2052, 2073, 2034, 2023, 2053, 2046, 2042, 2098, 2047, 2013, 2072, 2056, 2023, 2064, 2051, 2024, 2056, 2092, 2049, 2016, 2056, 2076, 2053, 2030, 2070, 2047, 2048, 2050, 2064, 2046, 2036, 2049, 2062, 2037, 2039, 2069, 2014, 2042, 2087, 2066, 2033, 2043, 2054, 2050, 2063, 2048, 2022, 2055, 2071, 2055, 2043, 2035, 2041, 2058, 2041, 2031, 2041, 2031, 2067, 2059, 2032, 2025, 2015, 2030, 2046, 2038, 2040, 2028, 2045, 2041, 2012, 2039, 2056, 2052, 2038, 2026, 2045, 2057, 2045, 2040, 2041, 2031, 2053, 2048, 2032, 2055, 2049, 2025, 2054, 2071, 2037, 2034, 2050, 2054, 2059, 2072, 2009, 2035, 2069, 2045, 2042, 2054, 2041, 2032, 2077, 2057, 2013, 2077, 2081, 2026, 2046, 2068, 2060, 2037, 2043, 2063, 2054, 2048, 2064, 2060, 2059, 2056, 2044, 2048, 2085, 2049, 2017, 2071, 2079, 2039, 2033, 2060, 2040, 2035, 2047, 2038, 2062, 2067, 2044, 2034, 2055, 2055, 2034, 2038, 2046, 2055, 2043, 2035, 2062, 2034, 2049, 2046, 2040, 2037, 2045, 2050, 2036, 2038, 2052, 2031, 2027, 2047, 2053, 2048, 2032, 2042, 2043, 2051, 2028, 2046, 2049, 2051, 2062, 2036, 2042, 2041, 2057, 2055, 2024, 2063, 2049, 2023, 2043, 2083, 2043, 2033, 2055, 2045, 2050, 2060, 2037, 2041, 2056, 2048, 2065, 2062, 2063, 2055, 2034, 2031, 2092, 2042, 2042, 2095, 2032, 2023, 2060, 2069, 2061, 2051, 2060, 2043, 2009, 2068, 2106, 2027, 2010, 2062, 2072, 2053, 2043, 2052, 2049, 2033, 2051, 2089, 2036, 2059, 2065, 2032, 2029, 2060, 2052, 2036, 2071, 2040, 2040, 2053, 2043, 2042, 2024, 2016, 2057, 2051, 2081, 2041, 2003, 2047, 2054, 2006, 2046, 2074, 2032, 2016, 2041, 2061, 2037, 2041, 2064, 2043, 2035, 2058, 2032, 2052, 2052, 2035, 2045, 2038, 2047, 2058, 2042, 2031, 2062, 2057, 2052, 2066, 2036, 2033, 2060, 2034, 2035, 2066, 2050, 2043, 2038, 2054, 2053, 2043, 2043, 2053, 2072, 2059, 2057, 2047, 2053, 2067, 2048, 2082, 2047, 2034, 2083, 2045, 2029, 2079, 2075, 2039, 2038, 2063, 2062, 2049, 2035, 2035, 2067, 2042, 2033, 2070, 2061, 2023, 2038, 2050, 2068, 2057, 2021, 2042, 2072, 2040, 2026, 2054, 2060, 2044, 2038, 2044, 2041, 2027, 2054, 2053, 2016, 2034, 2040, 2056, 2063, 2025, 2003, 2058, 2066, 2031, 2051, 2047, 2021, 2048, 2063, 2039, 2045, 2042, 2038, 2046, 2038, 2044, 2053, 2046, 2035, 2045, 2056, 2049, 2028, 2043, 2047, 2031, 2068, 2060, 2022, 2065, 2057, 2031, 2065, 2052, 2047, 2072, 2060, 2032, 2040, 2070, 2070, 2026, 2044, 2073, 2067, 2061, 2032, 2040, 2073, 2054, 2054, 2052, 2047, 2067, 2058, 2032, 2035, 2064, 2053, 2046, 2048, 2047, 2057, 2050, 2054, 2050, 2034, 2036, 2054, 2056, 2050, 2053, 2032, 2037, 2046, 2058, 2049, 2033, 2058, 2044, 2039, 2058, 2048, 2021, 2046, 2060, 2030, 2044, 2040, 2034, 2059, 2052, 2020, 2025, 2049, 2049, 2024, 2033, 2066, 2037, 2036, 2045, 2022, 2019, 2050, 2045, 2028, 2056, 2063, 2046, 2031, 2051, 2058, 2019, 2030, 2068, 2056, 2054, 2043, 2034, 2062, 2059, 2029, 2041, 2050, 2054, 2065, 2037, 2033, 2055, 2047, 2065, 2046, 2031, 2074, 2050, 2038, 2089, 2049, 2040, 2074, 2056, 2061, 2065, 2036, 2060, 2089, 2060, 2044, 2044, 2052, 2048, 2060, 2060, 2041, 2038, 2075, 2037, 2026, 2051, 2050, 2046, 2031, 2036, 2042, 2043, 2046, 2056, 2029, 2019, 2042, 2048, 2035, 2029, 2028, 2044, 2045, 2033, 2030, 2042, 2050, 2031, 2033, 2040, 2036, 2061, 2043, 2015, 2050, 2057, 2036, 2039, 2046, 2041, 2050, 2045, 2046, 2037, 2046, 2061, 2050, 2035, 2032, 2053, 2059, 2043, 2045, 2069, 2055, 2042, 2055, 2040, 2042, 2066, 2048, 2046, 2056, 2062, 2066, 2054, 2052, 2043, 2050, 2071, 2060, 2054, 2055, 2049, 2058, 2049, 2048, 2055, 2051, 2058, 2057, 2054, 2052, 2068, 2072, 2036, 2035, 2058, 2058, 2042, 2056, 2074, 2041, 2048, 2066, 2042, 2049, 2069, 2045, 2049, 2055, 2036, 2053, 2051, 2029, 2027, 2047, 2059, 2029, 2039, 2068, 2033, 2020, 2057, 2045, 2029, 2053, 2037, 2018, 2049, 2046, 2038, 2052, 2025, 2026, 2030, 2043, 2038, 2043, 2062, 2029, 2012, 2055, 2051, 2027, 2029, 2051, 2038, 2027, 2066, 2064, 2030, 2036, 2055, 2048, 2050, 2039, 2053, 2066, 2042, 2049, 2064, 2044, 2053, 2064, 2056, 2049, 2050, 2062, 2073, 2059, 2056, 2056, 2061, 2078, 2058, 2051, 2058, 2057, 2057, 2069, 2069, 2055, 2049, 2055, 2056, 2042, 2046, 2064, 2051, 2047, 2065, 2038, 2042, 2056, 2036, 2035, 2062, 2065, 2050, 2031, 2040, 2052, 2022, 2039, 2064, 2028, 2026, 2058, 2036, 2035, 2060, 2029, 2030, 2050, 2042, 2036, 2029, 2048, 2037, 2028, 2056, 2047, 2012, 2037, 2055, 2026, 2030, 2050, 2040, 2046, 2048, 2022, 2039, 2051, 2044, 2047, 2042, 2050, 2037, 2040, 2069, 2039, 2031, 2062, 2047, 2046, 2065, 2035, 2032, 2074, 2046, 2042, 2067, 2044, 2056, 2059, 2046, 2053, 2047, 2058, 2067, 2053, 2055, 2055, 2042, 2054, 2070, 2055, 2044, 2059, 2047, 2046, 2064, 2055, 2036, 2054, 2055, 2048, 2058, 2062, 2049, 2047, 2061, 2047, 2055, 2052, 2039, 2054, 2058, 2047, 2046, 2058, 2058, 2041, 2059, 2065, 2027, 2037, 2076, 2042, 2029, 2051, 2044, 2044, 2047, 2053, 2045, 2028, 2049, 2049, 2028, 2047, 2052, 2040, 2038, 2039, 2048, 2051, 2041, 2041, 2045, 2045, 2051, 2050, 2050, 2040, 2035, 2058, 2053, 2041, 2051, 2052, 2042, 2057, 2048, 2039, 2050, 2053, 2043, 2051, 2058, 2041, 2039, 2049, 2053, 2051, 2043, 2052, 2064, 2048, 2040, 2055, 2054, 2042, 2051, 2048, 2044, 2052, 2046, 2049, 2057, 2037, 2048, 2066, 2042, 2035, 2065, 2049, 2032, 2056, 2050, 2041, 2048, 2055, 2054, 2027, 2037, 2067, 2042, 2040, 2051, 2038, 2038, 2044, 2048, 2049, 2038, 2032, 2047, 2046, 2043, 2042, 2032, 2040, 2051, 2041, 2038, 2041, 2035, 2053, 2053, 2039, 2044, 2048, 2051, 2048, 2037, 2049, 2054, 2036, 2040, 2052, 2052, 2047, 2047, 2037, 2048, 2058, 2042, 2033, 2051, 2062, 2037, 2033, 2057, 2055, 2040, 2042, 2048, 2052, 2054, 2048, 2050, 2042, 2040, 2053, 2047, 2045, 2052, 2050, 2047, 2043, 2044, 2056, 2050, 2049, 2043, 2050, 2058, 2048, 2039, 2049, 2047, 2038, 2048, 2052, 2051, 2052, 2042, 2051, 2059, 2039, 2051, 2055, 2028, 2049, 2063, 2045, 2048, 2046, 2039, 2042, 2050, 2050, 2056, 2030, 2037, 2066, 2046, 2034, 2046, 2052, 2050, 2038, 2044, 2056, 2038, 2045, 2046, 2037, 2054, 2046, 2035, 2055, 2050, 2045, 2054, 2042, 2048, 2058, 2045, 2047, 2051, 2051, 2055, 2056, 2045, 2045, 2049, 2061, 2048, 2037, 2058, 2047, 2034, 2061, 2066, 2042, 2045, 2053, 2049, 2046, 2048, 2050, 2049, 2048, 2051, 2046, 2053, 2056, 2040, 2045, 2051, 2053, 2050, 2046, 2042, 2052, 2048, 2041, 2055, 2057, 2046, 2046, 2050, 2050, 2057, 2055, 2051, 2056, 2053, 2043, 2050, 2053, 2048, 2059, 2054, 2038, 2045, 2060, 2054, 2040, 2046, 2051, 2060, 2047, 2044, 2050, 2041, 2041, 2055, 2047, 2047, 2054, 2043, 2043, 2053, 2047, 2034, 2043, 2054, 2049, 2044, 2050, 2050, 2044, 2046, 2048, 2042, 2052, 2049, 2052, 2050, 2044, 2049, 2044, 2048, 2058, 2043, 2039, 2057, 2054, 2041, 2055, 2058, 2048, 2049, 2044, 2049, 2056, 2046, 2042, 2061, 2051, 2046, 2060, 2049, 2038, 2055, 2054, 2043, 2045, 2055, 2050, 2043, 2051, 2049, 2041, 2043, 2042, 2042, 2050, 2055, 2044, 2044, 2047, 2052, 2045, 2042, 2052, 2037, 2039, 2058, 2050, 2040, 2046, 2041, 2048, 2051, 2045, 2048, 2043, 2041, 2054, 2053, 2043, 2046, 2043, 2037, 2047, 2051, 2051, 2039, 2049, 2058, 2045, 2040, 2052, 2049, 2049, 2057, 2043, 2047, 2055, 2048, 2048, 2054, 2046, 2050, 2049, 2049, 2055, 2056, 2045, 2048, 2056, 2052, 2045, 2048, 2053, 2057, 2053, 2048, 2058, 2056, 2046, 2045, 2057, 2055, 2043, 2049, 2056, 2055, 2047, 2046, 2058, 2051, 2041, 2050, 2057, 2046, 2044, 2049, 2051, 2048, 2050, 2045, 2047, 2047, 2043, 2051, 2052, 2048, 2046, 2048, 2046, 2046, 2044, 2049, 2045, 2039, 2051, 2053, 2049, 2044, 2047, 2049, 2044, 2049, 2056, 2049, 2042, 2054, 2053, 2043, 2044, 2050, 2047, 2042, 2052, 2048, 2037, 2051, 2053, 2041, 2048, 2048, 2046, 2054, 2045, 2044, 2049, 2048, 2055, 2050, 2046, 2056, 2053, 2045, 2051, 2052, 2050, 2053, 2045, 2050, 2052, 2047, 2050, 2048, 2050, 2057, 2048, 2043, 2050, 2053, 2049, 2047, 2047, 2048, 2053, 2043, 2038, 2052, 2056, 2044, 2046, 2048, 2046, 2053, 2051, 2045, 2047, 2051, 2049, 2049, 2048, 2042, 2047, 2049, 2047, 2051, 2048, 2034, 2042, 2051, 2045, 2042, 2049, 2048, 2040, 2051, 2049, 2037, 2045, 2049, 2046, 2045, 2046, 2049, 2042, 2042, 2045, 2051, 2043, 2043, 2051, 2048, 2045, 2046, 2041, 2048, 2055, 2042, 2046, 2052, 2046, 2045, 2050, 2051, 2044, 2045, 2052, 2049, 2047, 2048, 2048, 2049, 2052, 2050, 2052, 2052, 2044, 2048, 2056, 2047, 2045, 2055, 2046, 2043, 2054, 2052, 2049, 2052, 2048, 2043, 2051, 2052, 2046, 2041, 2050, 2053, 2045, 2047, 2046, 2048, 2045, 2048, 2052, 2046, 2047, 2050, 2041, 2040, 2052, 2044, 2043, 2048, 2049, 2048, 2040, 2042, 2055, 2049, 2033, 2049, 2050, 2041, 2052, 2051, 2039, 2042, 2046, 2042, 2053, 2051, 2038, 2044, 2055, 2046, 2044, 2049, 2044, 2043, 2053, 2059, 2047, 2045, 2051, 2051, 2047, 2047, 2052, 2051, 2046, 2051, 2052, 2049, 2051, 2053, 2049, 2044, 2046, 2054, 2055, 2055, 2048, 2041, 2052, 2055, 2049, 2049, 2050, 2049, 2054, 2048, 2046, 2056, 2052, 2043, 2048, 2053, 2049, 2046, 2050, 2047, 2045, 2047, 2048, 2052, 2046, 2045, 2043, 2043, 2053, 2053, 2040, 2046, 2045, 2042, 2051, 2052, 2044, 2042, 2047, 2049, 2049, 2047, 2046, 2040, 2048, 2051, 2043, 2042, 2048, 2047, 2039, 2038, 2048, 2048, 2044, 2049, 2047, 2037, 2046, 2051, 2042, 2044, 2049, 2045, 2047, 2050, 2043, 2039, 2047, 2055, 2046, 2045, 2051, 2047, 2042, 2050, 2056, 2051, 2045, 2051, 2053, 2046, 2056, 2053, 2037, 2048, 2059, 2047, 2046, 2056, 2054, 2050, 2047, 2046, 2048, 2060, 2061, 2039, 2044, 2061, 2050, 2050, 2053, 2047, 2045, 2052, 2058, 2052, 2043, 2048, 2051, 2054, 2050, 2043, 2044, 2047, 2053, 2061, 2044, 2044, 2060, 2043, 2041, 2058, 2051, 2043, 2053, 2049, 2043, 2053, 2049, 2042, 2048, 2053, 2051, 2052, 2049, 2044, 2049, 2048, 2046, 2049, 2049, 2046, 2044, 2041, 2045, 2047, 2048, 2051, 2049, 2046, 2043, 2049, 2056, 2048, 2041, 2047, 2053, 2045, 2048, 2055, 2043, 2043, 2050, 2050, 2048, 2050, 2050, 2048, 2047, 2050, 2053, 2047, 2044, 2044, 2051, 2055, 2048, 2048, 2050, 2049, 2054, 2049, 2050, 2053, 2045, 2047, 2051, 2053, 2054, 2051, 2043, 2047, 2048, 2053, 2058, 2043, 2045, 2060, 2049, 2041, 2053, 2051, 2047, 2048, 2048, 2049, 2046, 2049, 2054, 2047, 2045, 2052, 2049, 2047, 2054, 2043, 2043, 2055, 2050, 2047, 2056, 2045, 2040, 2052, 2051, 2048, 2047, 2049, 2052, 2051, 2046, 2053, 2050, 2041, 2051, 2050, 2045, 2052, 2050, 2047, 2053, 2045, 2045, 2056, 2051, 2046, 2050, 2049, 2048, 2050, 2050, 2051, 2045, 2048, 2056, 2047, 2044, 2051, 2054, 2052, 2051, 2051, 2058, 2049, 2038, 2050, 2053, 2047, 2051, 2047, 2048, 2052, 2044, 2044, 2050, 2045, 2043, 2049, 2047, 2049, 2053, 2041, 2040, 2048, 2043, 2046, 2049, 2040, 2045, 2050, 2043, 2048, 2047, 2040, 2045, 2052, 2046, 2043, 2046, 2047, 2046, 2044, 2046, 2045, 2049, 2050, 2046, 2050, 2045, 2041, 2048, 2046, 2047, 2053, 2046, 2039, 2049, 2049, 2046, 2048, 2047, 2044, 2049, 2051, 2048, 2048, 2050, 2047, 2044, 2053, 2059, 2048, 2041, 2047, 2049, 2046, 2048, 2048, 2048, 2049, 2044, 2046, 2050, 2043, 2046, 2049, 2043, 2046, 2049, 2049, 2052, 2046, 2044, 2049, 2047, 2048, 2050, 2048, 2045, 2044, 2048, 2054, 2044, 2049, 2053, 2043, 2043, 2049, 2047, 2046, 2047, 2045, 2045, 2046, 2049, 2048, 2043, 2045, 2048, 2045, 2052, 2046, 2044, 2050, 2047, 2043, 2045, 2048, 2048, 2045, 2045, 2047, 2047, 2044, 2044, 2046, 2051, 2045, 2040, 2048, 2048, 2048, 2051, 2044, 2037, 2045, 2052, 2046, 2046, 2047, 2043, 2048, 2054, 2047, 2039, 2049, 2054, 2044, 2047, 2051, 2051, 2052, 2046, 2045, 2048, 2051, 2052, 2047, 2046, 2052, 2055, 2050, 2051, 2049, 2043, 2046, 2051, 2046, 2048, 2054, 2051, 2045, 2046, 2051, 2047, 2046, 2053, 2052, 2048, 2050, 2049, 2049, 2057, 2047, 2038, 2052, 2058, 2050, 2050, 2046, 2047, 2052, 2051, 2047, 2045, 2046, 2049, 2051, 2048, 2042, 2045, 2054, 2045, 2044, 2053, 2046, 2044, 2052, 2043, 2045, 2050, 2049, 2048, 2048, 2048, 2045, 2044, 2050, 2052, 2046, 2048, 2051, 2046, 2048, 2052, 2045, 2042, 2049, 2055, 2044, 2041, 2049, 2054, 2049, 2045, 2050, 2052, 2045, 2050, 2054, 2047, 2048, 2052, 2047, 2047, 2051, 2051, 2046, 2045, 2051, 2049, 2045, 2048, 2049, 2048, 2047, 2044, 2049, 2051, 2045, 2045, 2052, 2045, 2045, 2050, 2050, 2047, 2042, 2046, 2049, 2050, 2048, 2043, 2045, 2052, 2048, 2046, 2050, 2046, 2048, 2051, 2044, 2047, 2048, 2048, 2053, 2050, 2044, 2051, 2049, 2046, 2050, 2051, 2049, 2049, 2050, 2051, 2051, 2047, 2049, 2049, 2049, 2048, 2049, 2051, 2048, 2050, 2051, 2046, 2046, 2054, 2049, 2042, 2049, 2053, 2046, 2049, 2050, 2050, 2048, 2045, 2048, 2049, 2049, 2049, 2050, 2047, 2047, 2047, 2048, 2048, 2048, 2047, 2048, 2049, 2052, 2049, 2046, 2047, 2047, 2053, 2054, 2049, 2048, 2046, 2047, 2053, 2047, 2045, 2051, 2044, 2046, 2056, 2050, 2043, 2047, 2046, 2047, 2053, 2049, 2045, 2050, 2047, 2045, 2051, 2049, 2046, 2048, 2052, 2049, 2046, 2049, 2051, 2047, 2047, 2052, 2049, 2049, 2053, 2047, 2044, 2049, 2050, 2048, 2049, 2051, 2049, 2046, 2047, 2051, 2049, 2046, 2048, 2048, 2048, 2051, 2048, 2046, 2047, 2045, 2050, 2050, 2046, 2049, 2049, 2043, 2050, 2051, 2048, 2049, 2044, 2044, 2050, 2049, 2048, 2047, 2044, 2050, 2049, 2046, 2050, 2046, 2047, 2053, 2048, 2046, 2048, 2046, 2047, 2050, 2049, 2047, 2049, 2050, 2048, 2042, 2052, 2055, 2044, 2044, 2051, 2048, 2047, 2051, 2047, 2046, 2049, 2049, 2047, 2048, 2049, 2047, 2044, 2047, 2048, 2049, 2051, 2047, 2044, 2049, 2050, 2045, 2046, 2048, 2046, 2046, 2053, 2052, 2044, 2045, 2048, 2046, 2046, 2049, 2051, 2048, 2046, 2045, 2049, 2048, 2048, 2048, 2044, 2047, 2052, 2049, 2044, 2046, 2048, 2048, 2047, 2046, 2048, 2047, 2048, 2049, 2048, 2047, 2047, 2046, 2047, 2047, 2044, 2050, 2049, 2045, 2046, 2048, 2047, 2050, 2048, 2045, 2047, 2047, 2048, 2050, 2045, 2045, 2048, 2048, 2046, 2046, 2050, 2050, 2046, 2047, 2050, 2046, 2047, 2050, 2047, 2045, 2049, 2052, 2050, 2045, 2047, 2049, 2047, 2048, 2053, 2050, 2048, 2048, 2049, 2050, 2049, 2048, 2048, 2049, 2049, 2051, 2049, 2048, 2048, 2049, 2048, 2047, 2049, 2050, 2049, 2049, 2047, 2048, 2049, 2047, 2047, 2048, 2049, 2051, 2049, 2047, 2047, 2047, 2048, 2046, 2049, 2050, 2046, 2046, 2050, 2048, 2047, 2049, 2046, 2047, 2050, 2048, 2049, 2049, 2049, 2047, 2047, 2049, 2050, 2049, 2047, 2045, 2048, 2052, 2048, 2047, 2048, 2049, 2048, 2047, 2049, 2050, 2048, 2047, 2048, 2047, 2047, 2050, 2048, 2046, 2049, 2048, 2047, 2050, 2049, 2046, 2048, 2048, 2048, 2048, 2047, 2047, 2049, 2048, 2048, 2049, 2049, 2048, 2046, 2047, 2049, 2048, 2048, 2048, 2048, 2047, 2048, 2048, 2051, 2049, 2046, 2048, 2050, 2048, 2047, 2048, 2049, 2051, 2048, 2047, 2049, 2048, 2047, 2049, 2047, 2048, 2049, 2047, 2049, 2048, 2047, 2051, 2049, 2046, 2051, 2049, 2047, 2049, 2049, 2050, 2049, 2049, 2048, 2048, 2048, 2048, 2047, 2050, 2050, 2047, 2048, 2048, 2047, 2048, 2047, 2047, 2049, 2048, 2047, 2048, 2047, 2047, 2048, 2048, 2048, 2046, 2047, 2049, 2048, 2047, 2048, 2045, 2046, 2049, 2048, 2046, 2045, 2048, 2048, 2046, 2046, 2048, 2046, 2043, 2048, 2050, 2047, 2047, 2048, 2047, 2046, 2046, 2048, 2047, 2047, 2048, 2047, 2047, 2048, 2047, 2047, 2046, 2047, 2049, 2048, 2048, 2049, 2048, 2046, 2047, 2049, 2048, 2048, 2050, 2048, 2048, 2049, 2048, 2049, 2050, 2046, 2047, 2049, 2050, 2050, 2050, 2047, 2048, 2051, 2048, 2048, 2050, 2049, 2048, 2049, 2050, 2050, 2049, 2049, 2048, 2049, 2048, 2048, 2048, 2048, 2049, 2049, 2048, 2047, 2048, 2048, 2046, 2047, 2050, 2048, 2048, 2049, 2048, 2047, 2048, 2048, 2048, 2047, 2048, 2049, 2048, 2046, 2049, 2049, 2046, 2047, 2048, 2048, 2049, 2050, 2047, 2045, 2047, 2050, 2048, 2048, 2048, 2048, 2047, 2048, 2048, 2048, 2049, 2048, 2048, 2048, 2048, 2048, 2049, 2047, 2048, 2048, 2049, 2048, 2047, 2047, 2048, 2048, 2048, 2048, 2048, 2049, 2048, 2048, 2049, 2048, 2047, 2049, 2047, 2048, 2048, 2047, 2047, 2049, 2048, 2047, 2048, 2048, 2049, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2049, 2048, 2048, 2048, 2048, 2049, 2049, 2048, 2049, 2049, 2047, 2048, 2048, 2047, 2048, 2050, 2048, 2047, 2049, 2049, 2048, 2048, 2049, 2048, 2048, 2049, 2049, 2048, 2048, 2048, 2047, 2047, 2049, 2050, 2047, 2046, 2048, 2047, 2047, 2049, 2047, 2047, 2048, 2048, 2048, 2048, 2047, 2049, 2047, 2048, 2050, 2048, 2047, 2048, 2048, 2048, 2048, 2048, 2048, 2047, 2048, 2049, 2047, 2047, 2048, 2048, 2047, 2047, 2047, 2048, 2047, 2047, 2048, 2047, 2048, 2047, 2047, 2048, 2047, 2047, 2048, 2047, 2047, 2048, 2047, 2047, 2047, 2047, 2048, 2048, 2047, 2047, 2048, 2048, 2048, 2048, 2047, 2047, 2047, 2047, 2047, 2047, 2048, 2048, 2048, 2048, 2047, 2048, 2048, 2048, 2049, 2048, 2047, 2049, 2048, 2047, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2049, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2049, 2048, 2047, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2047, 2048, 2048, 2048, 2048, 2047, 2048, 2048, 2048, 2048, 2048, 2047, 2047, 2049, 2049, 2048, 2048, 2048, 2048, 2048, 2049, 2048, 2048, 2049, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2047, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2049, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048};

const int16_t HAT_OP[5500] = {2047, 2041, 2059, 1990, 2386, 1796, 2149, 2033, 2086, 1981, 2107, 1951, 1674, 2353, 1640, 2146, 1990, 1903, 2285, 1565, 2258, 2153, 1975, 2280, 1568, 2076, 2094, 2162, 2250, 1867, 2302, 2359, 1946, 2226, 2543, 2143, 1876, 2139, 2444, 1700, 2267, 1599, 2232, 1972, 1542, 2463, 1734, 2081, 2223, 1484, 2132, 2268, 1547, 2468, 1687, 2335, 1817, 2014, 2146, 1931, 2595, 1802, 2322, 2195, 1656, 2429, 1696, 2321, 1894, 1782, 2449, 1865, 1991, 1930, 1836, 2148, 2052, 1919, 2029, 2391, 1774, 2261, 1884, 2277, 2428, 2005, 1926, 2002, 2392, 1690, 2511, 1836, 2042, 2223, 1843, 2302, 1903, 1658, 2551, 1521, 1572, 2424, 2272, 1641, 2315, 1358, 1960, 2216, 2011, 2101, 1782, 2242, 2218, 2116, 1652, 2470, 1766, 1966, 1995, 2215, 2527, 1338, 2475, 1880, 1795, 2523, 1956, 1876, 2170, 2355, 1672, 2350, 2214, 2046, 1917, 2237, 2131, 2411, 2024, 2049, 2273, 2079, 1913, 2119, 2153, 2017, 2366, 1549, 2180, 2195, 1630, 2118, 1857, 2044, 2140, 1629, 2149, 1873, 1698, 2061, 1978, 1861, 2128, 1896, 2239, 2172, 1771, 2231, 1938, 2425, 1730, 1956, 2330, 1980, 2378, 1857, 2501, 1980, 2227, 1998, 1961, 2460, 1823, 2002, 2173, 2185, 1964, 1980, 2235, 2011, 2277, 1915, 1965, 2243, 1675, 2100, 1760, 2169, 1875, 1922, 2089, 1816, 1930, 2120, 2004, 1673, 2071, 2204, 1586, 2221, 1913, 1980, 2154, 1572, 2680, 1736, 1958, 2357, 1981, 2181, 1987, 2258, 2071, 1938, 2579, 1755, 2225, 2224, 1833, 2327, 2083, 2324, 1869, 2338, 1809, 1787, 2151, 2138, 2029, 1885, 1817, 2223, 2078, 1552, 2208, 1786, 1786, 2089, 1877, 1847, 2155, 1991, 2037, 1918, 2147, 2118, 2168, 2037, 2155, 2094, 1898, 2134, 2109, 2019, 2265, 2156, 2029, 2267, 2107, 2316, 1686, 2264, 2018, 1932, 2644, 1647, 2202, 2201, 1890, 2258, 1948, 2020, 2057, 2373, 1915, 1937, 2168, 2026, 1558, 2182, 2132, 1836, 2406, 1700, 2098, 1884, 2177, 1847, 1933, 2014, 2217, 2093, 1944, 2215, 1735, 2110, 1755, 2506, 1754, 2332, 2312, 1848, 2082, 1982, 2301, 2052, 2181, 2060, 2157, 2424, 2085, 1815, 2237, 1996, 2253, 1847, 1962, 2116, 2078, 1842, 2128, 2186, 1698, 2315, 1919, 1924, 1978, 2007, 2078, 1836, 1491, 2382, 1905, 1963, 1841, 2115, 2217, 1842, 2064, 1933, 2170, 1918, 2369, 1605, 2377, 2126, 1909, 2258, 1953, 2191, 1982, 2038, 2219, 1891, 2127, 2111, 2191, 2041, 2003, 2250, 1755, 2255, 2059, 1957, 2414, 1798, 2080, 1841, 1966, 2008, 2455, 1694, 2103, 2088, 2140, 2465, 1811, 2357, 1810, 2045, 2092, 2138, 1814, 1918, 2096, 1949, 1602, 2051, 2276, 1832, 2177, 1902, 1951, 2166, 2278, 1946, 1805, 2333, 2040, 2193, 2208, 1974, 2561, 2030, 1966, 2128, 2051, 2360, 1817, 2102, 2019, 2166, 2159, 1821, 2128, 1745, 2003, 2215, 1706, 2354, 1704, 2206, 2023, 1765, 1966, 2059, 2149, 2186, 1858, 2305, 2119, 2089, 1961, 1888, 2284, 2326, 1901, 2055, 2451, 1742, 2123, 1818, 2032, 2234, 1863, 2000, 1964, 2012, 2121, 1794, 2140, 1959, 1949, 2262, 2051, 2026, 1993, 2235, 2016, 1758, 2311, 2057, 2336, 1919, 2082, 1631, 2372, 1973, 1631, 2250, 2083, 1835, 2031, 2097, 1943, 2168, 1909, 2220, 2017, 2105, 1929, 2155, 1938, 2184, 2251, 2083, 2309, 2031, 2361, 1978, 2223, 1980, 2120, 1964, 2253, 1917, 1825, 2100, 2115, 1824, 1835, 2038, 2083, 1996, 1843, 2156, 1905, 2195, 2028, 1776, 2005, 1992, 2317, 2277, 1808, 2312, 2219, 2142, 2089, 2455, 1691, 2293, 2194, 1865, 2301, 1879, 1962, 2069, 1904, 1994, 1921, 2216, 1684, 2129, 2041, 2147, 1810, 2189, 2050, 2214, 2297, 2103, 1892, 2277, 2498, 1489, 2309, 2077, 2423, 1847, 2223, 2022, 2070, 2029, 1909, 2076, 1661, 2090, 1918, 1722, 1849, 2102, 1826, 1862, 2093, 1939, 1976, 2202, 2028, 2003, 1910, 2559, 2103, 2170, 1978, 1815, 2462, 2297, 2018, 2147, 2072, 1837, 2386, 1554, 2361, 2002, 1942, 2183, 1693, 2153, 2009, 2042, 1968, 1711, 2450, 1818, 2269, 2028, 2021, 1761, 2086, 2176, 1738, 2062, 2270, 2268, 2044, 1739, 1828, 2116, 2166, 1844, 2070, 1954, 2014, 2186, 1913, 2088, 2072, 2141, 1933, 2233, 1995, 2311, 1975, 2359, 1818, 2133, 2315, 2229, 1847, 2244, 2069, 2229, 2166, 1877, 2205, 2029, 1829, 2070, 1996, 2133, 2055, 2002, 2120, 1709, 2041, 2034, 1876, 2122, 1997, 2096, 1855, 2221, 2035, 1687, 2054, 2208, 1833, 2340, 2163, 2180, 2216, 1826, 2122, 2060, 1959, 2029, 2249, 1857, 2302, 1567, 2190, 2017, 2203, 2013, 1909, 2355, 1991, 2139, 2045, 2175, 2000, 2009, 2257, 1950, 2327, 1714, 2323, 1798, 2223, 1706, 2004, 2343, 1797, 2130, 2041, 1968, 2120, 2101, 1941, 1731, 1993, 2190, 1836, 2022, 1981, 2426, 1908, 1833, 2169, 2050, 1754, 2505, 2013, 2057, 2109, 1981, 2221, 2217, 1898, 2132, 2319, 1841, 2207, 1982, 1926, 2104, 1888, 2002, 1996, 2067, 1908, 2123, 2351, 1732, 1925, 2302, 1843, 2116, 1992, 1939, 2328, 1763, 1753, 2363, 2008, 2135, 1857, 2042, 2297, 1942, 1870, 2437, 1943, 1925, 1970, 2214, 1967, 2111, 2001, 1953, 2394, 2016, 1723, 2549, 2044, 1899, 2146, 1916, 2319, 1889, 2063, 2020, 2302, 1877, 2149, 2087, 2232, 2028, 1929, 2073, 1892, 2210, 2152, 1731, 2139, 1782, 2148, 2031, 2156, 1772, 2088, 2005, 1738, 2432, 1681, 2222, 2206, 1786, 2099, 2132, 2034, 2195, 2034, 1902, 1946, 2455, 1701, 2213, 2271, 1992, 1960, 1995, 2357, 1787, 2127, 1895, 2108, 1988, 2290, 1771, 2449, 1797, 2124, 2095, 1957, 2097, 2081, 1988, 2288, 2047, 1689, 2418, 1738, 2316, 1808, 1813, 2142, 2399, 1709, 1713, 2289, 2121, 1952, 2015, 1956, 2241, 1817, 2053, 2070, 1982, 1984, 2005, 1980, 2253, 2010, 2098, 2352, 2218, 1814, 2100, 1783, 2410, 2015, 1813, 2127, 2297, 1933, 2019, 2125, 1785, 2243, 2011, 2048, 1951, 2324, 1802, 2109, 1938, 2084, 2039, 1842, 2180, 2005, 1881, 1982, 2109, 2064, 1967, 2195, 2028, 2110, 1988, 2239, 2103, 2131, 2015, 1795, 2404, 2297, 1895, 2072, 2225, 2050, 1995, 2205, 1802, 2282, 2000, 2084, 1954, 1987, 2194, 1851, 1891, 2030, 2000, 1833, 1957, 2188, 2001, 1773, 2205, 2010, 1964, 1965, 2281, 2148, 1888, 2037, 1880, 2643, 1743, 1989, 2250, 2164, 2009, 2140, 2165, 2090, 2106, 1704, 2326, 2047, 2209, 1862, 2206, 1961, 2147, 2054, 2165, 2054, 2111, 1993, 2115, 2163, 2375, 1858, 2043, 2325, 1769, 2002, 1740, 2313, 1695, 2113, 1683, 2081, 2161, 1496, 2225, 1791, 2120, 1736, 2065, 2103, 2085, 1876, 1866, 2313, 2258, 2003, 2107, 1916, 2485, 1941, 2205, 1891, 2329, 2263, 1928, 2446, 1957, 2408, 2155, 1835, 2107, 2265, 1940, 1876, 2063, 2138, 1827, 2165, 2053, 2161, 1541, 2142, 2145, 2135, 1865, 1973, 2079, 2215, 1899, 2126, 1862, 2173, 1701, 1966, 2008, 2105, 1783, 2057, 2197, 2082, 2016, 1868, 2528, 1870, 2183, 2135, 2153, 1969, 1972, 2288, 2315, 1913, 2122, 2177, 2203, 2108, 2100, 1943, 2267, 1998, 2211, 1649, 2251, 2128, 2085, 1914, 1890, 1988, 1960, 2025, 2092, 1806, 2057, 2165, 1936, 2051, 1837, 2115, 2164, 2095, 1695, 2095, 2364, 1986, 2012, 2085, 2175, 2074, 2040, 2121, 2171, 2099, 2072, 2118, 1956, 2225, 2017, 1832, 1808, 2250, 2101, 2060, 2067, 1987, 2188, 2002, 2093, 2044, 2083, 2183, 2050, 2115, 1984, 1824, 2407, 2073, 1848, 1794, 2132, 2308, 1777, 2072, 1585, 2355, 1567, 2086, 1929, 2064, 2182, 1754, 2007, 1948, 2010, 2132, 1886, 2033, 2118, 2057, 2385, 2165, 2090, 1943, 2539, 2058, 2116, 1990, 2174, 2430, 1780, 2114, 2166, 2141, 2131, 2035, 1889, 2166, 2116, 1888, 2154, 1574, 2028, 1888, 2049, 2093, 1824, 1978, 2376, 1910, 1895, 1974, 1984, 2303, 1592, 2151, 2066, 2082, 2125, 2024, 2285, 1841, 1863, 2501, 1775, 2100, 2101, 1982, 2180, 1863, 2374, 1883, 2311, 1649, 2593, 2138, 2144, 2198, 1984, 2333, 1923, 2269, 2042, 2034, 2056, 1956, 2345, 2121, 1576, 2188, 1895, 1840, 1909, 1967, 2082, 1948, 1847, 1934, 1857, 1917, 2140, 1983, 2058, 2163, 1888, 2092, 2168, 1927, 2061, 1948, 2102, 2272, 1999, 2382, 1948, 2286, 1979, 2259, 1922, 2347, 2057, 2048, 2262, 2151, 2190, 1821, 2270, 1984, 2161, 1959, 2230, 2093, 2074, 1824, 1897, 2031, 2048, 2050, 1804, 2067, 2035, 2038, 1951, 1786, 1913, 1868, 1926, 1840, 2046, 2038, 1807, 2197, 1811, 1908, 2201, 2083, 2105, 2091, 2056, 2076, 2250, 2090, 2251, 1880, 2339, 1967, 2087, 2515, 2126, 1961, 2017, 2162, 2246, 1931, 2153, 2035, 2034, 2172, 2078, 2051, 1990, 2214, 1793, 2095, 2031, 2064, 2127, 1664, 2215, 1994, 1929, 1747, 2025, 2121, 1875, 2042, 1766, 1888, 2078, 1878, 1904, 1848, 1903, 2334, 2127, 2186, 1768, 2304, 2211, 2103, 1870, 2173, 2350, 1986, 2150, 2082, 2393, 2130, 1955, 2321, 1988, 2023, 2103, 2052, 2060, 2044, 1928, 1977, 2070, 2066, 1919, 2004, 2053, 1872, 2024, 1926, 2082, 2135, 1492, 2139, 2015, 2201, 1886, 1919, 2259, 2025, 1933, 1764, 2228, 1871, 2184, 1811, 2158, 2016, 2076, 2070, 1935, 2181, 2071, 2023, 2351, 2114, 1872, 2133, 2283, 2265, 2087, 1810, 2470, 2220, 2100, 2138, 1933, 2327, 1761, 2025, 1986, 2137, 1921, 1990, 2025, 1901, 2128, 1940, 1903, 1855, 2060, 1808, 1889, 2171, 2013, 1845, 2051, 1900, 2160, 1904, 2227, 2257, 1671, 2327, 1998, 2072, 2129, 2002, 2227, 2085, 2215, 2252, 1902, 2084, 2191, 2091, 2014, 1940, 2004, 2251, 2147, 2018, 2045, 2310, 1926, 2154, 2035, 2127, 1995, 2095, 2121, 1936, 2203, 2017, 2035, 1970, 1869, 2134, 1975, 1878, 1930, 2017, 2014, 1963, 1858, 2028, 2194, 1887, 2090, 2023, 1870, 2111, 2254, 2009, 2085, 1957, 2127, 2348, 2167, 1606, 2287, 2257, 1964, 2080, 2148, 2120, 2139, 1992, 2093, 1998, 2330, 2097, 2069, 2043, 2079, 2160, 2102, 2091, 1963, 1932, 1996, 1894, 1996, 2088, 2014, 2032, 1778, 2105, 2036, 1795, 2140, 1995, 1884, 1756, 2135, 2078, 2109, 1999, 1900, 2346, 1917, 2145, 2069, 2303, 1974, 2178, 1745, 2241, 2167, 2287, 1910, 2133, 2200, 1733, 2239, 1994, 2030, 2006, 1932, 2336, 1857, 1787, 2383, 1990, 1884, 2184, 2044, 2002, 2172, 1958, 2115, 1984, 2001, 1848, 2103, 2129, 2081, 2015, 2224, 1995, 1975, 2025, 2085, 2220, 1817, 2214, 2245, 1830, 2082, 2347, 1970, 1984, 1974, 2085, 2194, 2090, 2004, 1901, 1966, 2062, 1971, 1942, 2093, 1981, 2136, 2048, 1989, 2008, 1829, 2201, 1990, 1876, 2004, 1997, 2332, 2051, 2010, 2063, 1765, 2382, 2261, 1945, 2231, 2031, 2012, 2186, 2093, 2040, 2056, 2013, 2316, 1839, 2152, 1905, 2101, 1847, 2071, 1843, 1976, 2093, 2176, 2185, 1788, 2177, 2100, 1973, 2010, 2162, 1882, 1955, 1861, 2317, 1743, 2227, 1737, 1974, 2132, 1868, 1943, 2230, 2049, 1880, 2007, 1994, 2133, 2021, 1770, 2170, 2080, 1885, 2153, 2147, 2176, 1786, 2268, 1896, 2240, 2065, 2260, 2077, 2014, 2273, 1989, 2126, 2063, 2361, 1861, 2232, 2093, 1847, 2188, 1960, 2064, 1853, 2038, 2024, 1959, 2178, 1880, 1939, 1793, 2024, 1939, 1846, 2127, 2047, 2077, 1872, 2301, 1783, 1931, 2093, 2151, 1997, 2155, 1977, 2066, 2084, 2214, 2150, 1810, 2275, 2000, 2145, 2243, 2119, 2095, 1980, 2092, 2074, 2095, 1982, 2297, 2126, 1831, 2044, 2197, 2046, 1922, 2063, 1967, 1993, 2029, 1977, 2142, 2283, 1719, 1958, 2045, 2230, 1903, 2064, 2145, 1932, 1920, 2086, 2005, 2075, 2066, 1953, 2065, 1999, 2006, 2112, 2096, 1992, 1897, 2062, 2071, 2249, 1913, 2112, 2064, 2062, 2032, 2137, 2151, 1852, 2257, 2100, 2041, 2172, 1997, 2066, 2145, 1931, 2075, 2135, 1946, 2061, 2027, 1971, 2165, 1851, 2017, 2026, 1989, 2095, 2034, 1929, 1982, 2145, 1963, 1960, 2012, 2014, 2166, 2239, 1933, 1928, 2267, 1966, 2091, 1903, 2054, 2062, 2150, 1956, 2197, 2008, 2082, 2109, 2094, 2145, 1936, 2054, 2167, 2231, 2035, 1978, 2129, 2065, 1919, 2044, 2204, 2075, 2115, 1860, 1815, 2188, 2054, 2014, 1831, 2150, 1955, 1914, 2135, 2098, 1995, 1869, 1904, 2244, 2194, 1986, 2059, 2183, 2089, 2076, 1939, 2129, 2028, 2264, 2077, 1998, 2082, 1878, 2313, 2084, 2134, 1775, 2202, 2194, 2009, 2104, 1964, 2031, 1949, 2110, 1964, 1964, 2157, 2082, 1942, 2158, 1907, 2147, 1955, 2142, 2005, 2113, 1957, 2084, 1923, 1983, 2215, 1786, 2010, 1968, 1995, 1939, 2262, 1877, 1882, 2093, 2108, 1960, 2203, 2187, 2053, 2062, 1919, 2320, 2135, 2055, 1969, 2142, 2149, 2041, 2213, 2165, 2159, 2051, 1980, 2171, 2178, 1914, 2040, 2128, 2165, 1886, 2051, 1865, 2229, 1947, 1892, 1946, 2082, 1990, 1971, 2007, 1953, 2132, 1849, 1968, 2104, 2090, 1815, 2107, 2058, 2023, 1988, 1983, 2222, 1907, 2107, 2180, 2104, 2125, 2002, 1982, 2204, 2212, 2005, 2090, 2005, 2312, 2083, 1907, 2281, 2119, 2071, 2187, 1911, 2305, 1937, 2023, 1946, 2102, 1941, 1968, 2032, 2137, 1881, 2023, 2189, 1945, 2005, 1774, 2309, 2117, 1856, 2048, 2184, 1943, 2049, 2020, 2152, 2140, 1944, 2169, 2056, 1948, 2140, 2062, 2171, 1997, 1939, 2112, 1933, 2240, 2053, 1924, 2218, 1748, 1965, 2197, 2166, 2006, 1962, 2012, 2310, 1870, 2196, 2053, 2175, 2028, 1775, 2214, 2194, 2064, 1809, 2229, 1726, 2317, 1902, 2066, 2244, 1939, 2086, 1983, 2068, 2053, 2015, 2075, 2241, 1808, 2157, 1781, 2305, 2202, 1904, 2014, 1903, 2330, 2107, 1915, 1953, 2084, 2142, 1979, 1949, 2195, 2055, 2117, 2021, 2045, 1954, 2145, 1954, 2227, 2017, 1826, 2219, 2014, 2262, 1861, 2010, 2281, 1880, 1937, 2188, 2015, 2073, 2066, 1997, 2134, 1883, 2295, 2151, 2113, 1877, 2305, 1926, 2277, 2023, 1634, 2216, 2087, 2247, 1853, 2150, 2133, 2086, 2062, 2032, 2050, 2164, 1827, 2089, 1918, 2015, 1952, 1957, 2103, 1897, 2136, 2036, 1996, 1930, 2159, 1873, 2211, 1962, 1832, 2232, 2046, 2287, 1960, 2036, 2220, 2063, 2064, 2138, 1955, 2370, 1941, 1936, 2375, 1992, 2136, 2092, 1937, 2272, 1902, 1899, 2108, 2058, 1943, 1878, 1995, 2025, 2071, 1999, 2220, 1873, 2065, 1957, 2097, 2165, 1877, 2178, 1930, 2123, 2003, 2195, 1958, 2233, 2020, 1851, 2120, 2092, 2153, 1809, 1982, 2063, 2061, 2045, 2018, 1975, 2289, 1834, 2093, 2143, 1987, 2194, 2017, 2158, 1998, 1775, 2269, 2197, 2024, 2115, 1985, 1991, 2164, 2031, 1891, 1968, 2075, 2006, 1843, 2196, 2031, 2077, 2079, 1887, 2166, 1917, 2151, 2076, 1948, 2074, 2022, 2003, 2105, 2004, 2231, 2127, 2051, 2108, 1807, 2184, 2022, 1980, 1940, 2158, 1951, 2001, 2120, 1999, 2298, 1861, 1871, 2063, 2085, 2139, 2096, 1817, 2189, 2171, 1990, 2037, 2023, 2241, 2123, 2051, 2020, 2187, 1824, 2248, 2041, 1898, 2128, 1941, 2078, 1983, 2108, 1730, 2267, 1926, 1834, 2137, 1997, 2160, 2012, 1927, 2143, 2153, 2029, 2071, 2050, 2148, 2003, 2083, 2070, 2141, 1875, 2018, 2112, 2141, 2115, 1959, 2052, 2201, 2053, 2003, 2088, 1893, 2128, 1993, 1939, 2067, 2103, 2050, 2215, 1925, 1891, 2081, 1980, 2206, 2002, 2043, 1985, 2104, 1923, 2067, 1915, 2046, 1996, 1844, 2039, 2000, 2025, 1871, 2126, 1849, 1899, 2149, 2190, 2036, 2160, 2096, 2068, 2031, 2067, 2093, 2006, 2313, 2038, 2067, 2086, 2217, 2107, 1958, 1989, 2125, 2023, 1938, 2138, 2090, 2015, 1955, 1875, 1955, 2196, 1876, 2202, 1918, 1903, 2077, 1993, 2012, 1792, 2121, 2065, 2075, 1995, 2019, 2027, 2111, 2073, 1871, 1969, 2064, 2208, 2149, 1889, 2193, 2012, 2097, 1970, 2135, 2203, 1822, 2249, 2120, 1873, 2070, 2208, 1859, 2049, 2061, 2173, 2065, 2145, 2041, 2166, 2025, 1831, 2089, 1968, 2082, 1933, 2021, 2136, 1899, 2119, 1910, 2144, 1982, 1910, 2198, 1950, 1962, 1960, 2031, 2085, 1888, 1883, 2121, 2249, 2015, 1888, 2018, 2286, 1832, 1985, 2185, 1883, 2237, 2064, 2051, 2091, 2148, 1987, 2138, 1858, 2127, 2018, 2167, 1988, 1955, 2183, 2031, 2027, 1976, 2099, 2160, 2195, 1844, 2065, 1957, 2098, 2000, 1832, 2145, 2058, 1870, 2167, 2080, 2012, 1977, 1977, 2012, 1868, 2120, 2095, 2015, 1935, 2057, 2100, 2082, 2175, 1842, 2134, 1956, 2188, 2132, 1861, 2353, 2061, 2000, 2036, 2038, 2287, 2176, 1992, 2003, 1976, 2190, 1984, 2116, 1985, 2044, 1984, 2144, 2038, 2097, 1980, 1784, 2241, 1938, 1839, 2019, 2174, 2004, 1882, 2058, 1993, 2116, 2226, 1830, 2162, 2046, 1846, 2212, 1806, 2081, 2133, 2054, 2093, 2128, 1995, 2228, 2133, 2042, 2155, 1956, 2042, 2139, 2156, 2022, 2251, 1989, 2138, 1900, 2008, 2065, 2069, 2049, 1866, 2017, 1963, 2158, 2018, 1914, 2116, 1991, 1910, 1995, 2115, 2056, 2141, 1824, 1946, 2101, 2007, 2280, 1963, 1850, 2073, 2009, 2229, 2018, 2021, 2197, 1882, 2098, 2110, 2227, 1954, 2205, 2076, 1940, 2143, 2090, 2173, 2167, 1972, 1834, 2148, 2174, 2151, 2040, 1958, 2228, 2112, 2007, 2367, 1955, 2016, 1858, 2054, 2036, 2006, 1897, 1944, 2119, 1982, 1828, 2049, 2124, 1806, 1967, 2022, 2202, 2007, 1994, 2139, 2246, 2071, 2003, 2143, 2171, 2321, 1927, 2063, 2202, 2080, 2121, 1961, 2183, 2041, 2008, 1987, 2119, 2179, 1972, 1929, 2060, 1998, 1880, 2073, 2200, 1891, 2073, 2017, 2109, 1848, 2112, 2130, 2163, 1994, 2060, 2010, 2122, 2136, 1726, 2284, 1864, 2127, 1901, 2054, 2186, 1887, 2094, 1832, 1967, 2026, 2179, 2065, 2045, 2047, 2011, 2039, 2332, 2015, 1929, 2355, 2058, 1996, 2089, 2184, 2137, 2058, 1956, 2192, 2003, 2125, 2082, 1964, 1946, 2040, 1838, 1944, 1922, 2137, 1965, 1990, 1975, 2022, 2268, 1912, 2117, 1959, 1930, 2121, 2207, 2021, 2004, 2110, 2040, 2092, 2108, 1947, 2100, 2211, 2056, 2005, 1845, 2225, 2044, 1908, 2194, 1824, 2226, 2054, 2033, 2102, 2042, 2071, 2043, 1975, 2146, 1881, 2214, 2191, 2016, 1957, 1950, 2207, 1970, 2108, 1901, 2056, 1875, 2177, 1824, 2020, 1951, 1948, 2182, 1815, 2270, 1989, 2033, 2039, 2113, 2000, 1963, 2206, 2057, 2353, 2054, 2280, 1997, 2169, 2116, 2002, 2111, 1864, 2217, 2084, 2092, 1880, 2086, 2016, 1982, 2211, 1964, 1935, 2064, 1979, 1928, 1981, 2118, 1910, 2032, 2030, 1970, 2050, 2162, 2038, 1951, 1867, 2142, 2022, 1986, 2230, 1797, 2273, 2080, 1955, 1946, 2203, 2057, 2036, 1990, 2096, 2064, 1857, 2260, 1993, 2109, 2067, 2031, 2186, 2126, 2114, 2145, 2044, 2025, 2053, 2046, 2095, 2117, 2165, 1939, 2076, 2265, 1920, 2006, 2149, 1995, 1993, 2105, 1876, 1977, 2012, 2046, 2040, 1835, 1936, 2040, 1860, 2144, 1942, 1968, 1971, 2079, 1982, 2041, 1973, 2195, 1981, 2050, 2218, 1971, 2189, 2022, 2064, 2042, 2172, 2093, 2188, 2075, 2250, 2038, 2084, 2031, 2042, 2342, 1921, 2053, 2028, 2167, 1937, 2007, 2057, 2015, 2101, 1908, 2001, 2047, 2071, 1906, 2094, 1840, 2057, 1951, 2249, 1940, 1737, 2160, 2082, 2022, 1939, 2041, 2132, 1941, 2183, 1870, 2021, 2231, 2020, 2054, 1969, 2195, 2160, 2055, 1919, 2357, 1889, 2027, 2286, 1879, 2196, 2044, 2137, 2036, 2163, 2044, 2063, 2039, 2037, 2074, 1972, 2096, 2199, 1990, 1887, 2185, 2019, 2303, 1850, 2088, 2029, 1969, 2094, 1909, 2104, 1866, 2134, 2018, 1904, 2100, 1942, 1986, 2133, 2068, 2009, 2067, 1995, 2038, 2083, 2115, 2223, 1990, 1987, 2258, 1956, 2132, 2076, 2107, 2120, 1807, 2190, 2098, 1991, 2110, 2121, 2045, 2043, 2136, 2073, 2222, 1984, 1985, 2220, 2128, 1893, 2086, 2149, 2096, 2003, 1995, 1927, 2114, 2171, 1822, 1827, 2164, 1863, 2298, 1829, 1921, 2166, 2026, 2130, 1894, 2116, 1986, 2141, 1859, 2082, 2001, 2227, 1962, 2106, 2079, 2078, 2276, 1974, 2263, 2046, 1990, 2233, 2130, 2004, 2069, 2285, 1886, 2114, 1935, 2163, 1938, 1993, 2078, 1988, 1956, 1957, 1998, 2074, 2219, 2144, 1846, 1982, 2253, 2086, 2042, 1928, 2265, 2016, 2015, 1954, 2042, 2086, 2084, 2092, 2083, 1889, 2144, 1916, 2223, 1884, 1965, 2007, 1964, 2301, 1819, 2121, 2069, 2209, 1921, 2070, 2126, 2138, 2119, 1987, 2112, 2092, 2138, 1957, 2201, 1927, 2093, 2079, 2171, 1900, 1938, 2070, 2110, 2054, 1825, 2137, 2013, 2096, 1990, 2076, 2096, 1972, 1951, 2135, 1964, 2058, 2016, 2099, 2239, 1920, 2014, 2006, 2182, 2091, 1951, 1967, 1965, 2038, 1892, 2035, 2174, 2029, 2013, 2027, 2211, 1989, 2067, 2153, 2052, 2105, 1945, 2259, 2020, 2185, 2029, 2001, 2154, 2143, 2052, 1982, 2110, 2161, 1890, 1990, 2161, 2021, 2070, 1853, 2163, 1994, 2044, 2017, 2018, 2171, 1940, 2026, 1769, 2177, 1981, 1907, 2108, 2105, 2013, 1918, 2193, 1940, 2056, 1961, 2107, 1969, 2013, 2017, 2094, 2152, 2081, 2058, 2030, 2040, 2183, 2022, 2072, 2108, 2103, 2117, 2011, 2109, 2216, 2092, 2196, 2008, 1952, 2214, 2041, 2084, 2033, 2028, 1959, 1956, 2067, 1978, 1988, 1993, 2109, 1892, 2046, 2116, 1951, 1949, 2084, 2021, 1973, 2114, 1920, 2018, 2007, 2111, 2003, 2116, 1896, 2097, 1928, 2035, 2245, 1967, 1924, 2156, 2147, 1890, 2240, 2023, 2113, 2099, 1902, 2217, 2016, 2163, 1984, 2133, 2130, 2125, 2111, 1950, 2135, 2164, 1847, 2112, 2117, 2003, 2019, 2024, 2078, 1846, 1937, 2187, 2064, 1798, 2124, 2124, 2003, 1889, 2066, 2093, 1970, 1987, 2058, 2118, 2110, 2049, 1878, 2037, 2214, 2030, 1968, 2061, 2037, 1983, 1939, 2142, 2118, 2054, 2174, 2043, 2075, 1975, 2192, 2070, 2026, 2071, 2220, 1959, 2202, 2059, 2089, 2118, 2130, 2076, 2024, 2169, 1936, 1996, 2012, 2127, 1884, 2143, 2068, 1982, 1810, 1985, 2103, 1927, 1952, 1894, 1918, 2094, 2079, 2066, 1934, 2049, 2148, 2005, 2189, 2148, 2100, 1971, 2157, 2044, 1999, 2191, 1937, 2034, 2113, 2070, 2071, 2089, 1993, 2170, 1973, 1998, 2166, 2019, 2029, 2126, 2148, 1968, 2276, 2022, 2036, 1995, 2101, 2139, 2048, 1988, 1919, 2029, 2137, 2029, 1920, 2087, 2124, 2051, 1912, 1996, 1897, 2092, 2185, 2018, 1915, 2062, 2027, 2127, 2172, 1900, 2092, 2026, 2050, 2117, 2024, 2012, 2018, 2055, 2166, 2002, 2025, 2223, 1971, 2078, 1994, 2105, 2046, 1989, 2055, 2071, 2048, 2033, 2098, 2042, 2143, 2132, 2024, 2089, 2030, 1996, 2024, 2164, 2107, 1900, 2110, 2090, 2040, 2121, 1915, 1964, 1983, 1992, 2054, 2021, 2060, 1986, 1947, 2041, 2103, 2068, 2011, 2025, 2099, 2039, 2110, 2023, 2078, 1914, 2196, 2062, 2017, 2017, 2225, 2170, 1976, 2163, 1988, 1930, 2245, 2121, 1823, 2167, 2029, 2051, 2013, 2068, 2067, 2108, 1989, 2098, 2192, 1988, 1936, 2001, 2032, 2079, 2032, 2177, 1920, 2084, 2164, 2021, 2050, 2077, 2040, 1966, 2033, 2094, 2058, 1989, 2063, 2060, 1935, 1958, 2180, 2011, 1902, 2147, 2148, 1810, 2139, 2039, 2028, 2054, 2261, 2022, 2060, 2054, 2051, 2324, 1874, 1941, 2108, 2051, 2148, 1916, 2118, 2050, 2057, 2133, 2078, 2067, 2054, 2179, 1986, 2129, 1953, 2196, 1954, 1986, 2139, 2023, 2128, 1935, 2021, 2025, 2095, 2098, 1958, 1911, 1967, 2137, 2108, 1940, 2047, 1958, 2058, 2190, 1988, 1932, 1970, 2155, 2071, 1908, 2048, 2207, 2078, 1964, 2077, 2213, 1973, 2087, 2044, 2129, 2044, 1939, 2131, 1985, 2175, 2154, 1927, 2132, 2181, 1891, 2176, 2066, 2017, 1959, 1972, 2089, 2025, 2061, 2007, 2201, 1914, 2003, 2098, 2039, 2037, 2048, 1982, 2028, 2067, 1976, 2016, 1987, 2058, 2137, 2060, 1940, 2002, 2057, 2082, 1944, 2034, 2069, 2025, 2114, 2037, 2091, 2080, 2017, 1997, 2070, 2183, 1913, 2082, 2030, 2052, 2158, 1955, 2009, 2031, 2195, 2152, 1973, 2090, 1980, 2017, 2076, 2043, 2029, 2069, 2026, 2060, 2032, 2081, 2016, 1990, 2131, 2044, 1953, 1975, 2096, 1983, 2010, 2042, 2064, 2036, 2043, 2048, 2054, 2002, 1986, 2045, 1942, 1921, 2080, 2076, 2089, 2024, 2026, 2038, 2181, 2046, 2100, 1920, 2126, 2212, 1873, 2104, 2170, 1928, 2030, 2282, 1891, 2041, 2034, 2072, 1996, 2137, 2040, 1902, 1992, 2062, 2192, 1970, 1982, 2005, 2010, 2083, 2075, 1929, 2019, 2030, 2045, 2035, 2088, 1952, 2047, 2120, 2072, 2063, 2039, 2029, 2090, 2167, 1901, 1854, 2071, 2218, 1960, 1944, 2031, 2105, 2067, 1988, 2148, 2157, 1951, 2059, 2117, 1907, 2156, 2141, 2081, 2036, 1959, 2072, 2058, 2111, 1989, 1919, 1963, 2167, 2099, 1957, 1840, 2193, 2042, 2081, 1974, 2000, 2083, 1982, 2186, 1867, 2074, 2022, 2002, 2073, 2047, 2002, 2149, 2045, 2082, 2056, 2084, 1942, 2114, 2052, 1974, 2100, 1987, 1977, 2056, 2121, 2042, 1925, 2013, 2134, 1961, 2071, 2017, 2130, 1995, 2009, 2048, 2159, 2101, 2044, 2151, 1934, 2048, 2109, 2046, 1949, 2080, 2014, 2109, 2171, 1939, 2060, 2077, 2109, 2053, 2029, 1934, 2101, 2010, 2013, 1948, 1996, 2159, 1931, 2033, 1970, 2077, 1971, 2104, 2036, 1924, 2004, 2083, 2075, 2086, 2116, 2096, 2006, 2132, 2167, 1958, 2046, 2033, 2053, 1951, 2033, 2108, 2038, 2080, 2042, 1965, 2145, 1977, 2106, 1998, 2059, 1939, 1981, 2148, 2100, 1968, 1980, 2158, 2024, 2162, 2027, 2059, 2126, 1932, 2061, 2047, 2102, 2006, 1970, 2148, 2076, 2044, 2084, 1991, 2024, 2167, 1890, 2036, 2112, 2082, 1998, 2090, 1988, 2088, 2051, 2053, 2098, 1943, 2066, 2042, 2018, 1953, 2096, 2014, 2036, 2115, 2023, 2064, 2132, 1992, 1949, 2136, 2016, 2021, 2056, 2042, 1987, 2156, 2107, 2058, 2006, 2144, 2064, 2063, 2176, 2047, 2137, 2053, 1953, 2071, 2103, 2046, 2040, 1939, 1988, 2060, 2068, 1870, 2025, 2081, 2034, 1972, 2020, 2077, 2023, 2028, 2045, 2017, 2051, 2172, 1975, 2060, 2107, 2057, 2014, 2081, 2149, 2047, 2001, 2021, 2184, 2005, 2096, 2034, 2007, 2008, 2103, 1996, 2036, 2031, 1983, 2130, 2011, 2081, 1982, 2131, 2038, 2035, 2105, 1999, 2035, 2154, 2082, 1957, 2068, 2061, 2015, 2119, 2004, 2057, 2031, 1938, 2046, 2066, 2052, 1963, 2092, 1965, 2106, 2050, 2014, 2076, 2013, 2034, 2025, 2067, 2088, 2091, 1906, 2072, 2127, 2017, 1956, 2108, 2086, 2005, 2032, 2024, 2088, 1986, 2134, 2072, 2086, 1887, 2163, 2160, 2017, 2070, 1955, 2042, 2073, 2103, 1984, 2094, 2110, 2121, 1935, 2146, 2057, 2115, 2072, 1955, 2037, 2072, 2001, 2067, 2069, 1931, 2100, 2109, 2071, 2027, 2107, 2079, 2045, 2013, 1998, 2032, 2013, 2079, 2024, 2026, 2036, 2035, 2034, 1948, 2101, 2007, 1938, 2110, 2029, 2024, 2096, 2042, 2092, 2085, 2040, 2103, 2100, 1964, 2109, 1976, 2174, 1902, 2033, 2236, 2031, 2107, 1994, 2149, 2092, 2059, 2001, 1988, 2062, 2060, 1967, 2078, 2103, 2040, 2028, 2102, 2044, 2018, 2023, 2117, 2008, 1994, 1994, 2092, 2037, 2061, 1992, 1981, 2100, 2043, 2068, 2021, 1976, 2064, 2069, 2006, 2069, 2071, 2104, 1955, 2043, 2101, 2142, 1967, 2085, 2097, 1999, 2066, 2120, 2007, 2115, 2092, 2029, 2076, 2052, 2113, 2007, 2112, 2044, 1962, 2001, 2200, 1896, 2030, 2083, 2045, 2048, 1964, 2151, 2066, 1996, 1967, 2103, 2036, 2045, 2067, 2146, 2056, 2004, 2099, 2080, 2012, 2073, 2032, 1991, 2030, 2030, 2034, 2032, 2100, 1981, 2044, 2120, 2012, 1969, 2072, 2096, 2056, 2021, 1929, 2222, 2073, 2067, 2087, 2005, 2088, 2083, 2083, 2051, 2044, 2088, 2114, 1990, 2100, 2003, 2127, 2053, 1981, 2019, 2043, 2003, 2061, 2047, 1898, 2082, 2002, 2018, 2089, 2027, 2097, 2069, 2059, 2084, 2141, 2085, 1919, 2100, 2123, 2115, 1904, 2046, 2141, 2037, 2045, 2025, 2117, 2017, 2060, 1998, 2051, 2090, 2065, 1970, 2030, 2085, 2074, 2029, 2120, 2056, 1931, 2187, 2078, 1958, 2019, 2086, 2061, 1961, 2137, 2096, 2072, 2034, 2018, 2109, 2072, 2010, 2035, 2026, 2105, 1953, 2002, 2125, 2043, 2037, 2038, 1996, 2083, 2082, 2016, 2062, 2028, 2019, 2029, 2163, 1979, 2082, 2049, 2152, 2010, 2040, 2107, 2041, 2058, 2047, 2066, 2079, 2123, 2024, 2017, 2016, 2163, 1943, 2091, 2128, 2003, 1986, 2108, 2070, 1978, 2120, 2021, 1999, 2064, 2099, 1972, 2045, 2073, 2075, 1993, 2029, 2039, 2114, 2031, 2064, 1992, 2077, 2084, 2013, 2108, 1943, 2075, 2074, 2010, 2090, 2053, 1973, 2115, 2110, 2037, 2020, 2072, 2083, 2054, 2098, 2034, 2060, 2080, 2070, 2033, 2020, 2191, 2036, 2018, 2029, 2035, 2084, 1955, 2089, 1981, 1910, 2114, 2005, 2081, 2019, 2061, 2034, 1970, 2143, 2037, 2033, 2123, 2077, 2025, 2121, 2051, 2087, 2130, 1963, 2105, 2027, 2037, 2150, 1977, 2004, 2020, 1995, 2059, 1932, 2089, 2050, 1977, 2046, 1956, 2014, 2067, 2118, 2066, 1967, 2085, 2145, 2083, 2054, 2101, 2097, 2052, 2135, 2052, 2048, 2025, 2089, 2053, 1972, 2022, 2083, 1991, 2074, 2002, 2119, 1948, 2033, 2077, 1930, 2158, 1941, 2003, 2074, 2062, 2007, 2126, 2103, 1979, 2083, 2096, 2024, 2113, 2013, 2136, 1957, 2083, 2082, 1997, 2048, 1988, 2127, 1974, 2000, 2031, 2041, 2006, 2023, 2050, 2033, 2037, 2116, 1970, 2095, 2014, 2027, 2123, 2060, 2049, 2036, 2085, 2028, 2074, 2084, 2146, 2131, 1968, 2053, 2051, 2072, 2114, 1974, 2026, 1991, 2117, 2109, 1969, 1969, 2054, 2103, 2009, 1997, 2037, 2089, 1991, 2084, 1924, 2049, 2115, 1976, 2112, 1991, 2060, 1969, 2060, 2078, 2017, 2007, 2092, 2069, 2015, 2121, 2014, 2080, 2039, 2034, 2062, 2052, 2021, 2094, 1999, 2139, 1987, 2022, 2129, 2049, 2042, 2082, 2146, 1994, 2062, 2031, 2123, 2017, 2035, 1982, 2041, 2143, 1959, 2014, 2134, 1918, 2054, 2016, 2075, 2016, 2068, 1989, 2097, 2045, 1966, 2110, 2020, 2103, 1943, 2091, 2066, 2014, 2056, 2100, 2085, 1992, 2055, 2089, 2040, 2145, 1977, 2068, 2057, 2028, 2086, 2099, 1979, 2070, 2052, 1950, 2099, 2048, 2047, 2010, 2011, 2120, 2072, 2011, 2071, 2034, 2043, 2125, 1952, 2090, 2090, 2038, 2081, 1967, 2090, 2046, 2138, 1998, 1999, 2024, 2009, 2076, 2086, 1853, 2126, 2024, 2150, 2056, 1887, 2204, 2067, 2057, 1967, 2116, 2097, 2128, 1973, 2062, 2048, 2048, 2140, 1995, 1979, 2155, 2113, 2028, 1968, 1993, 2194, 1994, 2107, 1990, 2064, 2119, 2029, 2039, 1991, 2125, 1984, 2018, 2016, 2104, 2002, 2036, 2101, 2021, 2097, 2027, 2037, 2041, 2062, 2069, 2025, 1985, 2092, 2018, 2081, 2036, 1963, 2123, 1998, 2066, 2098, 2003, 2035, 2017, 2069, 2115, 1961, 2107, 2037, 2047, 2073, 2078, 2020, 2118, 2062, 2007, 2170, 1975, 2117, 2118, 2043, 2035, 2033, 2079, 2099, 2082, 2009, 2027, 2069, 2033, 2012, 2042, 1991, 2075, 2003, 1972, 2151, 1959, 2021, 2038, 2021, 2030, 2011, 2045, 2072, 2064, 2007, 2098, 2073, 2044, 2011, 2085, 2076, 2103, 2063, 2016, 2001, 2071, 2103, 1994, 2058, 2027, 2091, 2038, 2100, 2025, 2079, 2057, 1998, 2074, 2019, 2060, 2085, 2056, 2047, 2066, 2043, 2053, 2099, 2055, 2012, 2114, 2026, 2061, 2025, 2031, 2063, 1927, 2095, 2058, 2053, 2030, 2014, 2040, 2021, 2041, 2011, 2029, 2018, 2044, 2094, 2035, 2034, 2136, 1981, 2039, 2108, 2005, 2081, 2039, 2015, 2019, 2020, 2101, 2022, 2058, 2090, 2021, 2109, 1993, 1994, 2120, 1999, 2024, 2035, 2005, 2105, 2080, 1992, 2105, 2047, 2116, 2051, 2035, 2019, 2122, 2018, 2070, 2078, 1951, 2106, 2043, 2090, 1969, 2037, 2083, 2048, 1967, 1954, 2068, 2074, 2007, 1939, 2061, 2135, 2103, 2027, 1996, 2096, 2046, 2068, 1987, 2088, 2055, 2092, 2025, 2084, 2001, 2094, 2090, 1965, 2087, 1958, 2068, 2049, 2018, 2005, 2048, 2038, 2025, 2061, 2113, 2020, 2056, 2044, 2123, 2021, 2093, 2091, 2015, 2025, 2115, 2125, 1953, 2069, 2103, 2010, 1973, 2097, 1911, 2089, 2107, 2002, 1959, 2042, 2123, 1982, 2060, 1997, 2052, 2110, 1994, 1974, 2048, 2072, 2052, 1974, 2003, 2172, 2093, 2024, 2051, 2071, 2034, 2048, 2070, 2034, 2033, 2096, 2078, 1967, 2150, 1929, 2128, 2047, 2007, 2039, 2009, 2103, 2004, 2086, 2023, 2053, 2062, 2008, 2059, 2136, 2021, 1981, 2044, 2052, 2038, 2044, 2008, 2063, 2006, 2058, 2057, 2095, 2032, 2057, 2068, 2028, 2019, 2030, 2092, 2054, 2065, 1992, 2009, 2140, 2034, 2080, 2080, 2014, 2051, 2075, 2086, 1989, 2019, 2090, 2049, 1953, 2058, 2073, 2076, 2065, 2017, 2017, 2076, 2034, 2079, 1984, 2030, 2106, 2018, 2100, 2024, 2037, 2020, 2082, 2046, 1974, 2021, 2149, 2019, 2021, 2032, 2048, 2034, 2056, 2148, 1990, 2045, 2071, 2004, 2029, 2099, 1971, 2005, 2089, 2054, 2084, 2070, 2067, 2073, 2025, 2074, 2007, 2063, 2100, 2090, 2029, 1924, 2058, 2173, 2003, 2064, 2000, 2039, 2095, 2029, 2031, 2041, 2046, 2001, 2039, 2049, 2021, 2028, 2072, 2096, 1961, 2034, 2052, 2057, 2031, 2016, 2075, 2052, 2030, 2083, 2098, 2000, 2080, 2074, 2073, 2041, 2038, 2044, 2053, 2049, 2041, 1998, 2107, 2042, 2055, 2061, 2044, 2036, 2028, 2020, 2027, 1999, 2074, 2056, 2061, 2039, 2071, 2141, 2048, 2022, 2019, 2073, 2080, 2058, 2022, 2027, 2073, 2075, 2073, 1999, 2089, 2047, 2044, 2023, 2002, 2097, 2000, 2065, 1987, 2038, 2019, 2069, 2053, 2029, 2048, 2037, 2019, 2022, 2073, 2060, 2072, 2067, 2065, 2026, 2156, 2056, 2073, 2003, 2086, 2083, 1994, 2063, 2044, 2047, 2009, 2052, 2006, 2146, 1982, 2085, 2015, 2039, 2083, 1966, 2082, 2013, 2005, 2052, 2075, 2045, 2009, 2073, 2115, 2037, 1986, 2059, 2075, 2076, 2011, 1997, 2093, 2018, 2061, 2040, 2053, 2051, 2051, 2039, 2024, 2040, 2048, 2103, 2002, 2032, 2059, 2062, 2111, 2019, 2096, 2024, 2084, 1994, 2017, 2009, 2081, 2091, 1979, 2050, 2011, 2123, 2066, 2023, 2047, 2027, 2079, 2037, 2021, 2074, 2003, 2096, 2060, 2014, 2083, 2020, 2131, 2026, 1996, 2058, 2055, 2081, 1989, 2060, 2052, 2080, 1991, 2064, 2074, 2060, 2050, 2027, 2038, 2072, 2091, 2011, 2026, 2085, 2066, 2036, 2076, 2008, 2080, 2064, 2000, 2037, 2006, 2070, 2054, 2013, 2009, 2041, 2076, 2058, 2065, 2016, 2027, 2061, 2055, 1993, 2033, 2060, 2078, 2052, 1985, 2054, 2110, 2070, 2051, 2038, 2090, 2054, 2060, 2047, 2065, 2052, 2079, 2059, 2045, 2067, 2013, 2138, 2043, 2034, 2012, 2039, 2058, 2025, 2043, 2004, 2074, 2037, 2052, 2057, 2049, 2010, 2013, 2084, 2011, 1977, 2038, 2076, 2054, 2015, 2057, 2090, 2038, 2084, 2032, 2060, 2064, 2019, 2050, 2076, 2051, 2038, 2024, 2068, 2127, 2019, 2036, 2054, 2031, 2071, 2031, 2061, 2034, 2032, 2069, 2031, 2058, 2023, 2090, 2044, 2047, 1990, 2060, 2046, 2051, 2071, 1990, 2072, 2070, 2051, 2031, 2084, 2068, 2050, 2016, 2050, 2008, 2058, 2077, 2043, 2050, 2028, 2090, 2080, 2047, 2042, 2054, 2024, 2065, 2031, 2007, 2044, 2081, 2058, 1982, 2083, 2050, 2072, 2031, 2060, 2054, 2038, 2046, 2030, 2028, 2068, 2070, 1990, 2039, 2081, 2058, 2081, 2017, 2062, 2020, 2077, 2101, 1992, 2073, 2026, 2084, 2000, 2053, 2068, 2049, 2037, 2007, 2060, 1993, 2068, 2066, 2010, 2043, 2067, 2040, 2058, 2094, 2067, 2049, 2026, 2072, 2018, 2069, 2085, 2039, 2028, 2033, 2042, 2095, 2044, 2025, 2019, 2058, 2037, 2042, 2025, 2031, 2085, 2044, 2020, 2063, 2079, 2034, 2101, 1988, 2063, 2067, 2065, 2012, 2090, 2045, 2024, 2091, 2021, 2068, 2045, 2066, 2031, 2049, 2045, 2061, 2074, 2032, 2004, 2041, 2079, 2055, 2021, 2033, 2069, 2077, 1985, 2018, 2083, 2048, 2071, 1998, 2048, 2064, 2034, 2090, 2036, 2056, 2012, 2068, 2056, 2021, 2036, 2083, 2056, 2040, 2051, 2019, 2104, 2097, 2068, 1973, 2078, 2091, 2030, 2042, 2078, 2040, 2027, 2042, 2056, 2064, 2068, 2050, 1986, 2033, 2085, 2048, 2017, 2083, 2008, 2085, 2054, 2019, 2047, 2051, 2061, 2053, 2005, 2031, 2092, 2047, 2026, 2013, 2071, 2063, 2067, 2040, 2059, 2072, 2066, 2029, 2046, 2064, 2045, 2059, 2038, 2059, 2045, 2060, 2068, 2046, 2013, 2030, 2088, 2065, 2076, 2019, 2059, 2070, 2048, 2025, 2037, 2054, 2075, 2052, 2014, 2087, 2015, 2031, 2085, 2040, 2036, 2043, 2063, 2048, 2013, 2069, 2024, 2052, 2040, 2020, 2075, 2063, 2085, 2031, 2022, 2046, 2083, 2075, 2008, 2049, 2071, 2048, 2061, 2022, 2035, 2068, 2071, 2052, 2039, 2070, 2043, 2084, 2055, 2021, 2060, 2087, 2016, 2061, 2067, 2037, 2053, 2033, 2053, 2058, 2038, 2029, 2043, 2032, 2059, 2061, 2063, 2003, 2071, 2072, 2023, 2043, 2076, 2041, 2039, 2079, 1989, 2063, 2050, 2080, 2041, 2009, 2111, 2041, 2031, 2062, 2069, 2033, 2062, 2004, 2053, 2044, 2046, 2061, 2005, 2075, 2061, 2040, 2068, 2059, 2074, 2041, 2042, 2089, 2044, 2054, 2039, 2045, 2051, 2058, 2045, 2062, 2038, 2060, 2044, 2041, 2056};

const int16_t FX1[5500] = {2040, 2037, 2036, 2034, 2033, 2032, 2031, 2031, 2030, 2029, 2028, 2028, 2027, 2027, 2027, 2028, 2028, 2026, 2025, 2025, 2026, 2026, 2028, 2029, 2031, 2032, 2031, 2030, 2034, 2034, 2036, 2037, 2040, 2040, 2045, 2045, 2049, 2048, 2053, 2053, 2059, 2057, 2064, 2062, 2068, 2064, 2072, 2067, 2087, 2118, 2077, 2088, 2107, 2091, 2128, 2130, 2305, 2469, 2405, 2353, 2244, 2176, 2217, 2270, 2204, 2270, 2324, 2529, 2418, 2168, 2274, 2276, 2326, 2369, 2587, 2523, 2360, 2347, 2236, 2265, 2138, 2092, 2161, 2172, 2166, 2086, 2028, 2011, 2036, 2027, 1966, 2039, 2085, 1985, 1930, 1877, 1828, 1848, 1902, 2009, 2084, 1922, 1847, 1886, 1884, 1776, 1715, 1785, 1824, 1792, 1736, 1733, 1756, 1773, 1805, 1856, 1824, 1848, 1870, 1881, 1883, 1925, 1901, 1878, 1944, 1914, 1974, 1979, 1921, 2002, 2119, 2189, 2183, 2216, 2135, 2180, 2342, 2314, 2356, 2378, 2389, 2342, 2308, 2283, 2288, 2306, 2312, 2369, 2392, 2276, 2195, 2313, 2231, 2180, 2104, 2081, 2101, 2056, 2014, 1971, 1960, 1909, 1826, 1878, 1932, 1960, 1957, 1858, 1907, 1951, 2011, 1980, 1985, 1976, 1961, 2039, 2104, 2071, 1952, 1813, 1778, 1778, 1812, 1839, 1752, 1675, 1681, 1751, 1727, 1786, 1840, 1780, 1755, 1753, 1902, 1965, 1928, 1894, 1891, 1908, 1887, 1920, 1949, 1944, 1894, 1876, 1902, 1957, 1938, 1924, 1897, 1855, 1967, 2084, 2090, 2044, 2084, 2130, 2162, 2193, 2233, 2334, 2343, 2435, 2570, 2644, 2624, 2572, 2592, 2705, 2763, 2606, 2428, 2418, 2307, 2114, 1958, 1753, 1547, 1369, 1300, 1181, 1146, 1048, 861, 802, 911, 996, 1021, 1031, 1073, 1267, 1419, 1451, 1507, 1650, 1666, 1651, 1694, 1833, 1842, 1846, 1961, 2046, 2070, 1995, 2040, 2195, 2366, 2368, 2322, 2388, 2498, 2542, 2469, 2395, 2346, 2352, 2310, 2252, 2229, 2098, 1917, 1877, 1925, 1885, 1819, 1719, 1838, 2013, 2010, 1923, 1976, 2095, 2107, 2071, 2131, 2219, 2183, 2184, 2315, 2514, 2631, 2683, 2823, 2948, 3060, 3112, 2933, 2836, 2675, 2435, 2298, 2173, 1797, 1403, 1184, 1045, 916, 791, 630, 581, 731, 846, 966, 1308, 1394, 1305, 1612, 1743, 1691, 1928, 1918, 1781, 1946, 1851, 1827, 2073, 2054, 1810, 1912, 2094, 2194, 2530, 2621, 2547, 2771, 2788, 2857, 3154, 3117, 2805, 2700, 2727, 2598, 2543, 2300, 1981, 1954, 1842, 1749, 1919, 1948, 1790, 1758, 1851, 1996, 2200, 2214, 2174, 2347, 2355, 2347, 2525, 2632, 2687, 2639, 2716, 3003, 3009, 2856, 2867, 2923, 2780, 2513, 2407, 2277, 2117, 1707, 1462, 1470, 1081, 960, 1007, 846, 941, 922, 1078, 1466, 1409, 1484, 1636, 1695, 1866, 2004, 2014, 2008, 2112, 2035, 2044, 2104, 1948, 1982, 2087, 2107, 2254, 2304, 2354, 2491, 2643, 2617, 2793, 2918, 2803, 3003, 2959, 2835, 2752, 2515, 2492, 2339, 2145, 2016, 1932, 1934, 1668, 1762, 1774, 1638, 1765, 1847, 2027, 2157, 2210, 2273, 2353, 2486, 2558, 2716, 2823, 2936, 3087, 2968, 2986, 3006, 2781, 2652, 2512, 2326, 2115, 2027, 1658, 1517, 1358, 1066, 1045, 824, 955, 975, 948, 957, 1326, 1414, 1158, 1507, 1577, 1613, 1784, 1817, 2013, 1915, 1892, 1905, 1913, 1886, 1898, 2157, 2021, 2206, 2451, 2309, 2500, 2640, 2727, 2843, 2880, 3090, 3081, 3146, 2927, 2863, 2787, 2543, 2588, 2290, 2220, 2145, 1929, 1788, 1709, 1687, 1534, 1627, 1709, 1744, 1948, 1985, 2127, 2048, 2222, 2421, 2424, 2682, 2911, 2901, 2736, 2831, 2795, 2567, 2427, 2285, 2193, 1932, 1605, 1590, 1233, 932, 799, 849, 705, 710, 1016, 989, 1388, 1294, 1249, 1619, 1606, 1766, 1881, 2117, 2048, 1941, 2027, 1880, 1977, 1792, 1839, 1985, 1940, 2122, 2213, 2250, 2273, 2537, 2708, 2665, 3058, 3175, 3229, 3149, 2989, 3128, 2818, 2750, 2555, 2540, 2347, 1975, 2013, 1865, 1748, 1534, 1473, 1646, 1703, 1806, 1881, 2164, 2129, 2002, 2368, 2485, 2587, 2751, 2847, 2790, 2786, 2771, 2532, 2456, 2234, 2074, 1896, 1586, 1406, 1256, 961, 605, 626, 635, 516, 618, 827, 1080, 1288, 1073, 1266, 1657, 1596, 1657, 1932, 2098, 1966, 1877, 1969, 1911, 1835, 1794, 1907, 1995, 2033, 2186, 2328, 2325, 2476, 2802, 2912, 3006, 3296, 3439, 3420, 3257, 3126, 3117, 2935, 2736, 2480, 2463, 2237, 1866, 1760, 1738, 1633, 1409, 1401, 1615, 1718, 1643, 1790, 2076, 2124, 2075, 2326, 2589, 2582, 2683, 2840, 2810, 2809, 2711, 2664, 2516, 2295, 2169, 2013, 1752, 1425, 1252, 1124, 913, 700, 773, 846, 612, 708, 1077, 1391, 1191, 1111, 1711, 1691, 1561, 1751, 2112, 2058, 1699, 1890, 2087, 1939, 1778, 1894, 2002, 1957, 2056, 2199, 2389, 2456, 2598, 2812, 2833, 3094, 3199, 3361, 3315, 3032, 3079, 3032, 2917, 2542, 2433, 2477, 2096, 1815, 1876, 2022, 1694, 1420, 1606, 1875, 1792, 1699, 2026, 2220, 2153, 2131, 2458, 2632, 2506, 2702, 2838, 2844, 2728, 2693, 2730, 2476, 2290, 2167, 2025, 1673, 1437, 1373, 1100, 900, 771, 767, 677, 598, 822, 1094, 1236, 1072, 1378, 1574, 1496, 1648, 1947, 2203, 1978, 1827, 2036, 2110, 1931, 1777, 1915, 2020, 2028, 2060, 2215, 2397, 2429, 2580, 2706, 2955, 3152, 3306, 3386, 3273, 3221, 3088, 3056, 2905, 2684, 2645, 2418, 2076, 1912, 1940, 1859, 1645, 1509, 1599, 1801, 1689, 1767, 1904, 1999, 2139, 2127, 2378, 2527, 2488, 2485, 2699, 2801, 2638, 2615, 2580, 2526, 2395, 2203, 2080, 1781, 1738, 1545, 1386, 1101, 1156, 1183, 984, 862, 784, 1120, 1141, 1308, 1422, 1401, 1545, 1368, 1600, 1900, 1893, 1832, 1593, 1902, 1992, 1855, 1753, 1854, 2003, 2039, 2191, 2297, 2526, 2566, 2682, 2813, 2859, 3124, 3161, 3249, 3132, 3035, 2961, 2694, 2678, 2617, 2534, 2192, 2009, 1970, 2036, 1982, 1682, 1710, 1673, 1869, 1863, 1903, 2083, 2039, 2188, 2131, 2324, 2464, 2335, 2425, 2600, 2705, 2574, 2490, 2539, 2506, 2430, 2227, 2022, 1893, 1831, 1538, 1328, 1293, 1028, 966, 853, 962, 769, 748, 1305, 1307, 1200, 1359, 1426, 1421, 1593, 1966, 2057, 1683, 1690, 1995, 1969, 1821, 1853, 1948, 2089, 2110, 2220, 2243, 2344, 2604, 2584, 2689, 2933, 3066, 3077, 3155, 3207, 2993, 2715, 2805, 2844, 2650, 2486, 2288, 2012, 2001, 1991, 1972, 1767, 1659, 1879, 1817, 1809, 1869, 1940, 2001, 2079, 2217, 2313, 2330, 2345, 2414, 2562, 2605, 2604, 2437, 2568, 2663, 2407, 2157, 2069, 1924, 1740, 1571, 1439, 1276, 1122, 1073, 1034, 940, 771, 865, 1285, 1510, 1361, 1476, 1451, 1425, 1607, 1952, 1989, 1711, 1639, 1917, 1929, 1762, 1727, 1770, 1976, 2050, 2034, 2203, 2367, 2468, 2552, 2640, 2851, 2911, 3062, 3248, 3226, 2940, 2711, 2741, 2793, 2691, 2472, 2249, 2054, 2059, 2093, 1974, 1841, 1807, 1889, 1973, 1961, 1968, 1961, 2038, 2138, 2168, 2202, 2263, 2355, 2237, 2348, 2455, 2405, 2332, 2465, 2659, 2457, 2171, 2210, 2190, 2027, 1815, 1693, 1633, 1411, 1269, 1293, 1175, 876, 712, 1005, 1342, 1251, 1221, 1423, 1423, 1237, 1449, 1861, 1918, 1728, 1777, 1879, 1813, 1671, 1726, 1847, 1968, 1964, 2053, 2200, 2317, 2401, 2451, 2569, 2715, 2884, 3104, 3215, 3092, 2865, 2772, 2768, 2758, 2682, 2581, 2419, 2170, 2090, 2105, 1995, 1845, 1929, 2038, 2030, 1908, 1920, 2046, 2056, 1975, 2105, 2241, 2271, 2240, 2264, 2250, 2137, 2146, 2381, 2460, 2432, 2481, 2407, 2305, 2280, 2211, 2110, 2090, 1939, 1735, 1600, 1418, 1314, 1174, 1128, 884, 868, 1202, 1361, 1359, 1287, 1348, 1306, 1368, 1768, 1953, 1846, 1838, 1760, 1798, 1683, 1757, 1846, 1870, 1911, 1963, 2004, 2127, 2326, 2388, 2459, 2565, 2765, 3003, 3138, 3091, 2957, 2777, 2767, 2802, 2770, 2677, 2515, 2302, 2127, 2074, 2073, 2030, 2011, 2045, 2095, 2002, 2005, 2105, 2109, 2156, 2100, 2198, 2355, 2290, 2242, 2168, 2158, 2135, 2121, 2359, 2413, 2382, 2317, 2293, 2298, 2229, 2184, 2239, 2159, 1908, 1705, 1599, 1486, 1407, 1179, 1095, 873, 930, 1291, 1277, 1315, 1236, 1299, 1296, 1414, 1765, 1951, 1819, 1777, 1733, 1748, 1706, 1844, 1912, 1910, 1876, 1873, 1967, 2165, 2368, 2349, 2380, 2495, 2768, 2944, 3028, 3012, 2857, 2747, 2747, 2798, 2802, 2689, 2510, 2271, 2165, 2127, 2149, 2104, 2163, 2152, 2080, 2015, 2068, 2155, 2122, 2093, 2034, 2023, 2169, 2253, 2141, 1956, 1981, 2005, 1997, 2228, 2438, 2429, 2299, 2347, 2456, 2374, 2455, 2455, 2400, 2215, 1996, 1780, 1560, 1617, 1483, 1108, 1009, 1002, 892, 1092, 1249, 1351, 1155, 1233, 1497, 1548, 1712, 2002, 1892, 1741, 1706, 1747, 1747, 1699, 1804, 1754, 1632, 1644, 1799, 1917, 2101, 2150, 2184, 2334, 2622, 2861, 2919, 2905, 2910, 2786, 2770, 2894, 2948, 2723, 2495, 2372, 2268, 2168, 2204, 2225, 2158, 2094, 2115, 2113, 2141, 2241, 2230, 2150, 2086, 2160, 2130, 2180, 2243, 2090, 1902, 1918, 2020, 2123, 2141, 2291, 2290, 2164, 2343, 2495, 2484, 2489, 2524, 2418, 2165, 2073, 1975, 1789, 1550, 1437, 1288, 941, 1019, 1016, 810, 1038, 1170, 1253, 1253, 1404, 1599, 1505, 1649, 1930, 1830, 1705, 1807, 1790, 1663, 1619, 1654, 1577, 1534, 1699, 1843, 1866, 1983, 2090, 2199, 2360, 2651, 2812, 2889, 2964, 2987, 2900, 2858, 2862, 2793, 2662, 2518, 2460, 2314, 2193, 2199, 2108, 2049, 2073, 2204, 2269, 2252, 2299, 2250, 2184, 2216, 2279, 2260, 2166, 2158, 2085, 2005, 1955, 1861, 1887, 1952, 2016, 2099, 2173, 2298, 2283, 2313, 2450, 2503, 2475, 2466, 2445, 2226, 1988, 1889, 1711, 1539, 1478, 1292, 1147, 1032, 902, 914, 1161, 1327, 1331, 1307, 1485, 1496, 1525, 1768, 1881, 1678, 1609, 1733, 1687, 1573, 1587, 1579, 1480, 1499, 1678, 1800, 1886, 2032, 2131, 2243, 2370, 2630, 2806, 2926, 2989, 2932, 2839, 2816, 2842, 2827, 2673, 2539, 2456, 2348, 2269, 2301, 2246, 2185, 2210, 2306, 2366, 2385, 2429, 2402, 2326, 2313, 2285, 2181, 2165, 2106, 1932, 1844, 1879, 1741, 1637, 1764, 1923, 1920, 1977, 2197, 2323, 2310, 2510, 2712, 2640, 2652, 2680, 2618, 2427, 2220, 2045, 1879, 1619, 1559, 1383, 1154, 1041, 926, 794, 885, 1126, 1307, 1404, 1372, 1510, 1443, 1481, 1760, 1843, 1703, 1654, 1591, 1531, 1413, 1476, 1499, 1372, 1368, 1591, 1694, 1848, 2019, 2112, 2220, 2369, 2675, 2873, 2996, 3094, 3038, 2900, 2884, 2900, 2854, 2758, 2654, 2539, 2361, 2285, 2323, 2262, 2257, 2237, 2321, 2351, 2410, 2440, 2431, 2313, 2279, 2254, 2178, 2130, 2043, 1898, 1702, 1612, 1585, 1624, 1678, 1702, 1744, 1829, 1968, 2131, 2250, 2421, 2616, 2694, 2747, 2759, 2669, 2658, 2541, 2372, 2134, 1890, 1799, 1674, 1441, 1303, 1177, 1111, 1023, 1074, 1128, 1107, 1313, 1546, 1599, 1495, 1534, 1540, 1553, 1613, 1698, 1516, 1299, 1323, 1362, 1205, 1248, 1357, 1428, 1511, 1742, 1925, 2019, 2173, 2408, 2603, 2684, 2903, 2986, 3036, 2990, 2905, 2773, 2685, 2658, 2608, 2510, 2380, 2278, 2227, 2210, 2274, 2277, 2333, 2415, 2542, 2578, 2575, 2545, 2535, 2456, 2395, 2356, 2241, 2068, 1882, 1737, 1565, 1458, 1431, 1432, 1444, 1573, 1683, 1727, 1815, 2006, 2153, 2327, 2547, 2726, 2796, 2750, 2750, 2642, 2523, 2538, 2417, 2231, 2068, 1874, 1705, 1575, 1481, 1422, 1411, 1408, 1494, 1403, 1467, 1451, 1363, 1550, 1632, 1753, 1605, 1561, 1497, 1347, 1315, 1333, 1285, 1237, 1310, 1258, 1252, 1306, 1404, 1619, 1624, 1876, 2059, 2205, 2369, 2445, 2490, 2497, 2571, 2616, 2728, 2697, 2663, 2565, 2433, 2443, 2447, 2469, 2486, 2514, 2605, 2570, 2578, 2598, 2656, 2662, 2693, 2693, 2657, 2585, 2522, 2382, 2191, 2077, 2024, 1920, 1859, 1778, 1696, 1579, 1546, 1569, 1594, 1639, 1741, 1831, 1941, 2019, 1975, 2028, 2167, 2276, 2378, 2413, 2400, 2442, 2424, 2485, 2396, 2362, 2446, 2426, 2344, 2309, 2250, 2114, 2134, 2045, 2000, 1962, 1869, 1889, 1720, 1645, 1535, 1321, 1221, 1440, 1449, 1447, 1333, 1160, 1235, 1189, 1262, 1405, 1346, 1445, 1495, 1465, 1488, 1527, 1572, 1712, 1681, 1786, 1960, 1970, 2083, 2100, 2129, 2143, 2227, 2384, 2587, 2600, 2627, 2570, 2531, 2633, 2696, 2727, 2741, 2702, 2751, 2674, 2617, 2593, 2551, 2518, 2514, 2468, 2486, 2470, 2386, 2338, 2205, 2154, 2172, 2161, 2162, 2120, 2027, 1929, 1819, 1789, 1795, 1787, 1749, 1790, 1753, 1767, 1755, 1775, 1851, 1854, 1997, 2132, 2164, 2239, 2342, 2380, 2433, 2451, 2511, 2563, 2554, 2643, 2626, 2426, 2387, 2347, 2209, 2180, 2127, 2038, 1937, 1868, 1839, 1681, 1592, 1621, 1548, 1422, 1348, 1472, 1441, 1515, 1461, 1343, 1253, 1257, 1382, 1416, 1439, 1357, 1356, 1311, 1322, 1409, 1420, 1521, 1569, 1662, 1782, 1814, 1903, 1963, 2086, 2129, 2251, 2346, 2461, 2550, 2582, 2550, 2504, 2552, 2656, 2703, 2764, 2720, 2699, 2685, 2662, 2694, 2670, 2673, 2716, 2721, 2700, 2678, 2581, 2522, 2470, 2397, 2366, 2328, 2236, 2189, 2050, 1942, 1817, 1762, 1734, 1735, 1728, 1662, 1636, 1614, 1650, 1646, 1665, 1754, 1918, 1989, 1955, 2061, 2068, 2106, 2255, 2320, 2424, 2446, 2515, 2599, 2541, 2506, 2610, 2581, 2505, 2588, 2561, 2449, 2360, 2265, 2186, 2062, 1975, 1991, 1873, 1809, 1760, 1637, 1413, 1309, 1255, 1306, 1371, 1427, 1387, 1232, 1192, 1169, 1198, 1287, 1311, 1335, 1375, 1329, 1344, 1362, 1364, 1450, 1497, 1563, 1721, 1805, 1944, 2026, 2041, 2042, 2175, 2303, 2513, 2650, 2691, 2707, 2681, 2665, 2712, 2784, 2846, 2857, 2839, 2835, 2796, 2780, 2750, 2728, 2703, 2703, 2723, 2750, 2729, 2703, 2591, 2432, 2369, 2362, 2339, 2319, 2192, 2071, 1938, 1810, 1730, 1682, 1638, 1603, 1585, 1551, 1566, 1554, 1550, 1563, 1600, 1647, 1766, 1868, 1978, 2036, 2041, 2060, 2150, 2249, 2340, 2451, 2515, 2614, 2631, 2646, 2640, 2666, 2769, 2796, 2751, 2697, 2702, 2668, 2552, 2433, 2321, 2248, 2142, 2156, 2026, 1898, 1787, 1662, 1560, 1440, 1356, 1271, 1087, 1073, 1150, 1125, 1111, 959, 933, 905, 936, 1045, 1135, 1112, 1167, 1234, 1181, 1262, 1347, 1473, 1647, 1626, 1771, 1896, 1981, 2081, 2200, 2238, 2289, 2440, 2552, 2709, 2770, 2815, 2842, 2789, 2818, 2958, 3014, 3040, 3014, 3019, 2974, 2904, 2904, 2902, 2867, 2842, 2787, 2739, 2716, 2667, 2594, 2464, 2306, 2297, 2289, 2209, 2148, 2041, 1935, 1812, 1728, 1693, 1696, 1641, 1605, 1574, 1527, 1504, 1521, 1495, 1497, 1505, 1551, 1571, 1606, 1666, 1704, 1701, 1755, 1873, 1955, 2147, 2218, 2312, 2415, 2527, 2602, 2694, 2761, 2908, 2957, 2933, 2981, 2919, 2896, 2892, 2816, 2673, 2606, 2549, 2467, 2391, 2263, 2179, 2042, 1974, 1941, 1923, 1836, 1737, 1682, 1534, 1470, 1332, 1196, 1045, 873, 844, 857, 853, 785, 698, 650, 632, 678, 827, 914, 986, 1137, 1215, 1275, 1346, 1490, 1590, 1715, 1763, 1928, 2048, 2100, 2236, 2306, 2337, 2349, 2479, 2596, 2771, 2921, 2974, 3051, 3050, 3119, 3246, 3301, 3333, 3413, 3411, 3350, 3302, 3247, 3179, 3045, 2952, 2846, 2787, 2705, 2631, 2529, 2383, 2225, 2151, 2091, 2028, 2034, 1960, 1860, 1764, 1677, 1600, 1535, 1440, 1381, 1336, 1266, 1218, 1194, 1150, 1083, 1061, 1044, 1093, 1193, 1285, 1378, 1446, 1492, 1551, 1654, 1753, 1919, 2116, 2154, 2272, 2488, 2597, 2647, 2724, 2793, 2936, 3075, 3138, 3242, 3216, 3252, 3356, 3242, 3136, 3228, 3250, 3201, 3168, 3039, 2906, 2774, 2689, 2642, 2476, 2314, 2255, 2189, 2061, 1911, 1772, 1602, 1459, 1395, 1268, 1109, 980, 880, 737, 552, 380, 266, 153, 100, 202, 279, 349, 298, 334, 464, 528, 704, 836, 960, 1155, 1371, 1453, 1540, 1676, 1776, 1970, 2050, 2155, 2404, 2561, 2706, 2823, 2900, 2967, 3133, 3268, 3401, 3578, 3658, 3711, 3724, 3722, 3693, 3671, 3594, 3572, 3521, 3430, 3291, 3144, 3030, 2847, 2734, 2559, 2428, 2410, 2355, 2253, 2120, 1985, 1832, 1770, 1674, 1589, 1542, 1411, 1339, 1261, 1156, 1050, 969, 889, 842, 863, 862, 884, 930, 999, 1031, 1068, 1095, 1222, 1404, 1550, 1681, 1788, 1866, 1937, 2052, 2140, 2211, 2287, 2381, 2562, 2714, 2773, 2920, 2977, 3056, 3186, 3335, 3405, 3502, 3600, 3667, 3641, 3512, 3528, 3441, 3337, 3313, 3226, 3062, 2892, 2803, 2667, 2491, 2337, 2248, 2168, 2074, 2073, 1997, 1860, 1729, 1677, 1608, 1471, 1385, 1327, 1231, 1071, 941, 774, 597, 452, 376, 308, 182, 132, 108, 151, 92, 93, 189, 238, 383, 594, 840, 954, 1041, 1236, 1295, 1407, 1523, 1641, 1777, 1853, 2021, 2157, 2215, 2231, 2402, 2545, 2589, 2777, 2969, 3102, 3269, 3385, 3448, 3494, 3516, 3596, 3688, 3651, 3629, 3643, 3553, 3384, 3313, 3191, 3015, 2939, 2867, 2829, 2720, 2637, 2565, 2473, 2371, 2276, 2255, 2160, 2104, 2106, 2005, 1824, 1677, 1580, 1415, 1290, 1200, 1108, 1065, 994, 971, 925, 857, 839, 913, 984, 1032, 1187, 1295, 1374, 1456, 1550, 1591, 1650, 1745, 1856, 1961, 2017, 2071, 2123, 2181, 2210, 2297, 2356, 2418, 2551, 2702, 2840, 2960, 3013, 3118, 3195, 3301, 3356, 3411, 3422, 3440, 3469, 3354, 3273, 3106, 3067, 2974, 2846, 2769, 2657, 2570, 2485, 2434, 2331, 2223, 2176, 2138, 2133, 2103, 2047, 2003, 1904, 1835, 1772, 1663, 1523, 1445, 1370, 1278, 1140, 1013, 907, 810, 767, 662, 648, 575, 606, 643, 627, 662, 642, 692, 709, 737, 759, 787, 826, 887, 936, 989, 997, 1066, 1251, 1344, 1457, 1584, 1713, 1910, 2090, 2226, 2328, 2477, 2630, 2787, 2952, 2956, 3071, 3149, 3137, 3226, 3221, 3226, 3228, 3248, 3226, 3225, 3211, 3145, 3165, 3151, 3122, 3105, 3053, 2995, 2994, 2932, 2833, 2722, 2611, 2535, 2421, 2309, 2162, 2020, 1940, 1807, 1697, 1610, 1503, 1443, 1410, 1355, 1305, 1315, 1275, 1275, 1307, 1276, 1278, 1307, 1301, 1325, 1380, 1357, 1364, 1415, 1432, 1496, 1542, 1576, 1652, 1749, 1842, 1915, 2029, 2100, 2224, 2355, 2421, 2515, 2582, 2668, 2729, 2774, 2808, 2794, 2820, 2853, 2869, 2907, 2858, 2866, 2892, 2897, 2903, 2895, 2890, 2856, 2881, 2855, 2811, 2769, 2702, 2648, 2579, 2480, 2420, 2315, 2222, 2166, 2078, 2003, 1905, 1856, 1809, 1787, 1757, 1720, 1710, 1698, 1710, 1721, 1687, 1677, 1650, 1626, 1626, 1569, 1557, 1482, 1447, 1405, 1355, 1316, 1276, 1249, 1215, 1229, 1213, 1214, 1210, 1215, 1223, 1268, 1267, 1277, 1291, 1265, 1282, 1285, 1242, 1218, 1235, 1211, 1227, 1261, 1250, 1314, 1423, 1510, 1595, 1683, 1749, 1865, 2001, 2057, 2135, 2235, 2280, 2382, 2437, 2452, 2487, 2521, 2557, 2563, 2606, 2609, 2652, 2690, 2717, 2779, 2827, 2835, 2861, 2908, 2930, 2943, 2934, 2915, 2860, 2845, 2790, 2686, 2622, 2531, 2463, 2385, 2296, 2226, 2172, 2129, 2057, 2016, 1994, 1951, 1965, 1950, 1920, 1909, 1867, 1833, 1807, 1786, 1757, 1722, 1682, 1645, 1635, 1633, 1593, 1586, 1584, 1601, 1636, 1667, 1705, 1749, 1817, 1878, 1909, 1971, 2016, 2067, 2131, 2155, 2182, 2217, 2230, 2240, 2259, 2257, 2247, 2267, 2274, 2321, 2376, 2348, 2427, 2482, 2528, 2573, 2581, 2587, 2663, 2709, 2684, 2678, 2620, 2621, 2627, 2549, 2452, 2429, 2354, 2333, 2308, 2245, 2186, 2176, 2145, 2141, 2149, 2096, 2121, 2112, 2146, 2122, 2115, 2042, 2038, 2026, 1986, 1933, 1851, 1826, 1770, 1760, 1688, 1636, 1576, 1597, 1557, 1578, 1575, 1549, 1562, 1573, 1561, 1558, 1558, 1490, 1518, 1490, 1465, 1412, 1383, 1287, 1290, 1266, 1167, 1158, 1126, 1118, 1145, 1120, 1101, 1126, 1116, 1146, 1176, 1224, 1274, 1364, 1367, 1410, 1554, 1588, 1695, 1767, 1766, 1870, 1954, 2004, 2061, 2116, 2153, 2239, 2304, 2341, 2419, 2480, 2539, 2603, 2654, 2683, 2761, 2795, 2832, 2882, 2876, 2912, 2896, 2894, 2910, 2872, 2842, 2804, 2759, 2740, 2699, 2644, 2588, 2569, 2514, 2475, 2444, 2348, 2349, 2320, 2240, 2222, 2189, 2125, 2127, 2082, 2008, 1986, 1961, 1908, 1867, 1841, 1795, 1788, 1769, 1733, 1734, 1715, 1719, 1730, 1731, 1752, 1746, 1763, 1788, 1811, 1853, 1871, 1866, 1884, 1941, 1948, 1974, 2000, 1986, 2038, 2083, 2079, 2111, 2141, 2154, 2209, 2295, 2284, 2331, 2430, 2455, 2546, 2555, 2550, 2584, 2661, 2638, 2629, 2610, 2582, 2589, 2576, 2490, 2449, 2446, 2392, 2399, 2347, 2316, 2276, 2304, 2262, 2261, 2230, 2206, 2186, 2194, 2176, 2127, 2106, 2037, 2044, 1992, 1963, 1877, 1866, 1821, 1797, 1773, 1729, 1696, 1697, 1704, 1670, 1692, 1668, 1681, 1678, 1705, 1672, 1688, 1654, 1647, 1651, 1625, 1621, 1582, 1587, 1554, 1579, 1549, 1536, 1527, 1520, 1531, 1545, 1532, 1514, 1523, 1498, 1509, 1469, 1432, 1407, 1401, 1392, 1385, 1368, 1317, 1320, 1322, 1306, 1278, 1303, 1306, 1373, 1440, 1530, 1638, 1682, 1779, 1814, 1870, 1950, 2014, 2039, 2148, 2197, 2233, 2284, 2301, 2315, 2351, 2404, 2368, 2462, 2514, 2553, 2641, 2659, 2669, 2753, 2755, 2759, 2807, 2813, 2837, 2838, 2834, 2748, 2752, 2681, 2609, 2587, 2511, 2472, 2451, 2399, 2353, 2341, 2283, 2243, 2198, 2201, 2173, 2164, 2154, 2117, 2101, 2084, 2026, 1966, 1943, 1913, 1884, 1865, 1861, 1812, 1841, 1839, 1806, 1837, 1846, 1865, 1901, 1949, 1974, 2028, 2053, 2062, 2083, 2107, 2114, 2128, 2139, 2146, 2177, 2173, 2174, 2175, 2180, 2210, 2258, 2256, 2308, 2376, 2412, 2474, 2475, 2487, 2533, 2554, 2544, 2543, 2536, 2515, 2515, 2474, 2394, 2367, 2319, 2278, 2277, 2222, 2213, 2213, 2202, 2191, 2194, 2185, 2168, 2181, 2185, 2182, 2174, 2152, 2116, 2090, 2048, 2002, 1950, 1913, 1880, 1851, 1831, 1799, 1764, 1762, 1758, 1741, 1756, 1755, 1765, 1783, 1789, 1788, 1794, 1791, 1775, 1776, 1771, 1773, 1775, 1767, 1757, 1771, 1775, 1770, 1770, 1779, 1803, 1824, 1814, 1822, 1843, 1849, 1866, 1834, 1829, 1828, 1831, 1808, 1784, 1778, 1761, 1768, 1753, 1727, 1726, 1737, 1737, 1743, 1719, 1737, 1734, 1744, 1735, 1706, 1721, 1697, 1701, 1689, 1666, 1640, 1643, 1633, 1608, 1586, 1571, 1561, 1564, 1578, 1563, 1583, 1571, 1583, 1589, 1627, 1641, 1645, 1686, 1743, 1848, 1869, 1895, 1913, 1983, 2061, 2107, 2111, 2125, 2214, 2291, 2340, 2345, 2355, 2384, 2472, 2493, 2520, 2538, 2570, 2643, 2680, 2715, 2712, 2742, 2735, 2771, 2800, 2796, 2781, 2771, 2762, 2765, 2762, 2689, 2648, 2622, 2625, 2594, 2561, 2494, 2472, 2477, 2446, 2390, 2335, 2317, 2292, 2301, 2240, 2197, 2163, 2146, 2114, 2065, 2030, 1985, 1972, 1957, 1948, 1914, 1912, 1884, 1886, 1885, 1892, 1889, 1875, 1896, 1912, 1950, 1936, 1935, 1929, 1953, 1979, 1980, 1972, 1974, 2013, 2045, 2111, 2140, 2137, 2167, 2236, 2280, 2323, 2326, 2315, 2372, 2426, 2433, 2407, 2393, 2362, 2396, 2399, 2346, 2323, 2308, 2296, 2315, 2310, 2256, 2245, 2245, 2242, 2252, 2252, 2202, 2199, 2207, 2185, 2170, 2121, 2067, 2041, 2032, 1988, 1951, 1909, 1866, 1851, 1843, 1819, 1787, 1771, 1764, 1778, 1781, 1779, 1760, 1759, 1769, 1768, 1763, 1754, 1749, 1749, 1768, 1764, 1760, 1759, 1761, 1766, 1775, 1784, 1795, 1807, 1814, 1834, 1847, 1848, 1842, 1838, 1840, 1855, 1863, 1844, 1848, 1848, 1847, 1851, 1841, 1823, 1817, 1824, 1821, 1826, 1832, 1822, 1818, 1823, 1817, 1822, 1809, 1796, 1804, 1812, 1809, 1799, 1785, 1779, 1784, 1787, 1769, 1759, 1765, 1757, 1766, 1763, 1751, 1755, 1756, 1764, 1769, 1775, 1801, 1810, 1846, 1860, 1876, 1912, 1934, 2029, 2046, 2089, 2127, 2149, 2241, 2270, 2279, 2284, 2333, 2386, 2410, 2448, 2437, 2440, 2519, 2525, 2535, 2556, 2542, 2583, 2625, 2655, 2635, 2646, 2653, 2642, 2681, 2674, 2638, 2636, 2613, 2593, 2613, 2551, 2507, 2482, 2461, 2443, 2408, 2386, 2325, 2331, 2316, 2271, 2245, 2213, 2187, 2177, 2148, 2111, 2082, 2065, 2045, 2000, 1987, 1951, 1936, 1918, 1899, 1893, 1881, 1879, 1870, 1872, 1872, 1885, 1874, 1883, 1893, 1912, 1922, 1920, 1928, 1929, 1967, 1964, 1957, 1958, 1981, 2006, 2025, 2035, 2040, 2071, 2111, 2142, 2153, 2177, 2194, 2231, 2261, 2275, 2279, 2284, 2307, 2315, 2320, 2309, 2301, 2295, 2304, 2304, 2294, 2272, 2263, 2266, 2264, 2264, 2229, 2226, 2217, 2220, 2210, 2183, 2164, 2142, 2138, 2119, 2096, 2067, 2044, 2028, 2017, 1987, 1970, 1947, 1925, 1920, 1906, 1906, 1891, 1883, 1874, 1870, 1869, 1858, 1848, 1842, 1836, 1833, 1836, 1824, 1824, 1812, 1825, 1828, 1823, 1835, 1835, 1848, 1858, 1869, 1873, 1875, 1879, 1887, 1888, 1896, 1884, 1883, 1889, 1881, 1887, 1875, 1879, 1877, 1875, 1882, 1868, 1882, 1888, 1882, 1907, 1900, 1878, 1882, 1898, 1887, 1863, 1868, 1864, 1856, 1860, 1846, 1852, 1845, 1840, 1847, 1848, 1843, 1854, 1857, 1870, 1884, 1883, 1888, 1898, 1894, 1885, 1905, 1884, 1877, 1887, 1877, 1882, 1889, 1885, 1889, 1904, 1916, 1921, 1955, 1955, 1965, 2015, 2019, 2036, 2052, 2063, 2095, 2154, 2172, 2164, 2195, 2230, 2261, 2302, 2280, 2269, 2326, 2349, 2369, 2373, 2372, 2376, 2414, 2425, 2420, 2430, 2407, 2410, 2438, 2443, 2407, 2422, 2399, 2382, 2412, 2391, 2368, 2349, 2327, 2316, 2352, 2322, 2269, 2274, 2271, 2245, 2243, 2214, 2168, 2182, 2170, 2152, 2140, 2118, 2092, 2104, 2094, 2054, 2058, 2056, 2047, 2045, 2048, 2037, 2052, 2050, 2048, 2055, 2051, 2050, 2054, 2071, 2062, 2067, 2064, 2065, 2068, 2072, 2072, 2063, 2061, 2064, 2079, 2079, 2076, 2067, 2069, 2075, 2084, 2083, 2078, 2076, 2087, 2108, 2106, 2111, 2100, 2107, 2115, 2128, 2129, 2113, 2119, 2131, 2140, 2143, 2134, 2134, 2137, 2133, 2150, 2150, 2144, 2132, 2139, 2157, 2149, 2143, 2134, 2141, 2146, 2140, 2144, 2135, 2124, 2127, 2129, 2125, 2109, 2095, 2097, 2096, 2090, 2072, 2062, 2054, 2039, 2040, 2035, 2018, 2003, 1993, 1991, 1983, 1968, 1955, 1943, 1935, 1924, 1919, 1912, 1894, 1886, 1881, 1880, 1878, 1862, 1862, 1856, 1855, 1856, 1854, 1848, 1844, 1853, 1857, 1862, 1861, 1854, 1861, 1869, 1865, 1863, 1864, 1864, 1867, 1871, 1871, 1869, 1868, 1874, 1878, 1881, 1891, 1892, 1900, 1906, 1906, 1909, 1910, 1908, 1898, 1905, 1911, 1906, 1914, 1909, 1914, 1923, 1915, 1918, 1918, 1919, 1917, 1920, 1927, 1913, 1925, 1920, 1908, 1924, 1917, 1912, 1921, 1916, 1923, 1934, 1933, 1922, 1921, 1941, 1916, 1935, 1935, 1913, 1937, 1938, 1936, 1937, 1939, 1946, 1955, 1953, 1956, 1976, 1996, 1987, 2013, 2063, 2074, 2097, 2109, 2125, 2173, 2196, 2204, 2205, 2213, 2239, 2266, 2278, 2274, 2266, 2298, 2305, 2318, 2320, 2304, 2334, 2332, 2357, 2367, 2355, 2366, 2365, 2368, 2368, 2362, 2349, 2339, 2334, 2333, 2323, 2313, 2282, 2262, 2270, 2251, 2243, 2228, 2204, 2207, 2206, 2192, 2177, 2160, 2154, 2146, 2138, 2123, 2094, 2105, 2090, 2073, 2066, 2050, 2052, 2041, 2040, 2026, 2024, 2031, 2026, 2029, 2035, 2024, 2027, 2028, 2016, 2023, 2018, 2016, 2003, 1996, 2002, 1998, 1992, 1978, 1973, 1984, 1982, 1975, 1970, 1969, 1987, 2013, 2009, 2017, 2023, 2034, 2067, 2074, 2051, 2051, 2083, 2092, 2085, 2081, 2076, 2084, 2116, 2083, 2099, 2104, 2088, 2111, 2124, 2123, 2111, 2132, 2140, 2134, 2154, 2136, 2136, 2150, 2122, 2131, 2143, 2124, 2113, 2117, 2112, 2110, 2105, 2098, 2086, 2097, 2086, 2092, 2094, 2079, 2070, 2072, 2054, 2051, 2037, 2022, 2021, 2002, 2004, 1991, 1982, 1966, 1958, 1954, 1944, 1939, 1932, 1922, 1933, 1924, 1923, 1913, 1905, 1905, 1905, 1902, 1895, 1890, 1891, 1885, 1879, 1876, 1869, 1868, 1855, 1860, 1860, 1862, 1855, 1858, 1856, 1861, 1857, 1857, 1854, 1851, 1863, 1869, 1872, 1866, 1871, 1875, 1881, 1879, 1880, 1876, 1879, 1875, 1877, 1875, 1872, 1866, 1874, 1874, 1880, 1875, 1878, 1887, 1887, 1897, 1893, 1901, 1900, 1904, 1905, 1914, 1901, 1904, 1907, 1911, 1905, 1904, 1907, 1896, 1904, 1910, 1900, 1908, 1908, 1914, 1938, 1936, 1936, 1934, 1958, 1966, 1962, 1969, 1962, 1979, 2004, 1990, 2017, 2026, 2046, 2083, 2090, 2109, 2118, 2159, 2181, 2194, 2210, 2195, 2227, 2265, 2243, 2262, 2272, 2278, 2294, 2296, 2295, 2297, 2311, 2307, 2300, 2324, 2316, 2315, 2343, 2318, 2326, 2328, 2316, 2315, 2301, 2293, 2293, 2287, 2269, 2236, 2242, 2231, 2205, 2207, 2178, 2175, 2178, 2160, 2152, 2145, 2138, 2123, 2102, 2104, 2086, 2088, 2088, 2066, 2060, 2054, 2045, 2048, 2030, 2020, 2015, 2019, 2022, 2006, 2015, 2011, 2013, 2013, 2001, 2001, 2008, 1998, 1999, 1991, 1992, 1986, 1976, 1977, 1967, 1973, 1968, 1967, 1968, 1967, 1969, 1975, 1974, 1981, 1990, 2011, 2018, 2023, 2036, 2058, 2077, 2077, 2067, 2080, 2101, 2115, 2110, 2102, 2117, 2136, 2152, 2135, 2137, 2145, 2156, 2162, 2168, 2163, 2169, 2171, 2181, 2177, 2169, 2164, 2165, 2170, 2153, 2150, 2149, 2143, 2133, 2124, 2111, 2109, 2093, 2087, 2078, 2075, 2060, 2051, 2044, 2031, 2020, 2014, 1998, 1989, 1979, 1971, 1968, 1953, 1947, 1935, 1934, 1926, 1921, 1915, 1916, 1910, 1910, 1901, 1906, 1901, 1899, 1895, 1892, 1897, 1895, 1892, 1886, 1884, 1885, 1886, 1878, 1874, 1874, 1885, 1883, 1883, 1877, 1878, 1885, 1885, 1876, 1873, 1871, 1870, 1865, 1862, 1856, 1862, 1866, 1866, 1869, 1864, 1875, 1878, 1877, 1872, 1879, 1881, 1882, 1881, 1878, 1880, 1888, 1881, 1884, 1886, 1885, 1900, 1894, 1902, 1903, 1915, 1914, 1915, 1929, 1931, 1946, 1953, 1952, 1965, 1964, 1962, 1960, 1946, 1947, 1943, 1946, 1944, 1942, 1952, 1957, 1970, 1964, 1962, 1977, 1972, 1986, 1994, 1997, 2008, 2017, 2019, 2020, 2032, 2034, 2041, 2054, 2067, 2084, 2113, 2120, 2136, 2161, 2186, 2210, 2207, 2218, 2236, 2255, 2264, 2265, 2275, 2283, 2289, 2292, 2287, 2284, 2288, 2293, 2298, 2291, 2295, 2304, 2301, 2306, 2295, 2292, 2289, 2282, 2275, 2258, 2255, 2242, 2224, 2213, 2199, 2184, 2172, 2154, 2145, 2139, 2133, 2125, 2110, 2111, 2102, 2100, 2090, 2073, 2073, 2071, 2062, 2054, 2041, 2029, 2028, 2027, 2020, 2005, 2008, 2009, 2008, 2013, 2001, 2007, 2014, 2012, 2014, 2013, 2013, 2016, 2015, 2010, 2004, 2002, 2006, 1998, 2002, 1987, 1988, 1996, 1986, 1986, 1989, 1995, 1995, 1992, 1998, 1998, 2004, 2004, 2023, 2042, 2034, 2051, 2055, 2074, 2110, 2095, 2065, 2090, 2129, 2128, 2107, 2116, 2124, 2164, 2151, 2134, 2152, 2144, 2148, 2172, 2165, 2137, 2154, 2170, 2173, 2167, 2156, 2141, 2179, 2171, 2144, 2157, 2159, 2147, 2155, 2135, 2126, 2121, 2115, 2106, 2098, 2094, 2082, 2085, 2071, 2056, 2061, 2051, 2039, 2039, 2018, 2024, 2015, 2011, 2000, 1990, 1986, 1983, 1976, 1965, 1963, 1963, 1955, 1951, 1945, 1935, 1941, 1934, 1928, 1935, 1936, 1927, 1931, 1930, 1932, 1928, 1928, 1920, 1918, 1923, 1915, 1914, 1905, 1899, 1901, 1898, 1893, 1887, 1893, 1896, 1893, 1898, 1893, 1896, 1902, 1902, 1903, 1903, 1909, 1913, 1918, 1914, 1911, 1917, 1918, 1911, 1911, 1911, 1911, 1919, 1921, 1919, 1920, 1930, 1931, 1928, 1927, 1932, 1939, 1944, 1944, 1942, 1950, 1959, 1952, 1949, 1951, 1956, 1963, 1965, 1968, 1971, 1976, 1983, 1981, 1980, 1981, 1994, 1995, 1993, 1996, 1992, 2002, 2004, 2004, 1999, 2008, 2021, 2034, 2036, 2049, 2068, 2097, 2109, 2115, 2141, 2153, 2184, 2199, 2207, 2215, 2226, 2232, 2235, 2243, 2234, 2240, 2254, 2256, 2252, 2262, 2260, 2262, 2270, 2268, 2273, 2273, 2278, 2276, 2285, 2277, 2265, 2267, 2252, 2237, 2239, 2222, 2207, 2205, 2194, 2186, 2170, 2169, 2154, 2157, 2155, 2142, 2143, 2134, 2132, 2131, 2124, 2110, 2110, 2101, 2096, 2089, 2083, 2076, 2067, 2066, 2051, 2056, 2048, 2049, 2050, 2049, 2051, 2050, 2049, 2047, 2048, 2045, 2048, 2039, 2039, 2030, 2030, 2027, 2016, 2014, 2007, 2010, 2006, 2007, 2004, 2004, 2006, 2009, 2006, 2007, 2012, 2011, 2016, 2011, 2012, 2009, 2009, 2009, 2011, 2011, 2014, 2020, 2041, 2059, 2059, 2090, 2102, 2115, 2137, 2151, 2142, 2148, 2175, 2171, 2164, 2163, 2170, 2187, 2175, 2157, 2177, 2169, 2155, 2165, 2166, 2157, 2163, 2171, 2167, 2171, 2165, 2156, 2172, 2164, 2144, 2150, 2145, 2126, 2126, 2109, 2097, 2089, 2075, 2064, 2060, 2057, 2050, 2055, 2047, 2041, 2045, 2040, 2030, 2034, 2024, 2023, 2016, 2008, 2003, 1990, 1985, 1978, 1975, 1963, 1961, 1960, 1956, 1957, 1952, 1954, 1959, 1958, 1954, 1962, 1963, 1956, 1954, 1954, 1949, 1944, 1939, 1933, 1935, 1926, 1918, 1913, 1914, 1910, 1911, 1908, 1906, 1910, 1910, 1913, 1916, 1920, 1917, 1922, 1921, 1921, 1922, 1919, 1918, 1920, 1918, 1910, 1913, 1915, 1914, 1915, 1920, 1922, 1930, 1940, 1944, 1952, 1956, 1960, 1963, 1962, 1964, 1965, 1965, 1970, 1967, 1971, 1972, 1977, 1975, 1971, 1977, 1975, 1981, 1981, 1983, 1998, 2000, 2008, 2007, 2014, 2022, 2020, 2022, 2024, 2025, 2017, 2016, 2021, 2017, 2010, 2014, 2019, 2025, 2030, 2042, 2058, 2070, 2077, 2100, 2121, 2134, 2151, 2168, 2184, 2192, 2213, 2216, 2215, 2218, 2218, 2222, 2223, 2222, 2230, 2231, 2238, 2246, 2246, 2250, 2247, 2263, 2260, 2265, 2272, 2266, 2270, 2262, 2262, 2253, 2248, 2232, 2219, 2215, 2204, 2197, 2187, 2178, 2173, 2173, 2162, 2165, 2160, 2153, 2155, 2154, 2144, 2136, 2141, 2129, 2126, 2114, 2104, 2100, 2094, 2081, 2077, 2075, 2067, 2074, 2066, 2063, 2064, 2068, 2063, 2063, 2060, 2055, 2060, 2058, 2055, 2050, 2045, 2037, 2038, 2026, 2020, 2017, 2011, 2012, 2009, 2007, 2004, 2007, 2003, 2004, 2010, 2010, 2012, 2012, 2012, 2017, 2016, 2011, 2012, 2011, 2010, 2010, 2011, 2022, 2032, 2046, 2062, 2071, 2093, 2106, 2118, 2127, 2133, 2138, 2143, 2149, 2148, 2146, 2152, 2148, 2154, 2146, 2137, 2140, 2135, 2133, 2132, 2137, 2136, 2147, 2152, 2148, 2149, 2151, 2145, 2146, 2140, 2131, 2132, 2127, 2118, 2113, 2103, 2092, 2087, 2081, 2073, 2073, 2070, 2070, 2069, 2066, 2063, 2064, 2060, 2055, 2058, 2048, 2049, 2041, 2038, 2033, 2027, 2021, 2016, 2012, 2007, 2005, 2000, 1994, 1991, 1991, 1987, 1982, 1978, 1976, 1974, 1968, 1965, 1958, 1954, 1951, 1948, 1945, 1940, 1939, 1934, 1930, 1925, 1922, 1921, 1917, 1913, 1915, 1918, 1918, 1916, 1918, 1917, 1917, 1921, 1926, 1929, 1930, 1933, 1938, 1938, 1935, 1937, 1940, 1944, 1942, 1941, 1947, 1946, 1945, 1941, 1942, 1942, 1940, 1940, 1943, 1948, 1949, 1959, 1963, 1962, 1967, 1973, 1973, 1980, 1982, 1980, 1988, 1992, 1987, 1992, 1997, 1996, 2001, 1999, 1992, 1997, 1996, 1996, 2002, 2005, 2008, 2012, 2015, 2019, 2026, 2028, 2030, 2032, 2031, 2036, 2039, 2038, 2040, 2040, 2041, 2039, 2039, 2039, 2049, 2055, 2062, 2073, 2076, 2088, 2107, 2122, 2130, 2143, 2156, 2170, 2173, 2177, 2191, 2195, 2188, 2183, 2194, 2192, 2193, 2195, 2191, 2197, 2195, 2193, 2197, 2196, 2199, 2205, 2207, 2203, 2202, 2207, 2204, 2199, 2193, 2185, 2186, 2176, 2168, 2162, 2154, 2154, 2144, 2137, 2134, 2134, 2136, 2132, 2128, 2123, 2123, 2122, 2115, 2112, 2109, 2105, 2100, 2094, 2086, 2083, 2074, 2071, 2069, 2062, 2061, 2062, 2062, 2060, 2061, 2061, 2061, 2058, 2054, 2053, 2055, 2048, 2041, 2042, 2037, 2035, 2026, 2024, 2023, 2023, 2024, 2018, 2020, 2020, 2024, 2024, 2023, 2025, 2027, 2030, 2031, 2032, 2032, 2031, 2033, 2030, 2030, 2029, 2030, 2036, 2048, 2050, 2059, 2086, 2095, 2106, 2129, 2134, 2127, 2139, 2153, 2137, 2125, 2137, 2145, 2139, 2122, 2127, 2139, 2125, 2111, 2118, 2115, 2109, 2119, 2128, 2129, 2128, 2134, 2135, 2132, 2121, 2118, 2125, 2114, 2108, 2107, 2098, 2093, 2089, 2079, 2068, 2063, 2062, 2062, 2057, 2051, 2056, 2060, 2051, 2053, 2057, 2049, 2044, 2042, 2037, 2029, 2020, 2015, 2012, 2006, 2000, 1999, 1996, 1991, 1989, 1987, 1985, 1985, 1982, 1979, 1977, 1970, 1967, 1964, 1959, 1954, 1950, 1948, 1945, 1947, 1946, 1944, 1948, 1948, 1946, 1944, 1946, 1949, 1952, 1952, 1951, 1952, 1952, 1950, 1947, 1943, 1943, 1944, 1941, 1941, 1943, 1947, 1948, 1949, 1953, 1955, 1958, 1964, 1970, 1972, 1969, 1972, 1972, 1967, 1964, 1960, 1956, 1952, 1948, 1944, 1939, 1941, 1944, 1946, 1951, 1956, 1963, 1969, 1976, 1985, 1988, 1993, 1997, 1999, 2000, 2000, 1998, 1995};

const int16_t FX2[5500] = {2063, 2090, 1975, 1907, 1946, 2070, 2267, 2186, 2310, 1849, 1898, 2349, 1976, 1696, 1742, 2216, 2404, 2140, 2125, 2096, 1829, 1814, 1939, 1690, 1754, 2344, 2613, 3039, 2410, 1420, 1562, 1719, 1943, 1919, 2034, 2020, 2241, 2114, 2260, 2030, 2202, 2276, 1533, 2047, 2141, 2144, 2210, 2622, 1479, 1097, 1787, 2778, 3284, 1800, 771, 1525, 2634, 3360, 1974, 825, 1665, 2051, 2654, 3281, 3201, 901, 71, 1074, 3302, 3649, 1586, 811, 1505, 3568, 3023, 1556, 1182, 1627, 2121, 2320, 2274, 2231, 2031, 2083, 2162, 1874, 1672, 1908, 2901, 2721, 1204, 1056, 1950, 3036, 2670, 1199, 1146, 2258, 2893, 2485, 1931, 1804, 1979, 1037, 2135, 2695, 2016, 2065, 2154, 2566, 2444, 2151, 1795, 1735, 1699, 1864, 1870, 2066, 2022, 2199, 1988, 2670, 2954, 1324, 877, 1712, 2342, 2242, 2348, 2250, 1632, 2212, 2776, 1191, 1578, 2932, 2051, 2410, 2617, 1910, 1840, 880, 1535, 3368, 2526, 1044, 2022, 1972, 1705, 1975, 3554, 2703, 833, 1566, 2120, 2207, 1987, 1907, 2076, 2040, 2072, 2188, 2339, 1800, 1904, 2601, 1625, 1672, 2075, 2715, 2140, 1422, 1577, 1777, 2653, 3296, 2516, 2026, 1499, 1497, 2343, 1790, 1250, 1771, 2168, 2009, 2188, 2204, 2305, 2235, 1945, 1649, 2272, 3207, 2181, 894, 2160, 2777, 1353, 1880, 2598, 1838, 1977, 2166, 1766, 1556, 2245, 2542, 2166, 1829, 1494, 2371, 2542, 2214, 1702, 1382, 1685, 3067, 3081, 1375, 922, 2719, 2228, 1282, 1721, 2899, 2954, 1843, 1493, 1305, 2220, 2096, 2416, 2917, 2224, 1076, 1905, 2090, 1845, 2612, 2677, 1159, 1232, 2766, 1994, 1490, 2733, 2356, 2160, 2057, 1705, 1708, 1255, 2632, 3385, 1633, 1302, 2177, 2956, 2064, 879, 1326, 2794, 2619, 2035, 1996, 2203, 2167, 1754, 1714, 1696, 2093, 2290, 2493, 2312, 2137, 1980, 1934, 1889, 2293, 2415, 1448, 1401, 2225, 2207, 1958, 1838, 1926, 2081, 2207, 2265, 2241, 2165, 2033, 2117, 2207, 1884, 1997, 1649, 2005, 2142, 2186, 2311, 2969, 1563, 887, 2234, 2276, 2112, 2293, 1672, 1782, 2928, 2121, 1909, 2120, 1407, 1080, 2012, 3217, 2643, 1875, 2494, 1405, 1602, 2281, 1906, 2014, 1952, 1948, 2206, 2314, 2007, 2120, 2164, 2708, 1817, 1476, 1821, 1691, 2093, 1975, 2463, 2818, 1999, 1441, 2270, 2436, 1644, 1488, 1908, 2549, 2413, 1406, 2028, 2464, 2087, 1666, 2104, 2701, 1989, 1554, 1939, 2592, 1858, 1683, 2093, 2300, 2298, 2391, 1612, 1364, 2037, 2184, 2297, 2189, 2161, 2272, 2530, 1283, 1259, 2317, 2433, 1832, 2014, 2343, 2694, 1788, 1522, 2000, 1431, 2123, 2835, 2782, 1529, 1928, 2010, 1299, 2118, 2772, 2367, 1960, 1637, 1862, 2264, 2405, 1785, 1833, 1971, 2149, 2581, 1867, 1355, 1880, 2676, 2182, 2138, 1698, 1584, 2027, 2228, 2385, 2539, 1908, 1744, 1751, 2305, 2172, 1735, 2025, 1958, 2672, 2663, 1815, 965, 2018, 2630, 2060, 1732, 2236, 2428, 1830, 1682, 2434, 2335, 1407, 1252, 2364, 2963, 2008, 1880, 1608, 1804, 2499, 2579, 2143, 1429, 1826, 2800, 1747, 1604, 2465, 2165, 1788, 2368, 2133, 1588, 2085, 1858, 2338, 2401, 2084, 2078, 1837, 1409, 1822, 3019, 2725, 1296, 961, 1537, 2905, 3548, 2016, 1110, 1390, 2500, 2519, 2313, 1867, 739, 1607, 2816, 3501, 2247, 1228, 1226, 1855, 3028, 2523, 1939, 1846, 1508, 2183, 2588, 2132, 1405, 1206, 2199, 2785, 2990, 1627, 759, 1690, 2939, 3162, 2350, 828, 1006, 2899, 2719, 1698, 1779, 2717, 1838, 1621, 1846, 2049, 2635, 2420, 2012, 1301, 1091, 2728, 3275, 2091, 1292, 1527, 2804, 1941, 1815, 2628, 1873, 1026, 1813, 2413, 2330, 2713, 1985, 1701, 1899, 2694, 2062, 1426, 1985, 1704, 2451, 2713, 1343, 1747, 2768, 2279, 2205, 1491, 1897, 1785, 1994, 2840, 2301, 1387, 1629, 2076, 2292, 2622, 1816, 1743, 1462, 2021, 2783, 2401, 2057, 1666, 1353, 2361, 2636, 2505, 2142, 1013, 1691, 2681, 2929, 1523, 1647, 1918, 2276, 2298, 1477, 2108, 2793, 1791, 1022, 2272, 2885, 1957, 2046, 1950, 1388, 2391, 2245, 2382, 1790, 2176, 2254, 1520, 2039, 2297, 1520, 2231, 2315, 1811, 2098, 2370, 2223, 1300, 2111, 2806, 2144, 1939, 1236, 2290, 2675, 1634, 1969, 1367, 2510, 2627, 1983, 1389, 1996, 3033, 2258, 1271, 2088, 1888, 2381, 2501, 1113, 1158, 2732, 2654, 2162, 2110, 1371, 2318, 2927, 1713, 1704, 2300, 1254, 1722, 2943, 2232, 1313, 2196, 2647, 2140, 2113, 2264, 1386, 1467, 2091, 2327, 2745, 2187, 1584, 2396, 1586, 1201, 2215, 2656, 2625, 1648, 1478, 2045, 1745, 2377, 3219, 2241, 1083, 2220, 2554, 1842, 1465, 1511, 2365, 3215, 1994, 976, 1554, 1684, 3030, 3285, 2258, 1122, 857, 1524, 2774, 3602, 2259, 813, 1214, 2704, 2325, 2423, 2491, 1005, 1452, 2680, 2634, 1762, 2075, 2174, 2158, 2246, 1565, 1857, 1973, 1491, 1998, 3004, 2443, 2081, 2207, 1193, 1978, 3048, 1667, 806, 2237, 2391, 2065, 2703, 1682, 2094, 2208, 1754, 1963, 2130, 2392, 1955, 1552, 1731, 2235, 2422, 2263, 2036, 2022, 2232, 2601, 1372, 1696, 2471, 2316, 1370, 1909, 2384, 2589, 2094, 1559, 1969, 1488, 2307, 2152, 1817, 2081, 2412, 2172, 1826, 2448, 2013, 1809, 1912, 1510, 2655, 2290, 1790, 2400, 1961, 1666, 2466, 2214, 1439, 1852, 2452, 2991, 1575, 809, 2376, 2399, 1856, 1838, 3066, 2251, 1211, 2248, 1966, 1520, 2005, 2901, 2281, 1042, 2050, 2470, 2708, 1877, 1225, 2244, 2722, 2190, 1323, 1943, 1546, 1769, 3380, 2469, 1784, 1613, 1728, 2369, 1255, 2336, 3013, 2000, 1849, 1640, 2081, 1921, 1646, 2684, 2606, 1447, 1995, 2700, 1024, 1868, 2225, 2615, 2689, 1577, 1479, 2153, 2522, 1544, 1557, 3087, 2860, 1200, 1521, 1775, 2755, 2062, 1401, 1392, 3249, 3124, 1316, 1466, 1967, 2204, 2098, 2051, 1618, 1924, 2557, 2701, 1514, 1584, 2456, 1834, 1871, 2714, 1818, 1800, 2103, 2455, 1987, 1699, 2012, 2107, 2715, 1936, 1283, 2112, 2720, 2074, 1376, 1686, 2020, 2136, 2847, 2840, 1159, 1247, 2834, 2021, 1757, 2488, 1579, 1817, 2204, 2475, 2208, 1673, 2569, 1697, 1303, 1873, 2866, 2644, 1547, 1518, 1844, 2445, 2469, 2343, 1706, 1540, 2436, 2257, 1740, 1974, 2157, 2139, 2574, 1779, 900, 2499, 3030, 1950, 1184, 1887, 2625, 1799, 1688, 1998, 2615, 2370, 2199, 2160, 1290, 1715, 2385, 2168, 2249, 2010, 2145, 2160, 1316, 2799, 2175, 1665, 1891, 2144, 2282, 2063, 1479, 1176, 2484, 2951, 3381, 1148, 631, 1940, 3441, 2493, 1374, 1417, 1109, 3051, 3297, 1899, 1448, 1494, 2687, 2551, 976, 1504, 2415, 3247, 2002, 912, 1523, 2821, 3566, 1212, 1001, 2685, 2121, 1622, 2424, 2817, 1899, 914, 1707, 2444, 2330, 2830, 2141, 397, 1898, 3253, 1911, 2177, 2296, 2013, 2035, 2316, 1247, 637, 2636, 3694, 1977, 2348, 2304, 837, 1180, 3032, 2914, 1127, 1063, 2802, 2525, 1231, 2416, 2572, 1978, 2362, 1834, 1003, 2596, 2507, 1604, 2376, 2394, 1131, 1516, 2957, 2713, 1732, 2201, 1811, 1541, 1662, 2536, 2303, 2356, 1768, 1207, 2447, 3126, 1836, 1171, 2361, 1892, 2423, 2201, 1285, 2110, 2007, 1776, 3451, 1971, 1561, 2291, 1453, 1719, 2320, 2099, 2604, 1847, 1025, 3002, 2759, 1352, 1510, 2365, 2055, 1924, 2622, 1774, 1892, 2260, 1905, 1819, 1938, 2276, 2951, 1780, 1073, 2417, 2549, 1875, 1440, 2086, 2415, 2395, 1625, 1530, 2430, 2392, 1906, 1971, 1913, 1994, 1671, 2820, 2363, 2058, 1554, 1826, 2435, 2786, 1829, 547, 1984, 2826, 2266, 1941, 2303, 1545, 2553, 2114, 2123, 1201, 2134, 2378, 1632, 1963, 2765, 2401, 1764, 2214, 1727, 1323, 2271, 2985, 1429, 1931, 2824, 1572, 1002, 2566, 3169, 1832, 1253, 2760, 1880, 1631, 2189, 2107, 1928, 1985, 2411, 2136, 1767, 2211, 2368, 1219, 1360, 3138, 2917, 904, 1629, 2705, 1922, 2535, 2017, 1405, 1908, 2146, 2613, 2161, 2194, 1579, 1705, 2804, 2275, 1132, 2048, 2600, 1592, 1175, 2743, 3610, 1948, 408, 2177, 2205, 2398, 2654, 1256, 946, 2862, 3113, 1892, 1158, 1886, 2767, 1835, 2560, 1982, 1514, 1578, 2738, 1871, 1595, 2740, 2371, 1352, 2125, 3185, 1501, 1087, 2191, 2414, 1916, 1742, 2379, 1860, 2689, 2738, 959, 1058, 3113, 2758, 1222, 1279, 2568, 2837, 1965, 1448, 1928, 2128, 2444, 1964, 2234, 2276, 2018, 1492, 1379, 2754, 2397, 1964, 1553, 2279, 2086, 2717, 2036, 1104, 1488, 2371, 3113, 2613, 1326, 968, 2125, 3113, 1768, 1803, 2103, 2314, 1548, 1981, 2658, 2163, 2182, 1051, 1947, 2870, 2306, 1576, 1964, 1992, 2033, 2085, 2638, 2319, 1126, 1339, 2825, 2668, 1638, 1976, 1870, 2004, 2500, 1497, 1746, 2856, 2064, 1624, 2726, 1816, 1502, 2228, 2384, 1737, 1938, 1932, 2110, 2582, 2234, 1981, 1453, 2281, 1559, 2576, 2083, 1756, 1998, 2325, 2043, 1818, 2484, 2392, 2051, 1123, 2045, 2486, 1925, 2329, 2104, 1305, 2351, 2076, 2082, 1944, 2640, 1877, 1516, 2104, 1984, 2407, 2023, 2186, 1812, 1635, 2327, 3237, 1950, 748, 2029, 2610, 2094, 2116, 1756, 1210, 2346, 3289, 1632, 2010, 2003, 915, 2524, 2536, 2378, 2280, 1258, 2353, 2402, 1609, 1503, 2400, 2296, 2079, 2024, 2309, 1638, 1732, 2895, 1794, 1537, 2539, 2286, 1246, 2240, 1944, 1691, 2817, 2850, 1653, 1138, 1966, 3021, 1803, 1313, 2285, 2058, 2260, 2267, 2014, 1707, 2433, 2396, 1388, 1815, 2165, 2454, 2108, 1331, 1850, 2935, 2766, 1345, 1227, 2426, 2637, 2535, 1331, 778, 2716, 3114, 1443, 1975, 2138, 1820, 2528, 1825, 1293, 2584, 3098, 1497, 1100, 2093, 3246, 2026, 1454, 1892, 1604, 2577, 2422, 1547, 2337, 1930, 1805, 2616, 1650, 1929, 2604, 1784, 1783, 2485, 1904, 1715, 2351, 2100, 1897, 1802, 1706, 2631, 2440, 2220, 1706, 1843, 1680, 1921, 2866, 2259, 1452, 1300, 2622, 2590, 2704, 1292, 1155, 2657, 2224, 2016, 2014, 1914, 2083, 2647, 1961, 1177, 1732, 2617, 3210, 1238, 1795, 2375, 1524, 2287, 1911, 2427, 1691, 2389, 2381, 1326, 2048, 2192, 2281, 2056, 1742, 1855, 2333, 2209, 1700, 2028, 1954, 2904, 1972, 1657, 2062, 1690, 2083, 2048, 2884, 1872, 1557, 2161, 1992, 2064, 1978, 2399, 1604, 1486, 2838, 2038, 2001, 2698, 1233, 1463, 2640, 2290, 2475, 1483, 1889, 2100, 2765, 2031, 1136, 1985, 2479, 2317, 1907, 2084, 1576, 1860, 2232, 2671, 2098, 1754, 1531, 2114, 2271, 2534, 2118, 1984, 1823, 1412, 2317, 2261, 1897, 2254, 2734, 1544, 1594, 2235, 2316, 1861, 1664, 2122, 2395, 2254, 1991, 1772, 1914, 2284, 2322, 2013, 1688, 2103, 2317, 1959, 1520, 2480, 2644, 1607, 1269, 2525, 2149, 1670, 2911, 2172, 1280, 1495, 2754, 2589, 1726, 1690, 2220, 1734, 1879, 2979, 1792, 1822, 2253, 2105, 1648, 2114, 2177, 1908, 1854, 2060, 2656, 2051, 1531, 2206, 2030, 2040, 2435, 2269, 1599, 1234, 2430, 2243, 2522, 1639, 1705, 2413, 2528, 2048, 1953, 1736, 1344, 2291, 2856, 1942, 1655, 2256, 1500, 2662, 2130, 2112, 1806, 1494, 2885, 2049, 1578, 2057, 2278, 2009, 1967, 2025, 2008, 2086, 2112, 2379, 1897, 1764, 2233, 2089, 1905, 1992, 2325, 1498, 2090, 2495, 2031, 2108, 2191, 1798, 1510, 2383, 2081, 2378, 2317, 1298, 2180, 2449, 1558, 2369, 2153, 1520, 2253, 2675, 1694, 1644, 2041, 1865, 2591, 2372, 1417, 1743, 2857, 2091, 1799, 1957, 1916, 2231, 1856, 1805, 2077, 2725, 2022, 1866, 1806, 1893, 2466, 2035, 2136, 1772, 1848, 2419, 2129, 1545, 2163, 2264, 1970, 1863, 2299, 2141, 1911, 2362, 1954, 1980, 1806, 1864, 2172, 2660, 1909, 1102, 2401, 2471, 2377, 1707, 1621, 2058, 2337, 2399, 1579, 1835, 2333, 2209, 1982, 2127, 2000, 2130, 1877, 2289, 1737, 1802, 2378, 2098, 2002, 2205, 2026, 1479, 2142, 2713, 1915, 1412, 2147, 2638, 1826, 1861, 2306, 1861, 2150, 2192, 1908, 2040, 2048, 2046, 2139, 1990, 1863, 2084, 2033, 2355, 2119, 1696, 2062, 1964, 2056, 1785, 2412, 2757, 1852, 1215, 2211, 2617, 1924, 1659, 1819, 2081, 2420, 2358, 1873, 1493, 2170, 2987, 1660, 1520, 2164, 2293, 2222, 1583, 2042, 2202, 2120, 2168, 1974, 2075, 1720, 2423, 2014, 1567, 2418, 2143, 2081, 1805, 1943, 2397, 1991, 1823, 1749, 2622, 2388, 1696, 1808, 1872, 2685, 1865, 1596, 2131, 2273, 2060, 2199, 2023, 1496, 1998, 2864, 2398, 1177, 1882, 2496, 1921, 1945, 2186, 1993, 1979, 2103, 2130, 2125, 1906, 1734, 2520, 1866, 2208, 2055, 1839, 1889, 2294, 2339, 1771, 2015, 1929, 2317, 1778, 2043, 2199, 2367, 1703, 1762, 2387, 2236, 1872, 2395, 1824, 1452, 2296, 2298, 1855, 2327, 2145, 1663, 2016, 1887, 2473, 2048, 1825, 1974, 2667, 1864, 1480, 2514, 1968, 2065, 1973, 2082, 2013, 1751, 2421, 2134, 1834, 2261, 2127, 1589, 1935, 2387, 2537, 1751, 1401, 2205, 2506, 2019, 1988, 2034, 2116, 2123, 1778, 1886, 2131, 2326, 1758, 1993, 2537, 2006, 1493, 2277, 2434, 1862, 1876, 2046, 2289, 1700, 2130, 2369, 1847, 1757, 2476, 2147, 1799, 1856, 2031, 2338, 1904, 1820, 2322, 2192, 1908, 2277, 1893, 1777, 2267, 2190, 1813, 2169, 2158, 1835, 1784, 2543, 2321, 1438, 1856, 2246, 2288, 1982, 2033, 2052, 1912, 1926, 2118, 2480, 1881, 2104, 1697, 2238, 2514, 1572, 1833, 2173, 2048, 2087, 2184, 1972, 2196, 2162, 1902, 1675, 1739, 2523, 2578, 1771, 1848, 1937, 2237, 2202, 1994, 1677, 2174, 2483, 1818, 1819, 1935, 2410, 1788, 2008, 2201, 2033, 1992, 2173, 2352, 1629, 2062, 2291, 2204, 1701, 1880, 2101, 2171, 2279, 1813, 2110, 1891, 2109, 2165, 1965, 2225, 1913, 1976, 2170, 2098, 1727, 2202, 2149, 2011, 2276, 1742, 1940, 2152, 2296, 1970, 1924, 1952, 2037, 2222, 1734, 2136, 2369, 2058, 1949, 1961, 1887, 1958, 2438, 2343, 1541, 1698, 2410, 2475, 1866, 1843, 1765, 2348, 2295, 1895, 2094, 1823, 2172, 2129, 1876, 2325, 1556, 2025, 2570, 1771, 1988, 2375, 2184, 1862, 1984, 1559, 2306, 2361, 1902, 2190, 1808, 1714, 2689, 2078, 1698, 1960, 2051, 2325, 2106, 1925, 1928, 2103, 2302, 1599, 2026, 2487, 2014, 1665, 2329, 1936, 1873, 2402, 1962, 2006, 1796, 2426, 2204, 1769, 1868, 2022, 2195, 2365, 1804, 2034, 1955, 2245, 1922, 2139, 1940, 1872, 2362, 1915, 1898, 2256, 2124, 1876, 2160, 1719, 2283, 2209, 1944, 2053, 2015, 1976, 2090, 2148, 2173, 1993, 1740, 2153, 2160, 2174, 2119, 1757, 1802, 2210, 2388, 2104, 1772, 1835, 2203, 2264, 1942, 2068, 1988, 1952, 2233, 1854, 2207, 2060, 2067, 1948, 2006, 2154, 2127, 2173, 1769, 1824, 2225, 2261, 2020, 2054, 1808, 2144, 2241, 1879, 1998, 2167, 1973, 2070, 2135, 2247, 1901, 1782, 2120, 2139, 1967, 2060, 2236, 2033, 1916, 2038, 2111, 1960, 2015, 2223, 2009, 1995, 2103, 2010, 2117, 1890, 2029, 2092, 2138, 2090, 2012, 2069, 1898, 2204, 2130, 1903, 1861, 2291, 2079, 1764, 2227, 2245, 1829, 1936, 2275, 1980, 2115, 1992, 1884, 1906, 2290, 2105, 1911, 2061, 2239, 1969, 2038, 2064, 2000, 2120, 1960, 2118, 1908, 2103, 2210, 1938, 1948, 2091, 2195, 2100, 1777, 2040, 2158, 2070, 1938, 2031, 2249, 2014, 1924, 2263, 2196, 1767, 1953, 1994, 2108, 2149, 1892, 2278, 1988, 2115, 2011, 2004, 2141, 1799, 2119, 2176, 2121, 1983, 1832, 2168, 2245, 1862, 1944, 2265, 2135, 1913, 1903, 2109, 2074, 2104, 2066, 1978, 2116, 1995, 2049, 1940, 2235, 2093, 1817, 2034, 2231, 2105, 1944, 2037, 2003, 2115, 2117, 2004, 1954, 1937, 2228, 2137, 2034, 1997, 1941, 2013, 2141, 2170, 2044, 1928, 1846, 2202, 2249, 1979, 1955, 2103, 2067, 1888, 2057, 2313, 1920, 1848, 2170, 2239, 1921, 1998, 2035, 2140, 2178, 1716, 1979, 2332, 2077, 1986, 1961, 1935, 2170, 2266, 2018, 1777, 2216, 2048, 2025, 2064, 1859, 2082, 2183, 2102, 2013, 2049, 2111, 1990, 1977, 2066, 2067, 2110, 1989, 1952, 2076, 2181, 1920, 1963, 2140, 2045, 2129, 2089, 1977, 2063, 2041, 2025, 2022, 2057, 2113, 2068, 2079, 1836, 1939, 2329, 2116, 1960, 2118, 2064, 1893, 2142, 2134, 1862, 2023, 2172, 2047, 1971, 1925, 2138, 2266, 2123, 1845, 1931, 2134, 2007, 2106, 2150, 2036, 1893, 1993, 2157, 2083, 2043, 1997, 2020, 2019, 2004, 2300, 2097, 1842, 1995, 2168, 2042, 1978, 2030, 2097, 2094, 1951, 2008, 2145, 2102, 2023, 2042, 2169, 1947, 1967, 2049, 1949, 2271, 2055, 1949, 1949, 2103, 2157, 2097, 2038, 1822, 2120, 2154, 2037, 2036, 2115, 1845, 1989, 2242, 2178, 2150, 1908, 1805, 1956, 2280, 2179, 2012, 1881, 1999, 2175, 2172, 1931, 1927, 2099, 2141, 2037, 2053, 2004, 2013, 2144, 2162, 1831, 1928, 2264, 2197, 1908, 1913, 2202, 2030, 2035, 1999, 1988, 1911, 2161, 2190, 1928, 2169, 2208, 1928, 1824, 2167, 2157, 1861, 2024, 2123, 2100, 2055, 2067, 1922, 2126, 2092, 1876, 2046, 2201, 2139, 1961, 2019, 1901, 2069, 2132, 2053, 2081, 2070, 2063, 2013, 1931, 2113, 2151, 1985, 2019, 2037, 2045, 2048, 2101, 2018, 2110, 2080, 2057, 1985, 1940, 2074, 2070, 2030, 2240, 2050, 1785, 2136, 2124, 2089, 2094, 1903, 2027, 2012, 2127, 2176, 1982, 1942, 2042, 2091, 1977, 2173, 2138, 2009, 1884, 2031, 1991, 2171, 2240, 1878, 1969, 2117, 2074, 1941, 2039, 2057, 2082, 2143, 2100, 2030, 1984, 2022, 2089, 2025, 2109, 2026, 1923, 2059, 2201, 2029, 1973, 2075, 2035, 2113, 2099, 1965, 1885, 2023, 2131, 2152, 2044, 2001, 2020, 2082, 2048, 2027, 2124, 2032, 2101, 1982, 2039, 2024, 1982, 2148, 2153, 1956, 1864, 2146, 2188, 2018, 1983, 1986, 2008, 2122, 2165, 2079, 1970, 2041, 1964, 2041, 2144, 2161, 1929, 1960, 2087, 2059, 2009, 2095, 2137, 2068, 1995, 1995, 2078, 2034, 2027, 2127, 1998, 1994, 2083, 2120, 2123, 1847, 1978, 2078, 2257, 2054, 2011, 1991, 2030, 2059, 2097, 2060, 1936, 2122, 2056, 1984, 2118, 2043, 1991, 2037, 2059, 2109, 2083, 2124, 1903, 1921, 2136, 2163, 1973, 1939, 2085, 2020, 2092, 2187, 2048, 1909, 2084, 2131, 2037, 1987, 1987, 2078, 2138, 2016, 1964, 2082, 2178, 2028, 1967, 2048, 1959, 2125, 2049, 2052, 1984, 2115, 2046, 1996, 2077, 2086, 2033, 2085, 1964, 2029, 2233, 2046, 1884, 1969, 2128, 2094, 2047, 2082, 2057, 1921, 2080, 2163, 2068, 1940, 2029, 2072, 2085, 2073, 2066, 1965, 1993, 2085, 2124, 2095, 2043, 1894, 2003, 2117, 2056, 2057, 2051, 2112, 2003, 2028, 2144, 2036, 1889, 2073, 2143, 1944, 2092, 2069, 2016, 2058, 2045, 2125, 2043, 1956, 2079, 2018, 2075, 2089, 2001, 2041, 2113, 2085, 1949, 1996, 2102, 2163, 2038, 2036, 1938, 1997, 2110, 2193, 2043, 1882, 2101, 2135, 2033, 2038, 1988, 1998, 2050, 2084, 2048, 1984, 2131, 2120, 1930, 2044, 2127, 1990, 2038, 2140, 2069, 2009, 2041, 2019, 2012, 2049, 2038, 2143, 2123, 1890, 2059, 2092, 2009, 2046, 2090, 2010, 2092, 2114, 1948, 1985, 2057, 2109, 1998, 2087, 2125, 1950, 1988, 2132, 2146, 1970, 1990, 2126, 2000, 2003, 2125, 2002, 2063, 2110, 2091, 1931, 1945, 2180, 2189, 1977, 1872, 2114, 2099, 2114, 2043, 1990, 1944, 2070, 2138, 2023, 2017, 2020, 2126, 2038, 2006, 2112, 2048, 1915, 2116, 2243, 1916, 1935, 2175, 2087, 1977, 2021, 2017, 2119, 2127, 2006, 2004, 2053, 2027, 2052, 2013, 2085, 2062, 2014, 2086, 2071, 1983, 2081, 2162, 1972, 1960, 2002, 2122, 2082, 2035, 2042, 2099, 2010, 2010, 2094, 2026, 1947, 2059, 2104, 2084, 2099, 2052, 2040, 1943, 2066, 2124, 2006, 2035, 2195, 2005, 1858, 2041, 2129, 2131, 2033, 2031, 2022, 2033, 2125, 2064, 2064, 2003, 1926, 2082, 2107, 2021, 2031, 2087, 2067, 2052, 2070, 2022, 2099, 2050, 1928, 2005, 2104, 2119, 2153, 2033, 1875, 1982, 2122, 2125, 2039, 2000, 2070, 2124, 1966, 1971, 2003, 2072, 2194, 2041, 1997, 2093, 2043, 2040, 2028, 1996, 2055, 2087, 2017, 2041, 2100, 2082, 2043, 2103, 1960, 1959, 2092, 2033, 2084, 2059, 2072, 1971, 2094, 2110, 2001, 2045, 2070, 1999, 2003, 2068, 2039, 2081, 2127, 2053, 2000, 2102, 1990, 2000, 2074, 2033, 2104, 2064, 1992, 1969, 2063, 2169, 2082, 1990, 2012, 1999, 2160, 2083, 1940, 1977, 2126, 2110, 2054, 2048, 1996, 2007, 2013, 2134, 2094, 1976, 2012, 2088, 2027, 2073, 2113, 2113, 1989, 2020, 2071, 2022, 2041, 2039, 2020, 2103, 2002, 1972, 2099, 2104, 2084, 2045, 2049, 1981, 2057, 2098, 2016, 2028, 2045, 2007, 2105, 2083, 2002, 1995, 2091, 2138, 2026, 2003, 1964, 2098, 2074, 2032, 2007, 2093, 2130, 2047, 1941, 1950, 2082, 2130, 2185, 2014, 1929, 1968, 2098, 2092, 2100, 2010, 2019, 2058, 2048, 2075, 2072, 2034, 2017, 2054, 2034, 2078, 2047, 1998, 2018, 2040, 2106, 2086, 2026, 2013, 2035, 2105, 2092, 2003, 2011, 2062, 2085, 2030, 2028, 2095, 2052, 2048, 1997, 2085, 2074, 2048, 2098, 1981, 1932, 2093, 2119, 2104, 2019, 1912, 2053, 2157, 2083, 2054, 2005, 1976, 2056, 2113, 2055, 2035, 2061, 2028, 2032, 2076, 2092, 2052, 1947, 2028, 2070, 2070, 2097, 1982, 1992, 2063, 2112, 2101, 2087, 1961, 2036, 2124, 2010, 1981, 2061, 2090, 2013, 2005, 2086, 2117, 2096, 2047, 1967, 1965, 2094, 2083, 2030, 2079, 1980, 2037, 2157, 2071, 2005, 2055, 2082, 2051, 1993, 1927, 2081, 2135, 2071, 2051, 2037, 2017, 2093, 2079, 2018, 2063, 1973, 2021, 2135, 2059, 2015, 2029, 2044, 2039, 2108, 2036, 1994, 2045, 2035, 2094, 2123, 2085, 1906, 1993, 2050, 2124, 2133, 1992, 1949, 2060, 2120, 2136, 2015, 1932, 2009, 2121, 2113, 2025, 2051, 2000, 2083, 2075, 2020, 2067, 2118, 2017, 1950, 2016, 2063, 2091, 2071, 1956, 2046, 2119, 2072, 2058, 2006, 2057, 2033, 1997, 2096, 2102, 2028, 1965, 2068, 2123, 2051, 2013, 2004, 2081, 2075, 2058, 2086, 1989, 1990, 2105, 2077, 2056, 2039, 2039, 2034, 2065, 2039, 1985, 2098, 2041, 2052, 2077, 2009, 2014, 2086, 2075, 2024, 2047, 2043, 2067, 2019, 2106, 1992, 1996, 2100, 2086, 2096, 2020, 2027, 2019, 1992, 2038, 2116, 2129, 1987, 2019, 2076, 1991, 2033, 2081, 2123, 1957, 1962, 2146, 2120, 2097, 2001, 1947, 2079, 2088, 2047, 2059, 2056, 2066, 2009, 1974, 2065, 2149, 2099, 1972, 2012, 2029, 2079, 2054, 2039, 2070, 1988, 1998, 2071, 2162, 2050, 2014, 1994, 2066, 2095, 2028, 2051, 1998, 2114, 2045, 2023, 2028, 2061, 2053, 2056, 2088, 1999, 2057, 2095, 2011, 1977, 2079, 2117, 2052, 2011, 2018, 2027, 2038, 2046, 2004, 2099, 2135, 2074, 2012, 1976, 2068, 2100, 2051, 1957, 2052, 2064, 2078, 2075, 2082, 2006, 1997, 2061, 2052, 2054, 2048, 2042, 2034, 2037, 2091, 2085, 2013, 2049, 2063, 2077, 2023, 1970, 2074, 2046, 2034, 2120, 2035, 2017, 2063, 2090, 1994, 2051, 2085, 2037, 2060, 2039, 2018, 1998, 2072, 2097, 2043, 2043, 2044, 2041, 2081, 2066, 2026, 2039, 2064, 2087, 2085, 1964, 1892, 2122, 2193, 2064, 1948, 1967, 2069, 2071, 2110, 2071, 2007, 1998, 2075, 2083, 2022, 2009, 2062, 2051, 2055, 2064, 2089, 2023, 2012, 2065, 2051, 2038, 2046, 2083, 2021, 2030, 2075, 2027, 2026, 2086, 2060, 2040, 2108, 2029, 1973, 2029, 2108, 2044, 1996, 2038, 2057, 2143, 1998, 1998, 2106, 2063, 2070, 2051, 1964, 2067, 2065, 2026, 2040, 2062, 2027, 2082, 2096, 2014, 2034, 2039, 2042, 2083, 2032, 2017, 2088, 2022, 2026, 2035, 2124, 2071, 2014, 2022, 2016, 2084, 2076, 2013, 2028, 2035, 2138, 2082, 1986, 1993, 2035, 2125, 2045, 2057, 2026, 2070, 2052, 1981, 2063, 2043, 2051, 2100, 2037, 1992, 2106, 2113, 1977, 1995, 2056, 2067, 2103, 2068, 1979, 2008, 2095, 2029, 2052, 2101, 2021, 2016, 2046, 2076, 2046, 2009, 2094, 2109, 2016, 1973, 2023, 2070, 2019, 2106, 2042, 2030, 2070, 2033, 2073, 2094, 2034, 1939, 2081, 2091, 2019, 2099, 2080, 2013, 2010, 2046, 2074, 2036, 2054, 2015, 2037, 2054, 2060, 2094, 2019, 2054, 2075, 2033, 2016, 2086, 2115, 1988, 1985, 2066, 2074, 2052, 2055, 2048, 1990, 2063, 2125, 2069, 2003, 2030, 2041, 2034, 2087, 2056, 2016, 2089, 2054, 1991, 2039, 2102, 2104, 1939, 1993, 2135, 2051, 2058, 2078, 2027, 2027, 2039, 2043, 2040, 2077, 2033, 2016, 2053, 2046, 2054, 2064, 2065, 2027, 2018, 2023, 2067, 2064, 2079, 2041, 2032, 2102, 2061, 2008, 1988, 2067, 2048, 2094, 2054, 2042, 2031, 2004, 2067, 2075, 2024, 2051, 2093, 1984, 2036, 2106, 2082, 2026, 1998, 2072, 2070, 2043, 2049, 1981, 2039, 2105, 2110, 2035, 1984, 2030, 2078, 2083, 2067, 2028, 1999, 2045, 2083, 2056, 2050, 2045, 2058, 2016, 1978, 2062, 2139, 2075, 1966, 2017, 2070, 2106, 2083, 1988, 1986, 2087, 2097, 2031, 2019, 2063, 2092, 2059, 2036, 2039, 2023, 2024, 2056, 2061, 2017, 2076, 2055, 2008, 2091, 2074, 2030, 2049, 2048, 2050, 2061, 2049, 2058, 2025, 2044, 2079, 2071, 1958, 1975, 2122, 2143, 2017, 2018, 2082, 2022, 2036, 2077, 2056, 2028, 2023, 2065, 2066, 1985, 2008, 2128, 2040, 1948, 2109, 2147, 2091, 2015, 1990, 2006, 2028, 2097, 2034, 2028, 2047, 2053, 2103, 2060, 2019, 2025, 2052, 2060, 2065, 2016, 2036, 2082, 2070, 2028, 2059, 2046, 2023, 2050, 2060, 2021, 2019, 2049, 2103, 2068, 1999, 2080, 2060, 2060, 2026, 2011, 2051, 2072, 2056, 2000, 2054, 2096, 2024, 2069, 2080, 2014, 2006, 2060, 2105, 2065, 2000, 2027, 2047, 2057, 2059, 2050, 2037, 2028, 2062, 2104, 2020, 2008, 2082, 2078, 2039, 2000, 2024, 2068, 2102, 2038, 2038, 2043, 2038, 2047, 2038, 2036, 2076, 2027, 2023, 2045, 2029, 2070, 2102, 2052, 2014, 2060, 2051, 2023, 2071, 2069, 2049, 2005, 2038, 2100, 2074, 2026, 2020, 2060, 2035, 2024, 2053, 2090, 2050, 2016, 2048, 2045, 2072, 2076, 2045, 2014, 2014, 2049, 2069, 2129, 2020, 2012, 2013, 2052, 2074, 2056, 2071, 2009, 2036, 2045, 2072, 2090, 2013, 2012, 2054, 2064, 2066, 2037, 2064, 2054, 2037, 2039, 2013, 2070, 2079, 2050, 2005, 2050, 2097, 2038, 2033, 1978, 2083, 2129, 2031, 2009, 2011, 2051, 2082, 2074, 2058, 2067, 2043, 2002, 2027, 2060, 2065, 2071, 2024, 2028, 2070, 2034, 2070, 2045, 2058, 2054, 2014, 2052, 2103, 2072, 2007, 2024, 2020, 2047, 2075, 2076, 2029, 2013, 2059, 2057, 2040, 2031, 2062, 2080, 2019, 2039, 2045, 2030, 2092, 2040, 2040, 2076, 2040, 2024, 2061, 2074, 2061, 2034, 2039, 2063, 2001, 2014, 2134, 2072, 1991, 2054, 2080, 2017, 2020, 2069, 2065, 2028, 2022, 2064, 2055, 2039, 2067, 2056, 2032, 2064, 2034, 2046, 1998, 2082, 2108, 2010, 2004, 2025, 2079, 2101, 2053, 2033, 2009, 2063, 2058, 2070, 2045, 2015, 2078, 2051, 2028, 2060, 2040, 2019, 2098, 2090, 2014, 2022, 2062, 2014, 2002, 2059, 2095, 2068, 2078, 2036, 2029, 2004, 2089, 2104, 2024, 2005, 2036, 2068, 2040, 2036, 2058, 2068, 2013, 2041, 2068, 2060, 2055, 2071, 2015, 2003, 2039, 2100, 2083, 2027, 2005, 2018, 2085, 2080, 2035, 2043, 2046, 2054, 2059, 2025, 2055, 2044, 2021, 2061, 2115, 2049, 2046, 2024, 1985, 2043, 2102, 2109, 2026, 2022, 2014, 2032, 2091, 2089, 2049, 1994, 2042, 2039, 2055, 2054, 2051, 2070, 2035, 2046, 2056, 2061, 2033, 2010, 2065, 2102, 2038, 2031, 2032, 2028, 2048, 2079, 2074, 2011, 2021, 2079, 2075, 2042, 2044, 2048, 2045, 2033, 2028, 2054, 2061, 2058, 2042, 2066, 2034, 2029, 2060, 2051, 2034, 2042, 2090, 2038, 2012, 2024, 2053, 2082, 2058, 2051, 2034, 2046, 2037, 2036, 2053, 2090, 2071, 2032, 2024, 2031, 2047, 2069, 2065, 2004, 2034, 2087, 2077, 2015, 2049, 2051, 2035, 2051, 2080, 2019, 2009, 2089, 2071, 2068, 2031, 2040, 2010, 2043, 2096, 2048, 2031, 2030, 2092, 2052, 2009, 2064, 2073, 2026, 2027, 2052, 2060, 2020, 2059, 2100, 2037, 2005, 2042, 2073, 2065, 2062, 2020, 2028, 2041, 2090, 2058, 2036, 2022, 2023, 2089, 2051, 2013, 2049, 2073, 2053, 2035, 2054, 2069, 2013, 2024, 2060, 2097, 2069, 2014, 2027, 2042, 2062, 2045, 2047, 2058, 2026, 2044, 2057, 2063, 2063, 2035, 2017, 2031, 2085, 2083, 2057, 2005, 2001, 2074, 2106, 2058, 2017, 2032, 2062, 2064, 2009, 1997, 2057, 2115, 2050, 1999, 2052, 2101, 2057, 2008, 2032, 2040, 2086, 2049, 2026, 2039, 2073, 2066, 2031, 2024, 2045, 2066, 2080, 2038, 2008, 2026, 2075, 2081, 2048, 2022, 2024, 2063, 2070, 2071, 2037, 2007, 2029, 2073, 2059, 2062, 2054, 2050, 2042, 2041, 2036, 2081, 2038, 2010, 2053, 2078, 2039, 2053, 2081, 2011, 2013, 2068, 2081, 2059, 2024, 2031, 2052, 2049, 2033, 2062, 2080, 2040, 2039, 2016, 2070, 2067, 2025, 2015, 2064, 2083, 2041, 2047, 2045, 2050, 2037, 2068, 2074, 2015, 2035, 2079, 2050, 2024, 2016, 2071, 2057, 2069, 2048, 2011, 2059, 2054, 2059, 2036, 2048, 2033, 2037, 2047, 2055, 2082, 2060, 2019, 2037, 2043, 2049, 2063, 2062, 2037, 2007, 2089, 2078, 2029, 2028, 2028, 2057, 2097, 2077, 2019, 2007, 2039, 2062, 2053, 2042, 2057, 2061, 2019, 2039, 2091, 2078, 2033, 2022, 2051, 2028, 2046, 2064, 2049, 2061, 2041, 2020, 2038, 2076, 2054, 2042, 2046, 2050, 2045, 2031, 2073, 2057, 2022, 2028, 2084, 2082, 2027, 2042, 2029, 2050, 2038, 2056, 2062, 2036, 2028, 2052, 2092, 2032, 2021, 2042, 2075, 2054, 2035, 2055, 2059, 2052, 2024, 2039, 2061, 2045, 2063, 2063, 2032, 2031, 2050, 2069, 2044, 2040, 2064, 2066, 2019, 2049, 2039, 2035, 2070, 2053, 2041, 2054, 2066, 2015, 2042, 2048, 2054, 2073, 2051, 2030, 2025, 2047, 2060, 2047, 2064, 2059, 2031, 2044, 2061, 2048, 2034, 2057, 2038, 2040, 2067, 2040, 2043, 2071, 2057, 2052, 2025, 2029, 2040, 2056, 2065, 2043, 2044, 2027, 2036, 2088, 2069, 2053, 2025, 2035, 2041, 2070, 2062, 2029, 2055, 2072, 2050, 2001, 2033, 2059, 2074, 2052, 2032, 2037, 2076, 2032, 2039, 2058, 2055, 2046, 2038, 2060, 2031, 2043, 2066, 2059, 2039, 2043, 2042, 2050, 2051, 2031, 2054, 2054, 2052, 2056, 2022, 2048, 2078, 2069, 2016, 2021, 2060, 2067, 2062, 2039, 2033, 2029, 2067, 2074, 2028, 2040, 2037, 2048, 2076, 2059, 2052, 2021, 2035, 2056, 2051, 2060, 2052, 2048, 2019, 2033, 2081, 2081, 2031, 2025, 2060, 2041, 2046, 2056, 2072, 2033, 2024, 2059, 2038, 2061, 2056, 2056, 2027, 2027, 2051, 2070, 2079, 2033, 2028, 2038, 2062, 2079, 2044, 2029, 2034, 2024, 2061, 2074, 2061, 2040, 2025, 2043, 2047, 2057, 2051, 2044, 2069, 2049, 2034, 2043, 2063, 2034, 2029, 2056, 2056, 2069, 2045, 2032, 2040, 2062, 2057, 2032, 2049, 2061, 2036, 2041, 2058, 2040, 2050, 2079, 2061, 2015, 2036, 2063, 2045, 2024, 2059, 2067, 2048, 2052, 2045, 2040, 2042, 2054, 2059, 2031, 2032, 2060, 2076, 2057, 2025, 2030, 2043, 2055, 2053, 2059, 2033, 2029, 2076, 2075, 2011, 2043, 2060, 2050, 2063, 2039, 2052, 2060, 2053, 2020, 2010, 2065, 2075, 2037, 2043, 2069, 2043, 2041, 2053, 2049, 2053, 2048, 2037, 2051, 2062, 2038, 2042, 2040, 2056, 2059, 2052, 2050, 2045, 2023, 2036, 2080, 2055, 2034, 2064, 2066, 2028, 2042, 2048, 2045, 2037, 2038, 2050, 2066, 2058, 2026, 2048, 2054, 2045, 2057, 2048, 2051, 2032, 2048, 2063, 2040, 2052, 2047, 2047, 2044, 2047, 2055, 2060, 2048, 2046, 2049, 2023, 2023, 2074, 2080, 2045, 2028, 2039, 2051, 2052, 2057, 2049, 2030, 2044, 2067, 2054, 2047, 2043, 2025, 2052, 2077, 2047, 2038, 2043, 2045, 2046, 2035, 2052, 2064, 2040, 2040, 2055, 2056, 2050, 2044, 2051, 2043, 2043, 2042, 2065, 2061, 2040, 2045, 2044, 2053, 2052, 2049, 2028, 2048, 2049, 2049, 2055, 2053, 2061, 2048, 2044, 2047, 2054, 2054, 2032, 2037, 2053, 2059, 2058, 2038, 2029, 2053, 2066, 2063, 2025, 2033, 2053, 2046, 2067, 2039, 2042, 2059, 2059, 2050, 2030, 2042, 2046, 2073, 2051, 2026, 2048, 2050, 2047, 2045, 2048, 2061, 2049, 2043, 2044, 2042, 2049, 2053, 2050, 2030, 2061, 2065, 2042, 2022, 2044, 2068, 2051, 2049, 2054, 2044, 2030, 2045, 2073, 2053, 2039, 2040, 2046, 2055, 2047, 2044, 2042, 2063, 2050, 2037, 2045, 2050, 2038, 2049, 2062, 2043, 2053, 2050, 2046, 2041, 2055, 2054, 2026, 2045, 2067, 2058, 2050, 2036, 2040, 2049, 2052, 2048, 2045, 2056, 2039, 2059, 2049, 2041, 2043, 2058, 2039, 2034, 2055, 2058, 2056, 2039, 2043, 2049, 2058, 2058, 2032, 2035, 2048, 2051, 2058, 2062, 2055, 2033, 2036, 2060, 2055, 2037, 2047, 2044, 2051, 2047, 2054, 2046, 2042, 2049, 2045, 2041, 2058, 2044, 2043, 2060, 2054, 2045, 2043, 2050, 2041, 2037, 2048, 2061, 2062, 2047, 2044, 2035, 2054, 2060, 2048, 2046, 2048, 2041, 2032, 2058, 2063, 2048, 2035, 2047, 2064, 2050, 2028, 2050, 2068, 2049, 2032, 2048, 2046, 2055, 2047, 2042, 2049, 2044, 2056, 2050, 2047, 2042, 2045, 2053, 2046, 2054, 2044, 2040, 2058, 2057, 2044, 2048, 2046, 2043, 2049, 2043, 2046, 2050, 2050, 2047, 2047, 2046, 2046, 2068, 2045, 2039, 2048, 2044, 2048, 2044, 2059, 2047, 2046, 2055, 2043, 2041, 2051, 2050, 2055, 2047, 2045, 2043, 2047, 2052, 2037, 2057, 2058, 2035, 2039, 2048, 2058, 2049, 2043, 2046, 2045, 2052, 2058, 2051, 2041, 2039, 2041, 2050, 2063, 2057, 2046, 2041, 2042, 2046, 2050, 2048, 2046, 2055, 2047, 2050, 2046, 2050, 2051, 2045, 2039, 2054, 2050, 2045, 2054, 2052, 2049, 2040, 2048, 2049, 2043, 2041, 2058, 2054, 2053, 2043, 2047, 2044, 2046, 2052, 2050, 2045, 2039, 2049, 2053, 2048, 2049, 2057, 2045, 2040, 2045, 2049, 2046, 2048, 2053, 2050, 2038, 2051, 2061, 2051, 2025, 2050, 2060, 2038, 2045, 2059, 2057, 2040, 2037, 2054, 2057, 2045, 2043, 2039, 2049, 2056, 2053, 2045, 2047, 2045, 2047, 2046, 2049, 2050, 2056, 2046, 2045, 2043, 2044, 2057, 2049, 2046, 2040, 2052, 2059, 2050, 2042, 2040, 2037, 2052, 2060, 2059, 2042, 2034, 2052, 2049, 2051, 2044, 2051, 2062, 2049, 2040, 2052, 2040, 2035, 2059, 2056, 2041, 2044, 2056, 2053, 2051, 2033, 2035, 2056, 2065, 2047, 2036, 2049, 2047, 2053, 2049, 2052, 2046, 2038, 2047, 2050, 2050, 2049, 2051, 2043, 2044, 2056, 2055, 2043, 2043, 2045, 2052, 2054, 2041, 2040, 2047, 2060, 2052, 2051, 2046, 2033, 2047, 2056, 2054, 2046, 2044, 2049, 2051, 2049, 2047, 2044, 2054, 2045, 2043, 2052, 2053, 2042, 2043, 2051, 2042, 2048, 2067, 2055, 2043, 2044, 2039, 2044, 2058, 2051, 2047, 2049, 2044, 2042, 2047, 2057, 2048, 2045, 2045, 2047, 2046, 2054, 2051, 2048, 2045, 2047, 2056, 2050, 2044, 2041, 2048, 2048, 2046, 2054, 2043, 2046, 2052, 2053, 2049, 2046, 2050, 2042, 2046, 2053, 2053, 2044, 2043, 2052, 2048, 2052, 2052, 2038, 2046, 2055, 2047, 2043, 2050, 2052, 2043, 2044, 2052, 2050, 2053, 2051, 2043, 2041, 2049, 2061, 2045, 2038, 2047, 2052, 2047, 2050, 2052, 2048, 2043, 2044, 2047, 2051, 2055, 2048, 2039, 2045, 2056, 2048, 2050, 2050, 2053, 2041, 2041, 2052, 2051, 2050, 2047, 2045, 2046, 2046, 2056, 2049, 2044, 2040, 2054, 2058, 2048, 2041, 2041, 2049, 2049, 2057, 2045, 2046, 2046, 2051, 2053, 2043, 2040, 2043, 2053, 2056, 2048, 2046, 2045, 2051, 2047, 2043, 2050, 2048, 2048, 2049, 2050, 2048, 2049, 2050, 2047, 2050, 2049, 2047, 2043, 2045, 2047, 2050, 2055, 2048, 2040, 2048, 2051, 2044, 2050, 2047, 2045, 2048, 2051, 2050, 2051, 2047, 2043, 2049, 2049, 2051, 2050, 2043, 2045, 2047, 2050, 2055, 2047, 2046, 2046, 2048, 2047, 2049, 2058, 2050, 2036, 2042, 2048, 2050, 2053, 2052, 2043, 2044, 2047, 2050, 2058, 2052, 2040, 2044, 2052, 2044, 2051, 2052, 2043, 2044, 2054, 2051, 2041, 2048, 2054, 2047, 2044, 2042, 2052, 2052, 2046, 2048, 2046, 2048, 2049, 2050, 2046, 2047, 2046, 2047, 2050, 2053, 2052, 2042, 2044, 2045, 2045, 2054, 2049, 2047, 2050, 2047, 2050, 2050, 2047, 2046, 2043, 2042, 2053, 2058, 2047, 2044, 2047, 2049, 2052, 2052, 2043, 2047, 2045, 2045, 2050, 2053, 2050, 2047, 2046, 2049, 2052, 2045, 2046, 2050, 2043, 2045, 2055, 2052, 2046, 2047, 2047, 2046, 2046, 2053, 2049, 2049, 2048, 2046, 2045, 2048, 2053, 2045, 2046, 2046, 2050, 2053, 2048, 2047, 2049, 2049, 2045, 2048, 2046, 2046, 2052, 2047, 2049, 2049, 2049, 2050, 2042, 2043, 2051, 2053, 2047, 2048, 2050, 2048, 2049, 2047, 2042, 2045, 2051, 2049, 2054, 2044, 2047, 2051, 2049, 2048, 2048, 2046, 2045, 2049, 2053, 2044, 2045, 2050, 2047, 2056, 2051, 2044, 2043, 2051, 2048, 2043, 2047, 2052, 2049, 2048, 2049, 2049, 2043, 2042, 2050, 2051, 2049, 2050, 2047, 2049, 2048, 2047, 2049, 2047, 2049, 2048, 2051, 2046, 2043, 2050, 2049, 2046, 2049, 2053, 2050, 2044, 2045, 2050, 2050, 2048, 2044, 2047, 2053, 2048, 2043};
